@@ -68,8 +68,9 @@
 		(((uint64_t)_addr) >> (64 - _bits)))
 
 
-static int sock_pe_progress_buffered_rx(struct sock_rx_ctx *rx_ctx);
-
+#define SOCK_EP_MAX_PROGRESS_CNT 10
+static int sock_pe_progress_buffered_rx(struct sock_rx_ctx *rx_ctx,
+					bool shallow);
 
 static inline int sock_pe_is_data_msg(int msg_id)
 {
@@ -89,7 +90,7 @@ static inline ssize_t sock_pe_send_field(struct sock_pe_entry *pe_entry,
 					 void *field, size_t field_len,
 					 size_t start_offset)
 {
-	int ret;
+	ssize_t ret;
 	size_t offset, data_len;
 
 	if (pe_entry->done_len >= start_offset + field_len)
@@ -103,14 +104,14 @@ static inline ssize_t sock_pe_send_field(struct sock_pe_entry *pe_entry,
 		return -1;
 
 	pe_entry->done_len += ret;
-	return (ret == data_len) ? 0 : -1;
+	return ((size_t) ret == data_len) ? 0 : -1;
 }
 
 static inline ssize_t sock_pe_recv_field(struct sock_pe_entry *pe_entry,
 					 void *field, size_t field_len,
 					 size_t start_offset)
 {
-	int ret;
+	ssize_t ret;
 	size_t offset, data_len;
 
 	if (pe_entry->done_len >= start_offset + field_len)
@@ -123,7 +124,7 @@ static inline ssize_t sock_pe_recv_field(struct sock_pe_entry *pe_entry,
 		return -1;
 
 	pe_entry->done_len += ret;
-	return (ret == data_len) ? 0 : -1;
+	return ((size_t) ret == data_len) ? 0 : -1;
 }
 
 static inline void sock_pe_discard_field(struct sock_pe_entry *pe_entry)
@@ -231,7 +232,7 @@ static struct sock_pe_entry *sock_pe_acquire_entry(struct sock_pe *pe)
 
 static void sock_pe_report_send_cq_completion(struct sock_pe_entry *pe_entry)
 {
-	int ret = 0;
+	ssize_t ret = 0;
 	if (!(pe_entry->flags & SOCK_NO_COMPLETION)) {
 		if (pe_entry->comp->send_cq &&
 		    (!pe_entry->comp->send_cq_event ||
@@ -273,7 +274,7 @@ static void sock_pe_report_send_completion(struct sock_pe_entry *pe_entry)
 
 static void sock_pe_report_recv_cq_completion(struct sock_pe_entry *pe_entry)
 {
-	int ret = 0;
+	ssize_t ret = 0;
 	if (pe_entry->comp->recv_cq &&
 	    (!pe_entry->comp->recv_cq_event ||
 	     (pe_entry->flags & FI_COMPLETION)))
@@ -318,10 +319,10 @@ static void sock_pe_report_mr_completion(struct sock_domain *domain,
 	struct sock_mr *mr;
 
 	for (i = 0; i < pe_entry->msg_hdr.dest_iov_len; i++) {
-		fastlock_acquire(&domain->lock);
+		ofi_mutex_lock(&domain->lock);
 		mr = ofi_mr_map_get(&domain->mr_map,
 				    pe_entry->pe.rx.rx_iov[i].iov.key);
-		fastlock_release(&domain->lock);
+		ofi_mutex_unlock(&domain->lock);
 		if (!mr || (!mr->cq && !mr->cntr))
 			continue;
 
@@ -462,7 +463,7 @@ static void sock_pe_report_tx_rma_write_err(struct sock_pe_entry *pe_entry,
 static void sock_pe_progress_pending_ack(struct sock_pe *pe,
 					 struct sock_pe_entry *pe_entry)
 {
-	int len, data_len, i;
+	size_t len, data_len, i;
 	struct sock_conn *conn = pe_entry->conn;
 
 	if (!conn || pe_entry->rem)
@@ -549,9 +550,10 @@ static void sock_pe_send_response(struct sock_pe *pe,
 	sock_pe_progress_pending_ack(pe, pe_entry);
 }
 
-static inline int sock_pe_read_response(struct sock_pe_entry *pe_entry)
+static inline ssize_t sock_pe_read_response(struct sock_pe_entry *pe_entry)
 {
-	int ret, len, data_len;
+	ssize_t ret;
+	size_t len, data_len;
 
 	if (pe_entry->done_len >= sizeof(struct sock_msg_response))
 		return 0;
@@ -559,7 +561,7 @@ static inline int sock_pe_read_response(struct sock_pe_entry *pe_entry)
 	len = sizeof(struct sock_msg_hdr);
 	data_len = sizeof(struct sock_msg_response) - len;
 	ret = sock_pe_recv_field(pe_entry, &pe_entry->response.pe_entry_id,
-					data_len, len);
+				 data_len, len);
 	if (ret)
 		return ret;
 	pe_entry->response.pe_entry_id = ntohs(pe_entry->response.pe_entry_id);
@@ -629,7 +631,7 @@ static int sock_pe_handle_read_complete(struct sock_pe *pe,
 {
 	struct sock_pe_entry *waiting_entry;
 	struct sock_msg_response *response;
-	int len, i;
+	size_t len, i;
 
 	if (sock_pe_read_response(pe_entry))
 		return 0;
@@ -687,7 +689,7 @@ static int sock_pe_handle_atomic_complete(struct sock_pe *pe,
 	size_t datatype_sz;
 	struct sock_pe_entry *waiting_entry;
 	struct sock_msg_response *response;
-	int len, i;
+	size_t len, i;
 
 	if (sock_pe_read_response(pe_entry))
 		return 0;
@@ -710,7 +712,7 @@ static int sock_pe_handle_atomic_complete(struct sock_pe *pe,
 			    waiting_entry->pe.tx.tx_iov[i].res.ioc.count * datatype_sz,
 			    len))
 			return 0;
-		len += (waiting_entry->pe.tx.tx_iov[i].res.ioc.count * datatype_sz);
+		len += waiting_entry->pe.tx.tx_iov[i].res.ioc.count * datatype_sz;
 	}
 
 	if (waiting_entry->pe.rx.rx_op.atomic.res_iov_len)
@@ -838,7 +840,7 @@ static int sock_pe_process_rx_write(struct sock_pe *pe,
 
 	/* report error, if any */
 	if (rem) {
-		sock_pe_report_rx_error(pe_entry, rem, FI_ETRUNC);
+		sock_pe_report_rx_error(pe_entry, (int) rem, FI_ETRUNC);
 		goto out;
 	}
 
@@ -864,16 +866,15 @@ static void sock_pe_do_atomic(void *cmp, void *dst, void *src,
 {
 	char tmp_result[SOCK_EP_MAX_ATOMIC_SZ];
 
-	if (op >= OFI_SWAP_OP_START) {
-		ofi_atomic_swap_handlers[op - OFI_SWAP_OP_START][datatype](dst,
-			src, cmp, tmp_result, cnt);
+	if (ofi_atomic_isswap_op(op)) {
+		ofi_atomic_swap_handler(op, datatype, dst, src, cmp,
+					tmp_result, cnt);
                 if (cmp != NULL)
 			memcpy(cmp, tmp_result, ofi_datatype_size(datatype) * cnt);
-	} else if (fetch) {
-		ofi_atomic_readwrite_handlers[op][datatype](dst, src,
-			cmp /*results*/, cnt);
-	} else {
-		ofi_atomic_write_handlers[op][datatype](dst, src, cnt);
+	} else if (fetch && ofi_atomic_isreadwrite_op(op)) {
+		ofi_atomic_readwrite_handler(op, datatype, dst, src, cmp, cnt);
+	} else if (ofi_atomic_iswrite_op(op)) {
+		ofi_atomic_write_handler(op, datatype, dst, src, cnt);
 	}
 }
 
@@ -1035,10 +1036,10 @@ sock_pe_process_rx_tatomic(struct sock_pe *pe, struct sock_rx_ctx *rx_ctx,
 	pe_entry->tag = pe_entry->pe.rx.rx_iov[0].ioc.key;
 	pe_entry->data_len = entry_len;
 
-	fastlock_acquire(&rx_ctx->lock);
+	ofi_mutex_lock(&rx_ctx->lock);
 	rx_entry = sock_rx_new_buffered_entry(rx_ctx, entry_len);
 	if (!rx_entry) {
-		fastlock_release(&rx_ctx->lock);
+		ofi_mutex_unlock(&rx_ctx->lock);
 		return -FI_ENOMEM;
 	}
 
@@ -1059,8 +1060,8 @@ sock_pe_process_rx_tatomic(struct sock_pe *pe, struct sock_rx_ctx *rx_ctx,
 
 	pe_entry->pe.rx.rx_entry = rx_entry;
 
-	sock_pe_progress_buffered_rx(rx_ctx);
-	fastlock_release(&rx_ctx->lock);
+	sock_pe_progress_buffered_rx(rx_ctx, true);
+	ofi_mutex_unlock(&rx_ctx->lock);
 
 	pe_entry->is_complete = 1;
 
@@ -1076,7 +1077,7 @@ ssize_t sock_rx_peek_recv(struct sock_rx_ctx *rx_ctx, fi_addr_t addr,
 	struct sock_rx_entry *rx_buffered;
 	struct sock_pe_entry pe_entry;
 
-	fastlock_acquire(&rx_ctx->lock);
+	ofi_mutex_lock(&rx_ctx->lock);
 	rx_buffered = sock_rx_get_buffered_entry(rx_ctx,
 					(rx_ctx->attr.caps & FI_DIRECTED_RECV) ?
 						 addr : FI_ADDR_UNSPEC,
@@ -1106,7 +1107,7 @@ ssize_t sock_rx_peek_recv(struct sock_rx_ctx *rx_ctx, fi_addr_t addr,
 		sock_cq_report_error(rx_ctx->comp.recv_cq, &pe_entry, 0,
 				     FI_ENOMSG, -FI_ENOMSG, NULL, 0);
 	}
-	fastlock_release(&rx_ctx->lock);
+	ofi_mutex_unlock(&rx_ctx->lock);
 	return 0;
 }
 
@@ -1121,7 +1122,7 @@ ssize_t sock_rx_claim_recv(struct sock_rx_ctx *rx_ctx, void *context,
 	struct sock_pe_entry pe_entry;
 	struct sock_rx_entry *rx_buffered = NULL;
 
-	fastlock_acquire(&rx_ctx->lock);
+	ofi_mutex_lock(&rx_ctx->lock);
 	for (entry = rx_ctx->rx_buffered_list.next;
 	     entry != &rx_ctx->rx_buffered_list; entry = entry->next) {
 		rx_buffered = container_of(entry, struct sock_rx_entry, entry);
@@ -1162,36 +1163,53 @@ ssize_t sock_rx_claim_recv(struct sock_rx_ctx *rx_ctx, void *context,
 
 		if (rem) {
 			SOCK_LOG_DBG("Not enough space in posted recv buffer\n");
-			sock_pe_report_rx_error(&pe_entry, rem, FI_ETRUNC);
+			sock_pe_report_rx_error(&pe_entry, (int) rem, FI_ETRUNC);
 		} else {
 			sock_pe_report_recv_completion(&pe_entry);
 		}
 
 		dlist_remove(&rx_buffered->entry);
 		sock_rx_release_entry(rx_buffered);
+		if (rx_ctx->progress_start == entry)
+			rx_ctx->progress_start = &rx_ctx->rx_buffered_list;
 	} else {
 		ret = -FI_ENOMSG;
 	}
 
-	fastlock_release(&rx_ctx->lock);
+	ofi_mutex_unlock(&rx_ctx->lock);
 	return ret;
 }
 
-static int sock_pe_progress_buffered_rx(struct sock_rx_ctx *rx_ctx)
+/* Check buffered msg list against posted list. If shallow is true,
+ * we only check SOCK_EP_MAX_PROGRESS_CNT messages to prevent progress
+ * test taking too long */
+static int sock_pe_progress_buffered_rx(struct sock_rx_ctx *rx_ctx,
+					bool shallow)
 {
 	struct dlist_entry *entry;
 	struct sock_pe_entry pe_entry;
 	struct sock_rx_entry *rx_buffered, *rx_posted;
 	size_t i, rem = 0, offset, len, used_len, dst_offset, datatype_sz;
+	size_t max_cnt;
 	char *src, *dst;
 
 	if (dlist_empty(&rx_ctx->rx_entry_list) ||
 	    dlist_empty(&rx_ctx->rx_buffered_list))
 		return 0;
 
-	for (entry = rx_ctx->rx_buffered_list.next;
-	     entry != &rx_ctx->rx_buffered_list;) {
-
+	if (!shallow) {
+		/* ignoring rx_ctx->progress_start */
+		entry = rx_ctx->rx_buffered_list.next;
+		max_cnt = SIZE_MAX;
+	} else {
+		/* continue where last time left off */
+		entry = rx_ctx->progress_start;
+		if (entry == &rx_ctx->rx_buffered_list) {
+			entry = entry->next;
+		}
+		max_cnt = SOCK_EP_MAX_PROGRESS_CNT;
+	}
+	for (i = 0; i < max_cnt && entry != &rx_ctx->rx_buffered_list; i++) {
 		rx_buffered = container_of(entry, struct sock_rx_entry, entry);
 		entry = entry->next;
 
@@ -1235,11 +1253,10 @@ static int sock_pe_progress_buffered_rx(struct sock_rx_ctx *rx_ctx)
 			      rx_posted->iov[i].iov.addr + dst_offset;
 
 			if (datatype_sz) {
-				int cnt = len / datatype_sz;
-
 				sock_pe_do_atomic(NULL, dst, src,
 					rx_buffered->rx_op.atomic.datatype,
-					rx_buffered->rx_op.atomic.op, cnt, 0);
+					rx_buffered->rx_op.atomic.op,
+					len / datatype_sz, 0);
 			} else {
 				memcpy(dst, src, len);
 			}
@@ -1275,7 +1292,7 @@ static int sock_pe_progress_buffered_rx(struct sock_rx_ctx *rx_ctx)
 
 		if (rem) {
 			SOCK_LOG_DBG("Not enough space in posted recv buffer\n");
-			sock_pe_report_rx_error(&pe_entry, rem, FI_ETRUNC);
+			sock_pe_report_rx_error(&pe_entry, (int) rem, FI_ETRUNC);
 		} else {
 			sock_pe_report_recv_completion(&pe_entry);
 		}
@@ -1294,6 +1311,8 @@ static int sock_pe_progress_buffered_rx(struct sock_rx_ctx *rx_ctx)
 			rx_ctx->num_left++;
 		}
 	}
+	/* remember where we left off for next shallow progress */
+	rx_ctx->progress_start = entry;
 	return 0;
 }
 
@@ -1309,8 +1328,17 @@ static int sock_pe_process_rx_send(struct sock_pe *pe,
 	len = sizeof(struct sock_msg_hdr);
 
 	if (pe_entry->addr == FI_ADDR_NOTAVAIL &&
-	    pe_entry->ep_attr->ep_type == FI_EP_RDM && pe_entry->ep_attr->av)
+	    pe_entry->ep_attr->ep_type == FI_EP_RDM && pe_entry->ep_attr->av) {
+		if (pe_entry->conn->av_index == FI_ADDR_NOTAVAIL) {
+			/* this may happen when connection message comes in
+			 * before fi_av_insert. Let's try setting
+			 * conn->av_index now. */
+			pe_entry->conn->av_index =
+			    sock_av_get_addr_index(pe_entry->ep_attr->av,
+						   &(pe_entry->conn->addr));
+		}
 		pe_entry->addr = pe_entry->conn->av_index;
+	}
 
 	if (pe_entry->msg_hdr.op_type == SOCK_OP_TSEND) {
 		if (sock_pe_recv_field(pe_entry, &pe_entry->tag,
@@ -1328,8 +1356,9 @@ static int sock_pe_process_rx_send(struct sock_pe *pe,
 
 	data_len = pe_entry->msg_hdr.msg_len - len;
 	if (pe_entry->done_len == len && !pe_entry->pe.rx.rx_entry) {
-		fastlock_acquire(&rx_ctx->lock);
-		sock_pe_progress_buffered_rx(rx_ctx);
+		ofi_mutex_lock(&rx_ctx->lock);
+		rx_ctx->progress_start = &rx_ctx->rx_buffered_list;
+		sock_pe_progress_buffered_rx(rx_ctx, false);
 
 		rx_entry = sock_rx_get_entry(rx_ctx, pe_entry->addr, pe_entry->tag,
 					     pe_entry->msg_hdr.op_type == SOCK_OP_TSEND ? 1 : 0);
@@ -1341,7 +1370,7 @@ static int sock_pe_process_rx_send(struct sock_pe *pe,
 
 			rx_entry = sock_rx_new_buffered_entry(rx_ctx, data_len);
 			if (!rx_entry) {
-				fastlock_release(&rx_ctx->lock);
+				ofi_mutex_unlock(&rx_ctx->lock);
 				return -FI_ENOMEM;
 			}
 
@@ -1357,7 +1386,7 @@ static int sock_pe_process_rx_send(struct sock_pe *pe,
 			if (pe_entry->msg_hdr.op_type == SOCK_OP_TSEND)
 				rx_entry->is_tagged = 1;
 		}
-		fastlock_release(&rx_ctx->lock);
+		ofi_mutex_unlock(&rx_ctx->lock);
 		pe_entry->context = rx_entry->context;
 		pe_entry->pe.rx.rx_entry = rx_entry;
 	}
@@ -1382,7 +1411,7 @@ static int sock_pe_process_rx_send(struct sock_pe *pe,
 				     (char *) (uintptr_t) rx_entry->iov[i].iov.addr + offset,
 				     data_len);
 		if (ret <= 0)
-			return ret;
+			return (int) ret;
 
 		if (!pe_entry->buf)
 			pe_entry->buf = rx_entry->iov[i].iov.addr + offset;
@@ -1390,7 +1419,7 @@ static int sock_pe_process_rx_send(struct sock_pe *pe,
 		used = 0;
 		pe_entry->done_len += ret;
 		rx_entry->used += ret;
-		if (ret != data_len)
+		if ((size_t) ret != data_len)
 			return 0;
 	}
 
@@ -1406,7 +1435,7 @@ static int sock_pe_process_rx_send(struct sock_pe *pe,
 		pe_entry->flags |= FI_REMOTE_CQ_DATA;
 	pe_entry->flags &= ~FI_MULTI_RECV;
 
-	fastlock_acquire(&rx_ctx->lock);
+	ofi_mutex_lock(&rx_ctx->lock);
 	if (rx_entry->flags & FI_MULTI_RECV) {
 		if (sock_rx_avail_len(rx_entry) < rx_ctx->min_multi_recv) {
 			pe_entry->flags |= FI_MULTI_RECV;
@@ -1417,12 +1446,12 @@ static int sock_pe_process_rx_send(struct sock_pe *pe,
 			dlist_remove(&rx_entry->entry);
 	}
 	rx_entry->is_busy = 0;
-	fastlock_release(&rx_ctx->lock);
+	ofi_mutex_unlock(&rx_ctx->lock);
 
 	/* report error, if any */
 	if (rem) {
 		SOCK_LOG_ERROR("Not enough space in posted recv buffer\n");
-		sock_pe_report_rx_error(pe_entry, rem, FI_ETRUNC);
+		sock_pe_report_rx_error(pe_entry, (int) rem, FI_ETRUNC);
 		pe_entry->is_error = 1;
 		pe_entry->rem = pe_entry->total_len - pe_entry->done_len;
 		goto out;
@@ -1440,12 +1469,12 @@ out:
 	if (!rx_entry->is_buffered &&
 	    (!(rx_entry->flags & FI_MULTI_RECV) ||
 	     (pe_entry->flags & FI_MULTI_RECV))) {
-		fastlock_acquire(&rx_ctx->lock);
+		ofi_mutex_lock(&rx_ctx->lock);
 		sock_rx_release_entry(rx_entry);
 		rx_ctx->num_left++;
-		fastlock_release(&rx_ctx->lock);
+		ofi_mutex_unlock(&rx_ctx->lock);
 	}
-	return ret;
+	return (int) ret;
 }
 
 static int sock_pe_process_rx_conn_msg(struct sock_pe *pe,
@@ -1478,13 +1507,13 @@ static int sock_pe_process_rx_conn_msg(struct sock_pe *pe,
 
 	index = (ep_attr->ep_type == FI_EP_MSG) ? 0 : sock_av_get_addr_index(ep_attr->av, addr);
 	if (index != -1) {
-		fastlock_acquire(&map->lock);
+		ofi_mutex_lock(&map->lock);
 		conn = sock_ep_lookup_conn(ep_attr, index, addr);
 		if (conn == NULL || conn == SOCK_CM_CONN_IN_PROGRESS) {
-			if (ofi_idm_set(&ep_attr->av_idm, index, pe_entry->conn) < 0)
+			if (ofi_idm_set(&ep_attr->av_idm, (int) index, pe_entry->conn) < 0)
 				SOCK_LOG_ERROR("ofi_idm_set failed\n");
 		}
-		fastlock_release(&map->lock);
+		ofi_mutex_unlock(&map->lock);
 	}
 	pe_entry->conn->av_index = (ep_attr->ep_type == FI_EP_MSG || index == -1) ?
 		FI_ADDR_NOTAVAIL : index;
@@ -1630,7 +1659,7 @@ static int sock_pe_progress_tx_atomic(struct sock_pe *pe,
 				      struct sock_pe_entry *pe_entry,
 				      struct sock_conn *conn)
 {
-	int datatype_sz;
+	size_t datatype_sz;
 	union sock_iov iov[SOCK_EP_MAX_IOV_LIMIT];
 	ssize_t len, i, entry_len;
 
@@ -1927,13 +1956,12 @@ static int sock_pe_progress_tx_entry(struct sock_pe *pe,
 		goto out;
 
 	if (sock_comm_is_disconnected(pe_entry)) {
-		SOCK_LOG_DBG("conn disconnected: removing fd from pollset\n");
-		if (pe_entry->ep_attr->cmap.used > 0 &&
-		     pe_entry->conn->sock_fd != -1) {
-			fastlock_acquire(&pe_entry->ep_attr->cmap.lock);
-			sock_ep_remove_conn(pe_entry->ep_attr, pe_entry->conn);
-			fastlock_release(&pe_entry->ep_attr->cmap.lock);
-		}
+		ofi_straddr_log(&sock_prov, FI_LOG_WARN, FI_LOG_EP_DATA,
+				"Peer disconnected: removing fd from pollset",
+				&pe_entry->conn->addr.sa);
+		ofi_mutex_lock(&pe_entry->ep_attr->cmap.lock);
+		sock_ep_remove_conn(pe_entry->ep_attr, pe_entry->conn);
+		ofi_mutex_unlock(&pe_entry->ep_attr->cmap.lock);
 
 		sock_pe_report_tx_error(pe_entry, 0, FI_EIO);
 		pe_entry->is_complete = 1;
@@ -2006,13 +2034,12 @@ static int sock_pe_progress_rx_pe_entry(struct sock_pe *pe,
 	int ret;
 
 	if (sock_comm_is_disconnected(pe_entry)) {
-		SOCK_LOG_DBG("conn disconnected: removing fd from pollset\n");
-		if (pe_entry->ep_attr->cmap.used > 0 &&
-		     pe_entry->conn->sock_fd != -1) {
-			fastlock_acquire(&pe_entry->ep_attr->cmap.lock);
-			sock_ep_remove_conn(pe_entry->ep_attr, pe_entry->conn);
-			fastlock_release(&pe_entry->ep_attr->cmap.lock);
-		}
+		ofi_straddr_log(&sock_prov, FI_LOG_WARN, FI_LOG_EP_DATA,
+				"Peer disconnected: removing fd from pollset",
+				&pe_entry->conn->addr.sa);
+		ofi_mutex_lock(&pe_entry->ep_attr->cmap.lock);
+		sock_ep_remove_conn(pe_entry->ep_attr, pe_entry->conn);
+		ofi_mutex_unlock(&pe_entry->ep_attr->cmap.lock);
 
 		if (pe_entry->pe.rx.header_read)
 			sock_pe_report_rx_error(pe_entry, 0, FI_EIO);
@@ -2091,7 +2118,8 @@ static void sock_pe_new_rx_entry(struct sock_pe *pe, struct sock_rx_ctx *rx_ctx,
 
 static int sock_pe_new_tx_entry(struct sock_pe *pe, struct sock_tx_ctx *tx_ctx)
 {
-	int i, datatype_sz;
+	int i;
+	size_t datatype_sz;
 	struct sock_msg_hdr *msg_hdr;
 	struct sock_pe_entry *pe_entry;
 	struct sock_ep_attr *ep_attr;
@@ -2114,7 +2142,7 @@ static int sock_pe_new_tx_entry(struct sock_pe *pe, struct sock_tx_ctx *tx_ctx)
 	msg_hdr = &pe_entry->msg_hdr;
 	msg_hdr->msg_len = sizeof(*msg_hdr);
 
-	msg_hdr->pe_entry_id = PE_INDEX(pe, pe_entry);
+	msg_hdr->pe_entry_id = (uint16_t) PE_INDEX(pe, pe_entry);
 	SOCK_LOG_DBG("New TX on PE entry %p (%d)\n",
 		      pe_entry, msg_hdr->pe_entry_id);
 
@@ -2246,7 +2274,7 @@ static int sock_pe_new_tx_entry(struct sock_pe *pe, struct sock_tx_ctx *tx_ctx)
 	msg_hdr->version = SOCK_WIRE_PROTO_VERSION;
 
 	if (tx_ctx->av)
-		msg_hdr->rx_id = (uint16_t) SOCK_GET_RX_ID(pe_entry->addr,
+		msg_hdr->rx_id = (uint8_t) SOCK_GET_RX_ID(pe_entry->addr,
 							   tx_ctx->av->rx_ctx_bits);
 	else
 		msg_hdr->rx_id = 0;
@@ -2268,30 +2296,30 @@ void sock_pe_signal(struct sock_pe *pe)
 	if (pe->domain->progress_mode != FI_PROGRESS_AUTO)
 		return;
 
-	fastlock_acquire(&pe->signal_lock);
+	ofi_mutex_lock(&pe->signal_lock);
 	if (pe->wcnt == pe->rcnt) {
 		if (ofi_write_socket(pe->signal_fds[SOCK_SIGNAL_WR_FD], &c, 1) != 1)
 			SOCK_LOG_ERROR("Failed to signal\n");
 		else
 			pe->wcnt++;
 	}
-	fastlock_release(&pe->signal_lock);
+	ofi_mutex_unlock(&pe->signal_lock);
 }
 
 void sock_pe_poll_add(struct sock_pe *pe, int fd)
 {
-        fastlock_acquire(&pe->signal_lock);
+        ofi_mutex_lock(&pe->signal_lock);
         if (ofi_epoll_add(pe->epoll_set, fd, OFI_EPOLL_IN, NULL))
 			SOCK_LOG_ERROR("failed to add to epoll set: %d\n", fd);
-        fastlock_release(&pe->signal_lock);
+        ofi_mutex_unlock(&pe->signal_lock);
 }
 
 void sock_pe_poll_del(struct sock_pe *pe, int fd)
 {
-        fastlock_acquire(&pe->signal_lock);
+        ofi_mutex_lock(&pe->signal_lock);
         if (ofi_epoll_del(pe->epoll_set, fd))
 			SOCK_LOG_DBG("failed to del from epoll set: %d\n", fd);
-        fastlock_release(&pe->signal_lock);
+        ofi_mutex_unlock(&pe->signal_lock);
 }
 
 void sock_pe_add_tx_ctx(struct sock_pe *pe, struct sock_tx_ctx *ctx)
@@ -2358,29 +2386,29 @@ static int sock_pe_progress_rx_ep(struct sock_pe *pe,
 	if (!map->used)
 		return 0;
 
-	if (map->epoll_ctxs_sz < map->used) {
-		uint64_t new_size = map->used * 2;
-		void *ctxs;
+	if (map->epoll_size < map->used) {
+		int new_size = map->used * 2;
+		struct ofi_epollfds_event *events;
 
-		ctxs = realloc(map->epoll_ctxs,
-			       sizeof(*map->epoll_ctxs) * new_size);
-		if (ctxs) {
-			map->epoll_ctxs = ctxs;
-			map->epoll_ctxs_sz = new_size;
+		events = realloc(map->epoll_events,
+				 sizeof(*map->epoll_events) * new_size);
+		if (events) {
+			map->epoll_events = events;
+			map->epoll_size = new_size;
 		}
 	}
 
-	num_fds = ofi_epoll_wait(map->epoll_set, map->epoll_ctxs,
-	                        MIN(map->used, map->epoll_ctxs_sz), 0);
+	num_fds = ofi_epoll_wait(map->epoll_set, map->epoll_events,
+	                        MIN(map->used, map->epoll_size), 0);
 	if (num_fds < 0 || num_fds == 0) {
 		if (num_fds < 0)
 			SOCK_LOG_ERROR("epoll failed: %d\n", num_fds);
 		return num_fds;
 	}
 
-	fastlock_acquire(&map->lock);
+	ofi_mutex_lock(&map->lock);
 	for (i = 0; i < num_fds; i++) {
-		conn = map->epoll_ctxs[i];
+		conn = map->epoll_events[i].data.ptr;
 		if (!conn)
 			SOCK_LOG_ERROR("ofi_idm_lookup failed\n");
 
@@ -2389,7 +2417,7 @@ static int sock_pe_progress_rx_ep(struct sock_pe *pe,
 
 		sock_pe_new_rx_entry(pe, rx_ctx, ep_attr, conn);
 	}
-	fastlock_release(&map->lock);
+	ofi_mutex_unlock(&map->lock);
 
 	return 0;
 }
@@ -2401,11 +2429,11 @@ int sock_pe_progress_rx_ctx(struct sock_pe *pe, struct sock_rx_ctx *rx_ctx)
 	struct dlist_entry *entry;
 	struct sock_pe_entry *pe_entry;
 
-	fastlock_acquire(&pe->lock);
+	ofi_mutex_lock(&pe->lock);
 
-	fastlock_acquire(&rx_ctx->lock);
-	sock_pe_progress_buffered_rx(rx_ctx);
-	fastlock_release(&rx_ctx->lock);
+	ofi_mutex_lock(&rx_ctx->lock);
+	sock_pe_progress_buffered_rx(rx_ctx, true);
+	ofi_mutex_unlock(&rx_ctx->lock);
 
 	/* check for incoming data */
 	if (rx_ctx->ctx.fid.fclass == FI_CLASS_SRX_CTX) {
@@ -2435,7 +2463,7 @@ int sock_pe_progress_rx_ctx(struct sock_pe *pe, struct sock_rx_ctx *rx_ctx)
 out:
 	if (ret < 0)
 		SOCK_LOG_ERROR("failed to progress RX ctx\n");
-	fastlock_release(&pe->lock);
+	ofi_mutex_unlock(&pe->lock);
 	return ret;
 }
 
@@ -2505,7 +2533,7 @@ int sock_pe_progress_tx_ctx(struct sock_pe *pe, struct sock_tx_ctx *tx_ctx)
 	struct dlist_entry *entry;
 	struct sock_pe_entry *pe_entry;
 
-	fastlock_acquire(&pe->lock);
+	ofi_mutex_lock(&pe->lock);
 
 	/* progress tx_ctx in PE table */
 	for (entry = tx_ctx->pe_entry_list.next;
@@ -2520,11 +2548,11 @@ int sock_pe_progress_tx_ctx(struct sock_pe *pe, struct sock_tx_ctx *tx_ctx)
 		}
 	}
 
-	fastlock_acquire(&tx_ctx->rb_lock);
+	ofi_mutex_lock(&tx_ctx->rb_lock);
 	if (!ofi_rbempty(&tx_ctx->rb) && !dlist_empty(&pe->free_list)) {
 		ret = sock_pe_new_tx_entry(pe, tx_ctx);
 	}
-	fastlock_release(&tx_ctx->rb_lock);
+	ofi_mutex_unlock(&tx_ctx->rb_lock);
 	if (ret < 0)
 		goto out;
 
@@ -2532,7 +2560,7 @@ int sock_pe_progress_tx_ctx(struct sock_pe *pe, struct sock_tx_ctx *tx_ctx)
 out:
 	if (ret < 0)
 		SOCK_LOG_ERROR("failed to progress TX ctx\n");
-	fastlock_release(&pe->lock);
+	ofi_mutex_unlock(&pe->lock);
 	return ret;
 }
 
@@ -2579,20 +2607,20 @@ static void sock_pe_wait(struct sock_pe *pe)
 {
 	char tmp;
 	int ret;
-	void *ep_contexts[1];
+	struct ofi_epollfds_event event;
 
-	ret = ofi_epoll_wait(pe->epoll_set, ep_contexts, 1, -1);
+	ret = ofi_epoll_wait(pe->epoll_set, &event, 1, -1);
 	if (ret < 0)
 		SOCK_LOG_ERROR("poll failed : %s\n", strerror(ofi_sockerr()));
 
-	fastlock_acquire(&pe->signal_lock);
+	ofi_mutex_lock(&pe->signal_lock);
 	if (pe->rcnt != pe->wcnt) {
 		if (ofi_read_socket(pe->signal_fds[SOCK_SIGNAL_RD_FD], &tmp, 1) == 1)
 			pe->rcnt++;
 		else
 			SOCK_LOG_ERROR("Invalid signal\n");
 	}
-	fastlock_release(&pe->signal_lock);
+	ofi_mutex_unlock(&pe->signal_lock);
 	pe->waittime = ofi_gettime_ms();
 }
 
@@ -2696,12 +2724,12 @@ struct sock_pe *sock_pe_init(struct sock_domain *domain)
 	sock_pe_init_table(pe);
 	dlist_init(&pe->tx_list);
 	dlist_init(&pe->rx_list);
-	fastlock_init(&pe->lock);
-	fastlock_init(&pe->signal_lock);
+	ofi_mutex_init(&pe->lock);
+	ofi_mutex_init(&pe->signal_lock);
 	pthread_mutex_init(&pe->list_lock, NULL);
 	pe->domain = domain;
 
-	
+
 	ret = ofi_bufpool_create(&pe->pe_rx_pool,
 				 sizeof(struct sock_pe_entry), 16, 0, 1024, 0);
 	if (ret) {
@@ -2751,7 +2779,7 @@ err3:
 err2:
 	ofi_bufpool_destroy(pe->pe_rx_pool);
 err1:
-	fastlock_destroy(&pe->lock);
+	ofi_mutex_destroy(&pe->lock);
 	free(pe);
 	return NULL;
 }
@@ -2789,8 +2817,8 @@ void sock_pe_finalize(struct sock_pe *pe)
 	}
 
 	sock_pe_free_util_pool(pe);
-	fastlock_destroy(&pe->lock);
-	fastlock_destroy(&pe->signal_lock);
+	ofi_mutex_destroy(&pe->lock);
+	ofi_mutex_destroy(&pe->signal_lock);
 	pthread_mutex_destroy(&pe->list_lock);
 	ofi_epoll_close(pe->epoll_set);
 	free(pe);
