@@ -1,5 +1,5 @@
 /**
-* Copyright (C) Mellanox Technologies Ltd. 2001-2019.  ALL RIGHTS RESERVED.
+* Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2001-2019. ALL RIGHTS RESERVED.
 *
 * See file LICENSE for terms.
 */
@@ -8,6 +8,8 @@
 
 #include <uct/uct_test.h>
 
+#include <vector>
+
 extern "C" {
 #include <ucs/time/time.h>
 #include <ucs/datastruct/queue.h>
@@ -111,7 +113,7 @@ public:
     }
 
     void validate_connect(uct_ud_ep_t *ep, unsigned value,
-                          double timeout_sec=TEST_UD_TIMEOUT_IN_SEC) {
+                          double timeout_sec=TEST_UD_LINGER_TIMEOUT_IN_SEC) {
         ucs_time_t timeout = ucs_get_time() + ucs_time_from_sec(timeout_sec);
         while ((ep->dest_ep_id != value) && (ucs_get_time() < timeout)) {
             progress();
@@ -130,7 +132,7 @@ public:
     }
 
     void validate_recv(uct_ud_ep_t *ep, unsigned value,
-                       double timeout_sec=TEST_UD_TIMEOUT_IN_SEC) {
+                       double timeout_sec=TEST_UD_LINGER_TIMEOUT_IN_SEC) {
         ucs_time_t timeout = ucs_get_time() + ucs_time_from_sec(timeout_sec);
         while ((ucs_frag_list_sn(&ep->rx.ooo_pkts) < value - no_creq_cnt(ep)) &&
                (ucs_get_time() < timeout)) {
@@ -139,18 +141,18 @@ public:
         EXPECT_EQ(value - no_creq_cnt(ep), ucs_frag_list_sn(&ep->rx.ooo_pkts));
     }
 
-    void validate_flush() {
+    void validate_flush(unsigned base_psn = 1u) {
         /* 1 packets transmitted, 1 packets received */
-        EXPECT_EQ(2, ep(m_e1)->tx.psn);
-        EXPECT_EQ(1, ucs_frag_list_sn(&ep(m_e2)->rx.ooo_pkts));
+        EXPECT_EQ(base_psn + 1, ep(m_e1)->tx.psn);
+        EXPECT_EQ(base_psn, ucs_frag_list_sn(&ep(m_e2)->rx.ooo_pkts));
 
         /* no data transmitted back */
-        EXPECT_EQ(1, ep(m_e2)->tx.psn);
+        EXPECT_EQ(base_psn, ep(m_e2)->tx.psn);
 
         /* one packet was acked */
         EXPECT_EQ(0U, ucs_queue_length(&ep(m_e1)->tx.window));
-        EXPECT_EQ(1, ep(m_e1)->tx.acked_psn);
-        EXPECT_EQ(1, ep(m_e2)->rx.acked_psn);
+        EXPECT_EQ(base_psn, ep(m_e1)->tx.acked_psn);
+        EXPECT_EQ(base_psn, ep(m_e2)->rx.acked_psn);
     }
 
     void check_connection() {
@@ -236,7 +238,7 @@ UCS_TEST_SKIP_COND_P(test_ud, duplex_tx,
     EXPECT_TRUE(ucs_queue_is_empty(&ep(m_e1)->tx.window));
 }
 
-/* send full window, rcv ack after progreess, send some more */
+/* send full window, rcv ack after progress, send some more */
 UCS_TEST_SKIP_COND_P(test_ud, tx_window1,
                      !check_caps(UCT_IFACE_FLAG_AM_SHORT)) {
     unsigned i, N = 13;
@@ -251,7 +253,8 @@ UCS_TEST_SKIP_COND_P(test_ud, tx_window1,
     EXPECT_EQ(UCS_ERR_NO_RESOURCE, tx(m_e1));
 
     /* wait for ack */
-    ucs_time_t timeout = ucs_get_time() + ucs_time_from_sec(TEST_UD_TIMEOUT_IN_SEC);
+    ucs_time_t timeout =
+            ucs_get_time() + ucs_time_from_sec(TEST_UD_LINGER_TIMEOUT_IN_SEC);
     while ((ucs_get_time() < timeout) &&
             uct_ud_ep_no_window(ep(m_e1))) {
         short_progress_loop();
@@ -385,7 +388,7 @@ UCS_TEST_SKIP_COND_P(test_ud, crep_drop1,
 }
 
 /* check that creq is not left on tx window if
- * both sides connect simultaniously.
+ * both sides connect simultaneously.
  */
 UCS_TEST_SKIP_COND_P(test_ud, crep_drop2,
                      !check_caps(UCT_IFACE_FLAG_AM_SHORT)) {
@@ -411,7 +414,8 @@ UCS_TEST_SKIP_COND_P(test_ud, crep_drop2,
 
     /* Wait for TX win to be empty (which means that all
      * CONN packets are handled) */
-    ucs_time_t timeout = ucs_get_time() + ucs_time_from_sec(TEST_UD_TIMEOUT_IN_SEC);
+    ucs_time_t timeout =
+            ucs_get_time() + ucs_time_from_sec(TEST_UD_LINGER_TIMEOUT_IN_SEC);
     while (ucs_get_time() < timeout) {
         if(ucs_queue_is_empty(&ep(m_e1)->tx.window) &&
            ucs_queue_is_empty(&ep(m_e2)->tx.window)) {
@@ -532,10 +536,11 @@ UCS_TEST_SKIP_COND_P(test_ud, ca_ai,
     /* check initial window */
     disable_async(m_e1);
     disable_async(m_e2);
-    /* only test up to 'small' window when on valgrind
-     * valgrind drops rx packets when window is too big and resends are disabled in this test
+    /* only test up to 'small' window when on valgrind or EFA interface, as they
+     * drop rx packets when window is too big and resends are disabled in this test
      */
-    max_window = RUNNING_ON_VALGRIND ? 128 : UCT_UD_CA_MAX_WINDOW;
+    max_window = (ucs::is_aws() || RUNNING_ON_VALGRIND) ? 128 :
+                                                          UCT_UD_CA_MAX_WINDOW;
     connect();
     EXPECT_EQ(UCT_UD_CA_MIN_WINDOW, ep(m_e1)->ca.cwnd);
     EXPECT_EQ(UCT_UD_CA_MIN_WINDOW, ep(m_e2)->ca.cwnd);
@@ -545,7 +550,7 @@ UCS_TEST_SKIP_COND_P(test_ud, ca_ai,
     prev_cwnd = ep(m_e1)->ca.cwnd;
     rx_ack_count = 0;
 
-    /* window increase upto max window should
+    /* window increase up to max window should
      * happen when we receive acks */
     while (ep(m_e1)->ca.cwnd < max_window) {
        status = tx(m_e1);
@@ -714,21 +719,21 @@ UCS_TEST_SKIP_COND_P(test_ud, connect_iface_seq,
     m_e1->connect_to_iface(0, *m_e2);
     validate_connect(ep(m_e1), 0U);
     EXPECT_EQ(2, ep(m_e1)->tx.psn);
-    /* one becase of crep */
+    /* one because of crep */
     EXPECT_EQ(1, ucs_frag_list_sn(&ep(m_e1)->rx.ooo_pkts));
 
     /* now side two connects. existing ep will be reused */
     m_e2->connect_to_iface(0, *m_e1);
     validate_connect(ep(m_e2), 0U);
     EXPECT_EQ(2, ep(m_e2)->tx.psn);
-    /* one becase creq sets initial psn */
+    /* one because creq sets initial psn */
     EXPECT_EQ(1, ucs_frag_list_sn(&ep(m_e2)->rx.ooo_pkts));
 
     check_connection();
 }
 
 UCS_TEST_P(test_ud, connect_iface_sim) {
-    /* simultanious connect from both sides */
+    /* simultaneous connect from both sides */
     connect_to_iface();
 
     validate_connect(ep(m_e1), 0U);
@@ -738,7 +743,7 @@ UCS_TEST_P(test_ud, connect_iface_sim) {
 }
 
 UCS_TEST_P(test_ud, connect_iface_sim2v2) {
-    /* simultanious connect from both sides */
+    /* simultaneous connect from both sides */
     connect_to_iface(0);
     connect_to_iface(1);
 
@@ -755,10 +760,9 @@ UCS_TEST_P(test_ud, connect_iface_sim2v2) {
  * - flush() will also progress pending CREQs
  */
 UCS_TEST_P(test_ud, connect_iface_2k) {
-
     unsigned i;
-    unsigned cids[2000];
     unsigned count = 2000 / ucs::test_time_multiplier();
+    std::vector<unsigned> cids(count, 0);
 
     /* create 2k connections */
     for (i = 0; i < count; i++) {
@@ -834,7 +838,8 @@ UCS_TEST_SKIP_COND_P(test_ud, ep_destroy_flush,
 
 UCS_TEST_SKIP_COND_P(test_ud, ep_destroy_passive,
                      !check_caps(UCT_IFACE_FLAG_AM_SHORT)) {
-    connect();
+    connect_to_iface(0);
+    short_progress_loop(TEST_UD_PROGRESS_TIMEOUT);
 
     /* m_e2::ep[0] has to be revoked at the end of the testing */
     uct_ep_destroy(m_e2->ep(0));
@@ -843,7 +848,7 @@ UCS_TEST_SKIP_COND_P(test_ud, ep_destroy_passive,
     EXPECT_UCS_OK(tx(m_e1));
     EXPECT_UCS_OK(ep_flush_b(m_e1));
 
-    validate_flush();
+    validate_flush(3);
 
     /* revoke m_e2::ep[0] as it was destroyed manually */
     m_e2->revoke_ep(0);
@@ -915,7 +920,92 @@ UCS_TEST_SKIP_COND_P(test_ud, ctls_loss,
 
 UCT_INSTANTIATE_UD_TEST_CASE(test_ud)
 
-#ifdef HAVE_MLX5_HW
+
+class test_ud_peer_failure : public ud_base_test {
+public:
+    test_ud_peer_failure()
+    {
+        m_err_count = 0;
+    }
+
+    void init()
+    {
+        set_config("UD_TIMEOUT=3s");
+        ud_base_test::init();
+        connect();
+    }
+
+    void kill_receiver()
+    {
+        m_entities.remove(m_e2);
+    }
+
+    static ucs_status_t err_cb(void *arg, uct_ep_h ep, ucs_status_t status)
+    {
+        m_err_count++;
+        return UCS_OK;
+    }
+
+    uct_error_handler_t get_err_handler() const
+    {
+        return err_cb;
+    }
+
+protected:
+    static size_t m_err_count;
+};
+
+size_t test_ud_peer_failure::m_err_count = 0;
+
+UCS_TEST_SKIP_COND_P(test_ud_peer_failure, send_am_after_kill,
+                     !check_caps(UCT_IFACE_FLAG_AM_SHORT |
+                                 UCT_IFACE_FLAG_EP_CHECK),
+                     // Increase UD timer tick to not get NO_RESOURCE from AM
+                     "UD_TIMER_TICK?=2s") {
+    ucs_status_t status;
+
+    flush();
+
+    status = uct_ep_check(m_e1->ep(0), 0, NULL);
+    ASSERT_UCS_OK(status);
+    flush();
+    // Allow keepalive request to complete
+    short_progress_loop();
+
+    // Set TX window to big enough value to not get UCS_ERR_NO_RESOURCE from
+    // AM SHORT operations
+    set_tx_win(m_e1, 1024);
+
+    // We are still alive
+    EXPECT_EQ(0, m_err_count);
+
+    kill_receiver();
+
+    status = uct_ep_check(m_e1->ep(0), 0, NULL);
+    ASSERT_UCS_OK(status);
+
+    // Post AM operation to check that an error could be detected by EP_CHECK
+    // when an endpoint has an in-flight AM operation
+    const ucs_time_t loop_end_limit = ucs::get_deadline(10.0);
+    while ((m_err_count == 0) && (ucs_get_time() < loop_end_limit)) {
+        const uint64_t send_data = 0;
+        status = uct_ep_am_short(m_e1->ep(0), 0, 0, &send_data,
+                                 sizeof(send_data));
+        if (m_err_count == 0) {
+            ASSERT_UCS_OK(status);
+            // Have a 2-second deadline to wait for peer failure and ensure
+            // scheduling new AM operations while no error was detected
+            wait_for_flag(&m_err_count, 2.0);
+        }
+    }
+
+    EXPECT_EQ(1, m_err_count);
+}
+
+UCT_INSTANTIATE_UD_TEST_CASE(test_ud_peer_failure)
+
+
+#ifdef HAVE_MLX5_DV
 extern "C" {
 #include <uct/ib/mlx5/ib_mlx5.h>
 }
@@ -925,7 +1015,7 @@ class test_ud_iface_attrs : public test_uct_iface_attrs {
 public:
     attr_map_t get_num_iov() {
         attr_map_t iov_map;
-#ifdef HAVE_MLX5_HW
+#ifdef HAVE_MLX5_DV
         if (has_transport("ud_mlx5")) {
             // For am zcopy just small constant number of iovs is allowed
             // (to preserve some inline space for AM zcopy header)
