@@ -1,6 +1,6 @@
 
 /**
-* Copyright (C) Mellanox Technologies Ltd. 2001-2016.  ALL RIGHTS RESERVED.
+* Copyright (c) NVIDIA CORPORATION & AFFILIATES, 2001-2016. ALL RIGHTS RESERVED.
 * Copyright (C) UT-Battelle, LLC. 2016. ALL RIGHTS RESERVED.
 * Copyright (C) ARM Ltd. 2016.All rights reserved.
 * See file LICENSE for terms.
@@ -8,14 +8,6 @@
 
 #include "test_peer_failure.h"
 
-#if HAVE_CUDA
-extern "C" {
-#include <uct/cuda/cuda_ipc/cuda_ipc_ep.h>
-}
-#endif
-
-
-size_t test_uct_peer_failure::m_req_purge_count       = 0ul;
 const uint64_t test_uct_peer_failure::m_required_caps = UCT_IFACE_FLAG_AM_SHORT  |
                                                         UCT_IFACE_FLAG_PENDING   |
                                                         UCT_IFACE_FLAG_CB_SYNC   |
@@ -23,7 +15,8 @@ const uint64_t test_uct_peer_failure::m_required_caps = UCT_IFACE_FLAG_AM_SHORT
 
 test_uct_peer_failure::test_uct_peer_failure() :
     m_sender(NULL), m_nreceivers(2), m_tx_window(100),
-    m_err_count(0), m_am_count(0)
+    m_err_count(0), m_am_count(0), m_req_purge_count(0),
+    m_req_pending_count(0)
 {
 }
 
@@ -33,7 +26,7 @@ void test_uct_peer_failure::init()
 
     reduce_tl_send_queues();
 
-    /* To reduce test execution time decrease retransmition timeouts
+    /* To reduce test execution time decrease retransmission timeouts
      * where it is relevant */
     set_config("RC_TIMEOUT?=100us"); /* 100 us should be enough */
     set_config("RC_RETRY_COUNT?=4");
@@ -49,17 +42,16 @@ void test_uct_peer_failure::init()
     for (size_t i = 0; i < 2; ++i) {
         new_receiver();
     }
-
-    m_err_count       = 0;
-    m_req_purge_count = 0;
-    m_am_count        = 0;
 }
 
 ucs_status_t test_uct_peer_failure::am_dummy_handler(void *arg, void *data,
                                                      size_t length,
                                                      unsigned flags)
 {
-    reinterpret_cast<test_uct_peer_failure*>(arg)->m_am_count++;
+    test_uct_peer_failure* test = reinterpret_cast<test_uct_peer_failure*>(arg);
+    uint64_t index              = *reinterpret_cast<uint64_t*>(data);
+
+    test->m_am_count[index]++;
     return UCS_OK;
 }
 
@@ -84,7 +76,9 @@ ucs_status_t test_uct_peer_failure::pending_cb(uct_pending_req_t *self)
 
 void test_uct_peer_failure::purge_cb(uct_pending_req_t *self, void *arg)
 {
-    m_req_purge_count++;
+    test_uct_peer_failure *test =
+            reinterpret_cast<test_uct_peer_failure*>(arg);
+    test->m_req_purge_count++;
 }
 
 ucs_status_t test_uct_peer_failure::err_cb(void *arg, uct_ep_h ep,
@@ -95,6 +89,10 @@ ucs_status_t test_uct_peer_failure::err_cb(void *arg, uct_ep_h ep,
     self->m_err_count++;
     self->m_failed_eps.insert(std::make_pair(ep, status));
 
+    /* any new op is not determined */
+    uct_ep_pending_purge(ep, purge_cb, self);
+    EXPECT_EQ(self->m_req_pending_count, self->m_req_purge_count);
+
     switch (status) {
     case UCS_ERR_ENDPOINT_TIMEOUT:
     case UCS_ERR_CONNECTION_RESET:
@@ -107,8 +105,21 @@ ucs_status_t test_uct_peer_failure::err_cb(void *arg, uct_ep_h ep,
     }
 }
 
-void test_uct_peer_failure::kill_receiver()
+void test_uct_peer_failure::inject_error(unsigned idx)
 {
+    uct_ep_h ep         = m_sender->ep(idx);
+    ucs_status_t status = uct_ep_invalidate(ep, 0);
+
+    if (status == UCS_ERR_UNSUPPORTED) {
+        kill_receiver(idx);
+    } else {
+        ASSERT_UCS_OK(status);
+    }
+}
+
+void test_uct_peer_failure::kill_receiver(unsigned idx)
+{
+    /* TODO: add mapping sender idx -> receiver */
     ucs_assert(!m_receivers.empty());
     m_entities.remove(m_receivers.front());
     ucs_assert(m_entities.size() == m_receivers.size());
@@ -125,8 +136,10 @@ void test_uct_peer_failure::new_receiver()
     m_sender->connect(m_receivers.size() - 1, *m_receivers.back(), 0);
 
     if (m_sender->iface_attr().cap.flags & UCT_IFACE_FLAG_AM_SHORT) {
+        m_am_count.push_back(0);
         /* Make sure that TL is up and has resources */
-        am_handler_setter(this)(m_receivers.back());
+        uct_iface_set_am_handler(m_receivers.back()->iface(), 0,
+                                 am_dummy_handler, this, 0);
         send_recv_am(m_receivers.size() - 1);
     }
 }
@@ -134,8 +147,9 @@ void test_uct_peer_failure::new_receiver()
 void test_uct_peer_failure::set_am_handlers()
 {
     check_caps_skip(UCT_IFACE_FLAG_CB_SYNC);
-    std::for_each(m_receivers.begin(), m_receivers.end(),
-                  am_handler_setter(this));
+    for (auto e : m_receivers) {
+        uct_iface_set_am_handler(e->iface(), 0, am_dummy_handler, this, 0);
+    }
 }
 
 ucs_status_t test_uct_peer_failure::send_am(int index)
@@ -152,7 +166,8 @@ ucs_status_t test_uct_peer_failure::send_am(int index)
             return it->second;
         }
 
-        status = uct_ep_am_short(m_sender->ep(index), 0, 0, NULL, 0);
+        status = uct_ep_am_short(m_sender->ep(index), 0, uint64_t(index), NULL,
+                                 0);
     } while (status == UCS_ERR_NO_RESOURCE);
 
     return status;
@@ -160,14 +175,14 @@ ucs_status_t test_uct_peer_failure::send_am(int index)
 
 void test_uct_peer_failure::send_recv_am(int index, ucs_status_t exp_status)
 {
-    m_am_count = 0;
+    m_am_count[index] = 0;
 
     ucs_status_t status = send_am(index);
     EXPECT_EQ(exp_status, status);
 
     if (exp_status == UCS_OK) {
-        wait_for_flag(&m_am_count);
-        EXPECT_EQ(m_am_count, 1ul);
+        wait_for_flag(&m_am_count[index]);
+        EXPECT_EQ(m_am_count[index], 1ul);
     }
 }
 
@@ -210,6 +225,7 @@ ucs_status_t test_uct_peer_failure::flush_ep(size_t index,
 ucs_status_t test_uct_peer_failure::add_pending(uct_ep_h ep,
                                                 pending_send_request_t &req)
 {
+    ++m_req_pending_count;
     req.ep       = ep;
     req.uct.func = pending_cb;
     return uct_ep_pending_add(ep, &req.uct, 0);
@@ -238,19 +254,6 @@ void test_uct_peer_failure::fill_resources(bool expect_error,
     }
 }
 
-test_uct_peer_failure::am_handler_setter::am_handler_setter(test_uct_peer_failure *test) :
-    m_test(test)
-{
-}
-
-void
-test_uct_peer_failure::am_handler_setter::operator() (test_uct_peer_failure::entity *e)
-{
-    uct_iface_set_am_handler(e->iface(), 0,
-                             am_dummy_handler,
-                             reinterpret_cast<void*>(m_test), 0);
-}
-
 UCS_TEST_SKIP_COND_P(test_uct_peer_failure, peer_failure,
                      !check_caps(UCT_IFACE_FLAG_PUT_SHORT |
                                  m_required_caps))
@@ -258,7 +261,7 @@ UCS_TEST_SKIP_COND_P(test_uct_peer_failure, peer_failure,
     {
         scoped_log_handler slh(wrap_errors_logger);
 
-        kill_receiver();
+        inject_error();
         EXPECT_EQ(UCS_OK, uct_ep_put_short(ep0(), NULL, 0, 0, 0));
 
         flush();
@@ -284,7 +287,7 @@ UCS_TEST_SKIP_COND_P(test_uct_peer_failure, purge_failed_peer,
         ucs_status_t status;
 
         fill_resources(false, loop_end_limit);
-        kill_receiver();
+        inject_error();
 
         do {
             status = add_pending(ep0(), reqs[0]);
@@ -307,12 +310,6 @@ UCS_TEST_SKIP_COND_P(test_uct_peer_failure, purge_failed_peer,
     }
 
     EXPECT_GE(m_err_count, 0ul);
-
-    /* any new op is not determined */
-
-    uct_ep_pending_purge(ep0(), purge_cb, NULL);
-    EXPECT_EQ(num_pend_sends, m_req_purge_count);
-    EXPECT_GE(m_err_count, 0ul);
 }
 
 UCS_TEST_SKIP_COND_P(test_uct_peer_failure, two_pairs_send,
@@ -328,7 +325,7 @@ UCS_TEST_SKIP_COND_P(test_uct_peer_failure, two_pairs_send,
     /* kill the 1st receiver while sending on 2nd pair */
     {
         scoped_log_handler slh(wrap_errors_logger);
-        kill_receiver();
+        inject_error();
         send_am(0);
         send_recv_am(1);
         flush();
@@ -361,8 +358,10 @@ void test_uct_peer_failure_multiple::init()
 {
     size_t tx_queue_len = get_tx_queue_len();
 
-    if (ucs_get_page_size() > 4096) {
-        /* NOTE: Too much receivers may cause failure of ibv_open_device */
+    if ((ucs_get_page_size() > 4096) ||
+        (ucs_arch_get_cpu_model() == UCS_CPU_MODEL_ARM_AARCH64) ||
+        (has_ud() && ucs::is_aws())) {
+        /* NOTE: Too many receivers may cause failure of ibv_open_device */
         m_nreceivers = 10;
     } else {
         m_nreceivers = tx_queue_len;
@@ -434,7 +433,7 @@ UCS_TEST_SKIP_COND_P(test_uct_peer_failure_multiple, test,
             for (size_t i = 0; i < m_tx_window; ++i) {
                 EXPECT_UCS_OK(send_am(idx));
             }
-            kill_receiver();
+            inject_error(idx);
         }
         flush(timeout);
 
@@ -450,13 +449,14 @@ UCS_TEST_SKIP_COND_P(test_uct_peer_failure_multiple, test,
         flush(timeout);
     }
 
-    m_am_count = 0;
-    EXPECT_UCS_OK(send_am(m_nreceivers - 1));
+    int idx = m_nreceivers - 1;
+    m_am_count[idx] = 0;
+    EXPECT_UCS_OK(send_am(idx));
     ucs_debug("flushing");
-    flush_ep(m_nreceivers - 1);
+    flush_ep(idx);
     ucs_debug("flushed");
-    wait_for_flag(&m_am_count);
-    EXPECT_EQ(m_am_count, 1ul);
+    wait_for_flag(&m_am_count[idx]);
+    EXPECT_EQ(m_am_count[idx], 1ul);
 }
 
 UCT_INSTANTIATE_TEST_CASE(test_uct_peer_failure_multiple)
@@ -464,23 +464,28 @@ UCT_INSTANTIATE_TEST_CASE(test_uct_peer_failure_multiple)
 class test_uct_keepalive : public uct_test {
 public:
     test_uct_keepalive() :
-        m_ka(NULL), m_pid(getpid()), m_entity(NULL), m_err_handler_count(0)
+        m_pid(getpid()), m_entity(NULL), m_err_handler_count(0)
     {
     }
 
     void init()
     {
-        ASSERT_UCS_OK(uct_ep_keepalive_create(m_pid, &m_ka));
-
         m_entity = create_entity(0, err_handler_cb);
         m_entity->connect(0, *m_entity, 0);
         m_entities.push_back(m_entity);
+        ASSERT_TRUE(has_mm());
     }
 
     void cleanup()
     {
         m_entities.clear();
-        ucs_free(m_ka);
+    }
+
+    uct_keepalive_info_t *m_ka()
+    {
+        uct_mm_ep_t *ep = ucs_derived_of(m_entity->ep(0), uct_mm_ep_t);
+
+        return &ep->keepalive;
     }
 
     static ucs_status_t
@@ -494,12 +499,9 @@ public:
 protected:
     void do_keepalive()
     {
-        ucs_status_t status = uct_ep_keepalive_check(m_entity->ep(0), &m_ka,
-                                                     m_pid, 0, NULL);
-        EXPECT_UCS_OK(status);
+        uct_ep_keepalive_check(m_entity->ep(0), m_ka(), m_pid, 0, NULL);
     }
 
-    uct_keepalive_info_t *m_ka;
     pid_t                m_pid;
     entity               *m_entity;
     unsigned             m_err_handler_count;
@@ -513,7 +515,7 @@ UCS_TEST_P(test_uct_keepalive, ep_check)
     EXPECT_EQ(0u, m_err_handler_count);
 
     /* change start time saved in KA to force an error from EP check */
-    m_ka->start_time.tv_sec--;
+    m_ka()->start_time--;
 
     do_keepalive();
     EXPECT_EQ(0u, m_err_handler_count);
@@ -534,74 +536,85 @@ public:
         m_env.push_back(new ucs::scoped_setenv("UCX_TCP_KEEPIDLE", "inf"));
     }
 
+protected:
     void kill_receiver()
     {
         /* Hack: for SHM-based transports we can't really terminate
          * peer EP, but instead we bit change process owner info to force
          * ep_check failure. Simulation of case when peer process is
          * terminated and PID is immediately reused by another process */
-        uct_ep_h tl_ep                = ep0();
-        uct_keepalive_info_t *ka_info = NULL;
 
         if (has_mm()) {
-            uct_mm_ep_t *ep = ucs_derived_of(tl_ep, uct_mm_ep_t);
-            ka_info         = ep->keepalive;
-            ASSERT_TRUE(ka_info != NULL);
-        } else if (has_cuda_ipc()) {
-#if HAVE_CUDA
-            uct_cuda_ipc_ep_t *ep = ucs_derived_of(tl_ep, uct_cuda_ipc_ep_t);
-            ka_info               = ep->keepalive;
-            ASSERT_TRUE(ka_info != NULL);
-#endif
+            uct_mm_ep_t *ep = ucs_derived_of(ep0(), uct_mm_ep_t);
+            ep->keepalive.start_time--;
         } else if (has_cma()) {
-            uct_cma_ep_t *ep = ucs_derived_of(tl_ep, uct_cma_ep_t);
-            ka_info          = ep->keepalive;
-            ASSERT_TRUE(ka_info != NULL);
-        }
-
-        if (ka_info != NULL) {
-            ka_info->start_time.tv_sec--;
+            uct_cma_ep_t *ep = ucs_derived_of(ep0(), uct_cma_ep_t);
+            ep->keepalive.start_time--;
         }
 
+        /* kill real receiver since DC uses shared KA DCI */
         test_uct_peer_failure::kill_receiver();
     }
 
+    void test_ep_check(bool post_am)
+    {
+        ucs_status_t status;
+
+        scoped_log_handler slh(wrap_errors_logger);
+        flush();
+        EXPECT_EQ(0, m_err_count);
+
+        for (unsigned i = 0; i < 8; ++i) {
+            status = uct_ep_check(ep0(), 0, NULL);
+            ASSERT_UCS_OK(status);
+            flush();
+            /* Allow keepalive request to complete */
+            short_progress_loop();
+        }
+
+        /* We are still alive */
+        EXPECT_EQ(0, m_err_count);
+
+        kill_receiver();
+
+        status = uct_ep_check(ep0(), 0, NULL);
+        ASSERT_UCS_OK(status);
+
+        if (post_am) {
+            /* Post AM operation to check that an error could be detected by
+             * EP_CHECK when an endpoint has an in-flight AM operation */
+            const uint64_t send_data = 0;
+            status = uct_ep_am_short(ep0(), 0, 0, &send_data, sizeof(send_data));
+            if (m_err_count == 0) {
+                ASSERT_UCS_OK(status);
+            }
+        }
+
+        flush();
+
+        wait_for_flag(&m_err_count);
+        EXPECT_EQ(1, m_err_count);
+    }
+
 protected:
     ucs::ptr_vector<ucs::scoped_setenv> m_env;
 };
 
+
 UCS_TEST_SKIP_COND_P(test_uct_peer_failure_keepalive, killed,
                      !check_caps(UCT_IFACE_FLAG_EP_CHECK))
 {
-    ucs_status_t status;
+    test_ep_check(false);
+}
 
-    scoped_log_handler slh(wrap_errors_logger);
-    flush();
-    EXPECT_EQ(0, m_err_count);
-
-    status = uct_ep_check(ep0(), 0, NULL);
-    ASSERT_UCS_OK(status);
-    flush();
-
-    /* allow keepalive requests to complete */
-    short_progress_loop();
-
-    /* we are still alive */
-    EXPECT_EQ(0, m_err_count);
-
-    kill_receiver();
-
-    status = uct_ep_check(ep0(), 0, NULL);
-    ASSERT_UCS_OK(status);
-    flush();
-
-    wait_for_flag(&m_err_count);
-    EXPECT_EQ(1, m_err_count);
+UCS_TEST_SKIP_COND_P(test_uct_peer_failure_keepalive, killed_post_am,
+                     !check_caps(UCT_IFACE_FLAG_EP_CHECK |
+                                 UCT_IFACE_FLAG_AM_SHORT))
+{
+    test_ep_check(true);
 }
 
 UCT_INSTANTIATE_NO_SELF_TEST_CASE(test_uct_peer_failure_keepalive)
-_UCT_INSTANTIATE_TEST_CASE(test_uct_peer_failure_keepalive, cuda_ipc);
-
 
 class test_uct_peer_failure_rma_zcopy : public test_uct_peer_failure
 {
