@@ -17,27 +17,23 @@
  */
 
 int MPIR_Allgatherv_intra_brucks(const void *sendbuf,
-                                 int sendcount,
+                                 MPI_Aint sendcount,
                                  MPI_Datatype sendtype,
                                  void *recvbuf,
-                                 const int *recvcounts,
-                                 const int *displs,
-                                 MPI_Datatype recvtype,
-                                 MPIR_Comm * comm_ptr, MPIR_Errflag_t * errflag)
+                                 const MPI_Aint * recvcounts,
+                                 const MPI_Aint * displs,
+                                 MPI_Datatype recvtype, MPIR_Comm * comm_ptr, int coll_attr)
 {
     int comm_size, rank, j, i;
     int mpi_errno = MPI_SUCCESS;
-    int mpi_errno_ret = MPI_SUCCESS;
     MPI_Status status;
     MPI_Aint recvtype_extent, recvtype_sz;
-    int pof2, src, rem, send_cnt;
-    MPI_Aint curr_cnt, recv_cnt;
-    int dst, total_count;
+    int pof2, src, dst, rem;
+    MPI_Aint curr_cnt, send_cnt, recv_cnt, total_count;
     void *tmp_buf;
-    MPIR_CHKLMEM_DECL(1);
+    MPIR_CHKLMEM_DECL();
 
-    comm_size = comm_ptr->local_size;
-    rank = comm_ptr->rank;
+    MPIR_COMM_RANK_SIZE(comm_ptr, rank, comm_size);
 
     total_count = 0;
     for (i = 0; i < comm_size; i++)
@@ -51,19 +47,18 @@ int MPIR_Allgatherv_intra_brucks(const void *sendbuf,
     /* allocate a temporary buffer that can hold all the data */
     MPIR_Datatype_get_size_macro(recvtype, recvtype_sz);
 
-    MPIR_CHKLMEM_MALLOC(tmp_buf, void *, total_count * recvtype_sz, mpi_errno, "tmp_buf",
-                        MPL_MEM_BUFFER);
+    MPIR_CHKLMEM_MALLOC(tmp_buf, total_count * recvtype_sz);
 
     /* copy local data to the top of tmp_buf */
     if (sendbuf != MPI_IN_PLACE) {
         mpi_errno = MPIR_Localcopy(sendbuf, sendcount, sendtype,
-                                   tmp_buf, recvcounts[rank] * recvtype_sz, MPI_BYTE);
+                                   tmp_buf, recvcounts[rank] * recvtype_sz, MPIR_BYTE_INTERNAL);
         MPIR_ERR_CHECK(mpi_errno);
     } else {
         mpi_errno = MPIR_Localcopy(((char *) recvbuf +
                                     displs[rank] * recvtype_extent),
                                    recvcounts[rank], recvtype, tmp_buf,
-                                   recvcounts[rank] * recvtype_sz, MPI_BYTE);
+                                   recvcounts[rank] * recvtype_sz, MPIR_BYTE_INTERNAL);
         MPIR_ERR_CHECK(mpi_errno);
     }
 
@@ -75,21 +70,17 @@ int MPIR_Allgatherv_intra_brucks(const void *sendbuf,
         src = (rank + pof2) % comm_size;
         dst = (rank - pof2 + comm_size) % comm_size;
 
-        mpi_errno = MPIC_Sendrecv(tmp_buf, curr_cnt * recvtype_sz, MPI_BYTE, dst,
+        mpi_errno = MPIC_Sendrecv(tmp_buf, curr_cnt * recvtype_sz, MPIR_BYTE_INTERNAL, dst,
                                   MPIR_ALLGATHERV_TAG,
                                   ((char *) tmp_buf + curr_cnt * recvtype_sz),
-                                  (total_count - curr_cnt) * recvtype_sz, MPI_BYTE,
-                                  src, MPIR_ALLGATHERV_TAG, comm_ptr, &status, errflag);
+                                  (total_count - curr_cnt) * recvtype_sz, MPIR_BYTE_INTERNAL,
+                                  src, MPIR_ALLGATHERV_TAG, comm_ptr, &status, coll_attr);
+        MPIR_ERR_CHECK(mpi_errno);
         if (mpi_errno) {
-            /* for communication errors, just record the error but continue */
-            *errflag =
-                MPIX_ERR_PROC_FAILED ==
-                MPIR_ERR_GET_CLASS(mpi_errno) ? MPIR_ERR_PROC_FAILED : MPIR_ERR_OTHER;
-            MPIR_ERR_SET(mpi_errno, *errflag, "**fail");
-            MPIR_ERR_ADD(mpi_errno_ret, mpi_errno);
             recv_cnt = 0;
-        } else
+        } else {
             MPIR_Get_count_impl(&status, recvtype, &recv_cnt);
+        }
         curr_cnt += recv_cnt;
 
         pof2 *= 2;
@@ -106,19 +97,12 @@ int MPIR_Allgatherv_intra_brucks(const void *sendbuf,
         for (i = 0; i < rem; i++)
             send_cnt += recvcounts[(rank + i) % comm_size];
 
-        mpi_errno = MPIC_Sendrecv(tmp_buf, send_cnt * recvtype_sz, MPI_BYTE,
+        mpi_errno = MPIC_Sendrecv(tmp_buf, send_cnt * recvtype_sz, MPIR_BYTE_INTERNAL,
                                   dst, MPIR_ALLGATHERV_TAG,
                                   ((char *) tmp_buf + curr_cnt * recvtype_sz),
-                                  (total_count - curr_cnt) * recvtype_sz, MPI_BYTE,
-                                  src, MPIR_ALLGATHERV_TAG, comm_ptr, MPI_STATUS_IGNORE, errflag);
-        if (mpi_errno) {
-            /* for communication errors, just record the error but continue */
-            *errflag =
-                MPIX_ERR_PROC_FAILED ==
-                MPIR_ERR_GET_CLASS(mpi_errno) ? MPIR_ERR_PROC_FAILED : MPIR_ERR_OTHER;
-            MPIR_ERR_SET(mpi_errno, *errflag, "**fail");
-            MPIR_ERR_ADD(mpi_errno_ret, mpi_errno);
-        }
+                                  (total_count - curr_cnt) * recvtype_sz, MPIR_BYTE_INTERNAL,
+                                  src, MPIR_ALLGATHERV_TAG, comm_ptr, MPI_STATUS_IGNORE, coll_attr);
+        MPIR_ERR_CHECK(mpi_errno);
     }
 
     /* Rotate blocks in tmp_buf down by (rank) blocks and store
@@ -128,7 +112,7 @@ int MPIR_Allgatherv_intra_brucks(const void *sendbuf,
     for (i = 0; i < (comm_size - rank); i++) {
         j = (rank + i) % comm_size;
         mpi_errno = MPIR_Localcopy((char *) tmp_buf + send_cnt * recvtype_sz,
-                                   recvcounts[j] * recvtype_sz, MPI_BYTE,
+                                   recvcounts[j] * recvtype_sz, MPIR_BYTE_INTERNAL,
                                    (char *) recvbuf + displs[j] * recvtype_extent,
                                    recvcounts[j], recvtype);
         MPIR_ERR_CHECK(mpi_errno);
@@ -137,7 +121,7 @@ int MPIR_Allgatherv_intra_brucks(const void *sendbuf,
 
     for (i = 0; i < rank; i++) {
         mpi_errno = MPIR_Localcopy((char *) tmp_buf + send_cnt * recvtype_sz,
-                                   recvcounts[i] * recvtype_sz, MPI_BYTE,
+                                   recvcounts[i] * recvtype_sz, MPIR_BYTE_INTERNAL,
                                    (char *) recvbuf + displs[i] * recvtype_extent,
                                    recvcounts[i], recvtype);
         MPIR_ERR_CHECK(mpi_errno);
@@ -146,11 +130,6 @@ int MPIR_Allgatherv_intra_brucks(const void *sendbuf,
 
   fn_exit:
     MPIR_CHKLMEM_FREEALL();
-    if (mpi_errno_ret)
-        mpi_errno = mpi_errno_ret;
-    else if (*errflag != MPIR_ERR_NONE)
-        MPIR_ERR_SET(mpi_errno, *errflag, "**coll_fail");
-
     return mpi_errno;
   fn_fail:
     goto fn_exit;
