@@ -5,10 +5,10 @@
 
 #include "mpiimpl.h"
 
-int MPIR_Alltoallv_allcomm_nb(const void *sendbuf, const int *sendcounts, const int *sdispls,
-                              MPI_Datatype sendtype, void *recvbuf, const int *recvcounts,
-                              const int *rdispls, MPI_Datatype recvtype, MPIR_Comm * comm_ptr,
-                              MPIR_Errflag_t * errflag)
+int MPIR_Alltoallv_allcomm_nb(const void *sendbuf, const MPI_Aint * sendcounts,
+                              const MPI_Aint * sdispls, MPI_Datatype sendtype, void *recvbuf,
+                              const MPI_Aint * recvcounts, const MPI_Aint * rdispls,
+                              MPI_Datatype recvtype, MPIR_Comm * comm_ptr, int coll_attr)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIR_Request *req_ptr = NULL;
@@ -19,7 +19,7 @@ int MPIR_Alltoallv_allcomm_nb(const void *sendbuf, const int *sendcounts, const
                         recvtype, comm_ptr, &req_ptr);
     MPIR_ERR_CHECK(mpi_errno);
 
-    mpi_errno = MPIC_Wait(req_ptr, errflag);
+    mpi_errno = MPIC_Wait(req_ptr);
     MPIR_ERR_CHECK(mpi_errno);
     MPIR_Request_free(req_ptr);
 
