@@ -14,11 +14,10 @@ int MPIR_TSP_Ireduce_sched_intra_tree(const void *sendbuf, void *recvbuf, MPI_Ai
                                       int buffer_per_child, MPIR_TSP_sched_t sched)
 {
     int mpi_errno = MPI_SUCCESS;
-    int mpi_errno_ret ATTRIBUTE((unused)) = MPI_SUCCESS;
     int i, j, t;
     MPI_Aint num_chunks, chunk_size_floor, chunk_size_ceil;
     int offset = 0;
-    size_t extent, type_size;
+    MPI_Aint extent, type_size;
     MPI_Aint type_lb, true_extent;
     int is_commutative;
     int size;
@@ -38,13 +37,11 @@ int MPIR_TSP_Ireduce_sched_intra_tree(const void *sendbuf, void *recvbuf, MPI_Ai
     int dtcopy_id = -1;
     int nvtcs;
     int tag;
-    MPIR_Errflag_t errflag ATTRIBUTE((unused)) = MPIR_ERR_NONE;
-    MPIR_CHKLMEM_DECL(3);
+    MPIR_CHKLMEM_DECL();
 
     MPIR_FUNC_ENTER;
 
-    size = MPIR_Comm_size(comm);
-    rank = MPIR_Comm_rank(comm);
+    MPIR_COMM_RANK_SIZE(comm, rank, size);
     is_root = (rank == root);
 
     /* main algorithm */
@@ -69,7 +66,45 @@ int MPIR_TSP_Ireduce_sched_intra_tree(const void *sendbuf, void *recvbuf, MPI_Ai
 
     /* initialize the tree */
     my_tree.children = NULL;
-    mpi_errno = MPIR_Treealgo_tree_create(rank, size, tree_type, k, tree_root, &my_tree);
+    if (tree_type == MPIR_TREE_TYPE_TOPOLOGY_AWARE || tree_type == MPIR_TREE_TYPE_TOPOLOGY_AWARE_K) {
+        mpi_errno =
+            MPIR_Treealgo_tree_create_topo_aware(comm, tree_type, k, tree_root,
+                                                 MPIR_CVAR_IREDUCE_TOPO_REORDER_ENABLE, &my_tree);
+    } else if (tree_type == MPIR_TREE_TYPE_TOPOLOGY_WAVE) {
+        MPIR_Csel_coll_sig_s coll_sig = {
+            .coll_type = MPIR_CSEL_COLL_TYPE__IREDUCE,
+            .comm_ptr = comm,
+            .u.ireduce.sendbuf = sendbuf,
+            .u.ireduce.recvbuf = recvbuf,
+            .u.ireduce.count = count,
+            .u.ireduce.datatype = datatype,
+            .u.ireduce.op = op,
+            .u.ireduce.root = tree_root,
+        };
+
+        int overhead = MPIR_CVAR_IREDUCE_TOPO_OVERHEAD;
+        int lat_diff_groups = MPIR_CVAR_IREDUCE_TOPO_DIFF_GROUPS;
+        int lat_diff_switches = MPIR_CVAR_IREDUCE_TOPO_DIFF_SWITCHES;
+        int lat_same_switches = MPIR_CVAR_IREDUCE_TOPO_SAME_SWITCHES;
+
+        MPII_Csel_container_s *cnt = MPIR_Csel_search(comm->csel_comm, coll_sig);
+        MPIR_Assert(cnt);
+
+        if (cnt->id == MPII_CSEL_CONTAINER_TYPE__ALGORITHM__MPIR_Ireduce_intra_tsp_tree) {
+            overhead = cnt->u.ireduce.intra_tsp_tree.topo_overhead;
+            lat_diff_groups = cnt->u.ireduce.intra_tsp_tree.topo_diff_groups;
+            lat_diff_switches = cnt->u.ireduce.intra_tsp_tree.topo_diff_switches;
+            lat_same_switches = cnt->u.ireduce.intra_tsp_tree.topo_same_switches;
+        }
+
+        mpi_errno =
+            MPIR_Treealgo_tree_create_topo_wave(comm, k, tree_root,
+                                                MPIR_CVAR_IREDUCE_TOPO_REORDER_ENABLE,
+                                                overhead, lat_diff_groups, lat_diff_switches,
+                                                lat_same_switches, &my_tree);
+    } else {
+        mpi_errno = MPIR_Treealgo_tree_create(rank, size, tree_type, k, tree_root, &my_tree);
+    }
     MPIR_ERR_CHECK(mpi_errno);
     num_children = my_tree.num_children;
 
@@ -89,7 +124,9 @@ int MPIR_TSP_Ireduce_sched_intra_tree(const void *sendbuf, void *recvbuf, MPI_Ai
      * the programmer once that memory is no longer required */
     if (!is_tree_leaf) {
         child_buffer = MPIR_TSP_sched_malloc(sizeof(void *) * num_children, sched);
+        MPIR_Assert(child_buffer != NULL);
         child_buffer[0] = MPIR_TSP_sched_malloc(extent * count, sched);
+        MPIR_Assert(child_buffer[0] != NULL);
         child_buffer[0] = (void *) ((char *) child_buffer[0] - type_lb);
         for (i = 1; i < num_children; i++) {
             if (buffer_per_child) {
@@ -132,16 +169,13 @@ int MPIR_TSP_Ireduce_sched_intra_tree(const void *sendbuf, void *recvbuf, MPI_Ai
         reduce_buffer = (void *) sendbuf;
     }
 
-    MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+    MPIR_ERR_CHECK(mpi_errno);
 
     /* initialize arrays to store graph vertex indices */
-    MPIR_CHKLMEM_MALLOC(vtcs, int *, sizeof(int) * (num_children + 1),
-                        mpi_errno, "vtcs buffer", MPL_MEM_COLL);
+    MPIR_CHKLMEM_MALLOC(vtcs, sizeof(int) * (num_children + 1));
     if (num_children > 0) {
-        MPIR_CHKLMEM_MALLOC(reduce_id, int *, sizeof(int) * num_children,
-                            mpi_errno, "reduce_id buffer", MPL_MEM_COLL);
-        MPIR_CHKLMEM_MALLOC(recv_id, int *, sizeof(int) * num_children,
-                            mpi_errno, "recv_id buffer", MPL_MEM_COLL);
+        MPIR_CHKLMEM_MALLOC(reduce_id, sizeof(int) * num_children);
+        MPIR_CHKLMEM_MALLOC(recv_id, sizeof(int) * num_children);
     }
 
     /* do pipelined reduce */
@@ -177,7 +211,7 @@ int MPIR_TSP_Ireduce_sched_intra_tree(const void *sendbuf, void *recvbuf, MPI_Ai
             mpi_errno = MPIR_TSP_sched_irecv(recv_address, msgsize, datatype, child, tag, comm,
                                              sched, nvtcs, vtcs, &recv_id[i]);
 
-            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+            MPIR_ERR_CHECK(mpi_errno);
             /* Setup dependencies for reduction. Reduction depends on the corresponding recv to complete */
             vtcs[0] = recv_id[i];
             nvtcs = 1;
@@ -204,7 +238,7 @@ int MPIR_TSP_Ireduce_sched_intra_tree(const void *sendbuf, void *recvbuf, MPI_Ai
                 reduce_id[i] = vtx_id;
 
             }
-            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+            MPIR_ERR_CHECK(mpi_errno);
         }
 
         if (is_commutative && buffer_per_child) {       /* wait for all the reductions */
@@ -222,7 +256,7 @@ int MPIR_TSP_Ireduce_sched_intra_tree(const void *sendbuf, void *recvbuf, MPI_Ai
             mpi_errno =
                 MPIR_TSP_sched_isend(reduce_address, msgsize, datatype, my_tree.parent, tag, comm,
                                      sched, nvtcs, vtcs, &vtx_id);
-            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+            MPIR_ERR_CHECK(mpi_errno);
         }
 
         /* send data to the root of the collective operation */
@@ -236,7 +270,7 @@ int MPIR_TSP_Ireduce_sched_intra_tree(const void *sendbuf, void *recvbuf, MPI_Ai
                     MPIR_TSP_sched_irecv((char *) recvbuf + offset * extent, msgsize, datatype,
                                          tree_root, tag, comm, sched, 0, NULL, &vtx_id);
             }
-            MPIR_ERR_COLL_CHECKANDCONT(mpi_errno, errflag);
+            MPIR_ERR_CHECK(mpi_errno);
         }
 
         offset += msgsize;
