@@ -5,14 +5,14 @@
 
 #include "mpiimpl.h"
 
-int MPIR_Neighbor_alltoallv_allcomm_nb(const void *sendbuf, const int sendcounts[],
-                                       const int sdispls[], MPI_Datatype sendtype, void *recvbuf,
-                                       const int recvcounts[], const int rdispls[],
-                                       MPI_Datatype recvtype, MPIR_Comm * comm_ptr)
+int MPIR_Neighbor_alltoallv_allcomm_nb(const void *sendbuf, const MPI_Aint sendcounts[],
+                                       const MPI_Aint sdispls[], MPI_Datatype sendtype,
+                                       void *recvbuf, const MPI_Aint recvcounts[],
+                                       const MPI_Aint rdispls[], MPI_Datatype recvtype,
+                                       MPIR_Comm * comm_ptr)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIR_Request *req_ptr = NULL;
-    MPIR_Errflag_t errflag;
 
     /* just call the nonblocking version and wait on it */
     mpi_errno =
@@ -20,7 +20,7 @@ int MPIR_Neighbor_alltoallv_allcomm_nb(const void *sendbuf, const int sendcounts
                                  rdispls, recvtype, comm_ptr, &req_ptr);
     MPIR_ERR_CHECK(mpi_errno);
 
-    mpi_errno = MPIC_Wait(req_ptr, &errflag);
+    mpi_errno = MPIC_Wait(req_ptr);
     MPIR_ERR_CHECK(mpi_errno);
     MPIR_Request_free(req_ptr);
 
