@@ -50,6 +50,18 @@ cvars:
       scope       : MPI_T_SCOPE_ALL_EQ
       description : >-
         This cvar enables yaksa based reduction for local reduce.
+
+    - name        : MPIR_CVAR_YAKSA_REDUCTION_THRESHOLD
+      category    : COLLECTIVE
+      type        : int
+      default     : -1
+      class       : none
+      verbosity   : MPI_T_VERBOSITY_USER_BASIC
+      scope       : MPI_T_SCOPE_ALL_EQ
+      description : >-
+        This cvar disables yaksa based reduction for messages above the threshold.
+        The default is no limit.
+
 === END_MPI_T_CVAR_INFO_BLOCK ===
 */
 
@@ -68,7 +80,7 @@ static int typerep_do_copy(void *outbuf, const void *inbuf, MPI_Aint num_bytes,
     int rc;
 
     if (typerep_req) {
-        *typerep_req = MPIR_TYPEREP_REQ_NULL;
+        typerep_req->req = MPIR_TYPEREP_REQ_NULL;
     }
 
     if (num_bytes == 0) {
@@ -86,18 +98,20 @@ static int typerep_do_copy(void *outbuf, const void *inbuf, MPI_Aint num_bytes,
 
         yaksa_info_t info = MPII_yaksa_get_info(&inattr, &outattr);
         if (typerep_req) {
+            typerep_req->info = info;
             rc = yaksa_ipack(inbuf, num_bytes, YAKSA_TYPE__BYTE, 0, outbuf, num_bytes,
                              &actual_pack_bytes, info, YAKSA_OP__REPLACE,
-                             (yaksa_request_t *) typerep_req);
+                             (yaksa_request_t *) & typerep_req->req);
+            MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
+            MPIR_Assert(actual_pack_bytes == num_bytes);
         } else {
             rc = yaksa_pack(inbuf, num_bytes, YAKSA_TYPE__BYTE, 0, outbuf, num_bytes,
                             &actual_pack_bytes, info, YAKSA_OP__REPLACE);
+            MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
+            MPIR_Assert(actual_pack_bytes == num_bytes);
+            rc = MPII_yaksa_free_info(info);
+            MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
         }
-        MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
-        MPIR_Assert(actual_pack_bytes == num_bytes);
-
-        rc = MPII_yaksa_free_info(info);
-        MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
     }
 
   fn_exit:
@@ -120,7 +134,7 @@ static int typerep_do_pack(const void *inbuf, MPI_Aint incount, MPI_Datatype dat
     int rc;
 
     if (typerep_req) {
-        *typerep_req = MPIR_TYPEREP_REQ_NULL;
+        typerep_req->req = MPIR_TYPEREP_REQ_NULL;
     }
 
     if (incount == 0) {
@@ -175,18 +189,19 @@ static int typerep_do_pack(const void *inbuf, MPI_Aint incount, MPI_Datatype dat
 
     uintptr_t real_pack_bytes;
     if (typerep_req) {
+        typerep_req->info = info;
         rc = yaksa_ipack(inbuf, incount, type, inoffset, outbuf, max_pack_bytes,
                          &real_pack_bytes, info, YAKSA_OP__REPLACE,
-                         (yaksa_request_t *) typerep_req);
+                         (yaksa_request_t *) & typerep_req->req);
+        MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
     } else {
         rc = yaksa_pack(inbuf, incount, type, inoffset, outbuf, max_pack_bytes,
                         &real_pack_bytes, info, YAKSA_OP__REPLACE);
-    }
-    MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
-
-    if (info) {
-        rc = MPII_yaksa_free_info(info);
         MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
+        if (info) {
+            rc = MPII_yaksa_free_info(info);
+            MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
+        }
     }
 
     *actual_pack_bytes = (MPI_Aint) real_pack_bytes;
@@ -200,56 +215,53 @@ static int typerep_do_pack(const void *inbuf, MPI_Aint incount, MPI_Datatype dat
 
 /* This function checks whether the operation is supported in yaksa for the
  * provided datatype */
-int MPIR_Typerep_reduce_is_supported(MPI_Op op, MPI_Datatype datatype)
+int MPIR_Typerep_reduce_is_supported(MPI_Op op, MPI_Aint count, MPI_Datatype datatype)
 {
     yaksa_op_t yaksa_op;
 
     if (!MPIR_CVAR_ENABLE_YAKSA_REDUCTION)
         return 0;
 
+    /* HACK: check count to decide whether we need reduce buffer swap, ref. maint/gen_coll.py */
+    if (count > 0) {
+        MPI_Aint data_sz;
+        MPIR_Pack_size(count, datatype, &data_sz);
+        if (MPIR_CVAR_YAKSA_REDUCTION_THRESHOLD > 0 &&
+            data_sz > MPIR_CVAR_YAKSA_REDUCTION_THRESHOLD) {
+            return 0;
+        }
+    }
+
     /* yaksa pup code currently treat unsigned integer type the same as
      * the corresponding signed integer type, which will not work with
      * most op other than REPLACE.
      */
-    if (datatype == MPI_UNSIGNED_CHAR ||
-        datatype == MPI_UNSIGNED_SHORT ||
-        datatype == MPI_UNSIGNED ||
-        datatype == MPI_UNSIGNED_LONG ||
-        datatype == MPI_UNSIGNED_LONG_LONG ||
-        datatype == MPI_UINT8_T ||
-        datatype == MPI_UINT16_T || datatype == MPI_UINT32_T || datatype == MPI_UINT64_T) {
+    int mpir_type = datatype & MPIR_TYPE_TYPE_MASK;
+    int type_size = MPIR_Datatype_get_basic_size(datatype);
+
+    if (mpir_type == MPIR_TYPE_UNSIGNED) {
         return 0;
     }
 
-    if ((datatype == MPI_DOUBLE || datatype == MPI_C_DOUBLE_COMPLEX) &&
-        (!MPIR_CVAR_GPU_DOUBLE_SUPPORT))
-        return 0;
-    if ((datatype == MPI_LONG_DOUBLE || datatype == MPI_C_LONG_DOUBLE_COMPLEX) &&
-        (!MPIR_CVAR_GPU_LONG_DOUBLE_SUPPORT))
-        return 0;
+    if (mpir_type == MPIR_TYPE_FLOAT) {
+        if (!MPIR_CVAR_GPU_DOUBLE_SUPPORT && type_size == 8) {
+            return 0;
+        }
+        if (!MPIR_CVAR_GPU_LONG_DOUBLE_SUPPORT && type_size > 8) {
+            return 0;
+        }
+    }
 
-    if ((datatype == MPI_C_COMPLEX || datatype == MPI_C_DOUBLE_COMPLEX ||
-         datatype == MPI_C_LONG_DOUBLE_COMPLEX) && (!MPIR_CVAR_YAKSA_COMPLEX_SUPPORT))
-        return 0;
-#ifdef HAVE_FORTRAN_BINDING
-    if ((datatype == MPI_COMPLEX || datatype == MPI_DOUBLE_COMPLEX || datatype == MPI_COMPLEX8 ||
-         datatype == MPI_COMPLEX16) && (!MPIR_CVAR_YAKSA_COMPLEX_SUPPORT))
-        return 0;
-    if (datatype == MPI_DOUBLE_COMPLEX && (!MPIR_CVAR_GPU_DOUBLE_SUPPORT))
-        return 0;
-#endif
-#ifdef HAVE_CXX_BINDING
-    if ((datatype == MPI_CXX_FLOAT_COMPLEX || datatype == MPI_CXX_DOUBLE_COMPLEX ||
-         datatype == MPI_CXX_LONG_DOUBLE_COMPLEX) && (!MPIR_CVAR_YAKSA_COMPLEX_SUPPORT))
-        return 0;
-    if (datatype == MPI_CXX_DOUBLE_COMPLEX && (!MPIR_CVAR_GPU_DOUBLE_SUPPORT))
-        return 0;
-    if (datatype == MPI_CXX_LONG_DOUBLE_COMPLEX && (!MPIR_CVAR_GPU_LONG_DOUBLE_SUPPORT))
-        return 0;
-#endif
-    if ((datatype == MPI_FLOAT || datatype == MPI_DOUBLE || datatype == MPI_LONG_DOUBLE) &&
-        (op == MPI_LXOR || op == MPI_LAND || op == MPI_LOR))
+    if (mpir_type == MPIR_TYPE_COMPLEX) {
+        if (!MPIR_CVAR_YAKSA_COMPLEX_SUPPORT) {
+            return 0;
+        }
+    }
+
+    if ((mpir_type != MPIR_TYPE_SIGNED && mpir_type != MPIR_TYPE_UNSIGNED) &&
+        (op == MPI_LXOR || op == MPI_LAND || op == MPI_LOR)) {
         return 0;
+    }
 
     yaksa_op = MPII_Typerep_get_yaksa_op(op);
 
@@ -271,7 +283,7 @@ static int typerep_do_unpack(const void *inbuf, MPI_Aint insize, void *outbuf, M
     int rc;
 
     if (typerep_req) {
-        *typerep_req = MPIR_TYPEREP_REQ_NULL;
+        typerep_req->req = MPIR_TYPEREP_REQ_NULL;
     }
 
     if (insize == 0) {
@@ -323,18 +335,19 @@ static int typerep_do_unpack(const void *inbuf, MPI_Aint insize, void *outbuf, M
 
     uintptr_t real_unpack_bytes;
     if (typerep_req) {
+        typerep_req->info = info;
         rc = yaksa_iunpack(inbuf, real_insize, outbuf, outcount, type, outoffset,
                            &real_unpack_bytes, info, YAKSA_OP__REPLACE,
-                           (yaksa_request_t *) typerep_req);
+                           (yaksa_request_t *) & typerep_req->req);
+        MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
     } else {
         rc = yaksa_unpack(inbuf, real_insize, outbuf, outcount, type, outoffset, &real_unpack_bytes,
                           info, YAKSA_OP__REPLACE);
-    }
-    MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
-
-    if (info) {
-        rc = MPII_yaksa_free_info(info);
         MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
+        if (info) {
+            rc = MPII_yaksa_free_info(info);
+            MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
+        }
     }
 
     *actual_unpack_bytes = (MPI_Aint) real_unpack_bytes;
@@ -431,12 +444,41 @@ int MPIR_Typerep_wait(MPIR_Typerep_req typerep_req)
     int mpi_errno = MPI_SUCCESS;
     int rc;
 
-    if (typerep_req == MPIR_TYPEREP_REQ_NULL)
+    if (typerep_req.req == MPIR_TYPEREP_REQ_NULL)
         goto fn_exit;
 
-    rc = yaksa_request_wait((yaksa_request_t) typerep_req);
+    rc = yaksa_request_wait((yaksa_request_t) typerep_req.req);
     MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
 
+    rc = MPII_yaksa_free_info(typerep_req.info);
+    MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
+  fn_exit:
+    MPIR_FUNC_EXIT;
+    return mpi_errno;
+  fn_fail:
+    goto fn_exit;
+}
+
+int MPIR_Typerep_test(MPIR_Typerep_req typerep_req, int *completed)
+{
+    MPIR_FUNC_ENTER;
+    int mpi_errno = MPI_SUCCESS;
+    int rc;
+
+    if (typerep_req.req == MPIR_TYPEREP_REQ_NULL) {
+        MPIR_Assert(0);
+        goto fn_exit;
+    }
+
+    rc = yaksa_request_test((yaksa_request_t) typerep_req.req, completed);
+    MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
+
+    if (*completed) {
+        if (typerep_req.info) {
+            rc = MPII_yaksa_free_info(typerep_req.info);
+            MPIR_ERR_CHKANDJUMP(rc, mpi_errno, MPI_ERR_INTERN, "**yaksa");
+        }
+    }
   fn_exit:
     MPIR_FUNC_EXIT;
     return mpi_errno;
@@ -491,7 +533,8 @@ int MPIR_Typerep_op(void *source_buf, MPI_Aint source_count, MPI_Datatype source
     MPIR_Assert(HANDLE_IS_BUILTIN(op));
     MPIR_Assert(MPIR_DATATYPE_IS_PREDEFINED(source_dtp));
 
-    bool use_yaksa = MPIR_Typerep_reduce_is_supported(op, source_dtp);
+    /* use count=0 since we don't make MPIR_CVAR_YAKSA_REDUCTION_THRESHOLD decision here */
+    bool use_yaksa = MPIR_Typerep_reduce_is_supported(op, 0, source_dtp);
     if (use_yaksa) {
         int source_is_contig, target_is_contig;
         MPIR_Datatype_is_contig(source_dtp, &source_is_contig);
