@@ -21,7 +21,7 @@ cvars:
     - name        : MPIR_CVAR_GPU_FAST_COPY_MAX_SIZE_H2D
       category    : CH4
       type        : int
-      default     : 1048576
+      default     : 4096
       class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
@@ -32,13 +32,25 @@ cvars:
     - name        : MPIR_CVAR_GPU_FAST_COPY_MAX_SIZE_D2H
       category    : CH4
       type        : int
-      default     : 32768
+      default     : 256
       class       : none
       verbosity   : MPI_T_VERBOSITY_USER_BASIC
       scope       : MPI_T_SCOPE_ALL_EQ
       description : >-
         If a send message size is less than or equal to MPIR_CVAR_GPU_FAST_COPY_MAX_SIZE_D2H (in
         bytes), then enable GPU-based fast memcpy.
+
+    - name        : MPIR_CVAR_GPU_FAST_COPY_MAX_SIZE_D2D
+      category    : CH4
+      type        : int
+      default     : 128
+      class       : none
+      verbosity   : MPI_T_VERBOSITY_USER_BASIC
+      scope       : MPI_T_SCOPE_ALL_EQ
+      description : >-
+        If a send message size is less than or equal to MPIR_CVAR_GPU_FAST_COPY_MAX_SIZE_D2D (in
+        bytes), then enable GPU-based fast memcpy.
+
 === END_MPI_T_CVAR_INFO_BLOCK ===
 */
 
@@ -63,7 +75,7 @@ static int do_localcopy(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype se
     MPI_Aint sendsize, recvsize, sdata_sz, rdata_sz, copy_sz;
     char *buf = NULL;
     MPL_pointer_attr_t send_attr, recv_attr;
-    MPIR_CHKLMEM_DECL(1);
+    MPIR_CHKLMEM_DECL();
 
     MPIR_FUNC_ENTER;
 
@@ -160,7 +172,7 @@ static int do_localcopy(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype se
         } else if (MPL_gpu_attr_is_strict_dev(&send_attr) || MPL_gpu_attr_is_strict_dev(&recv_attr)) {
             MPL_gpu_malloc_host((void **) &buf, COPY_BUFFER_SZ);
         } else {
-            MPIR_CHKLMEM_MALLOC(buf, char *, COPY_BUFFER_SZ, mpi_errno, "buf", MPL_MEM_BUFFER);
+            MPIR_CHKLMEM_MALLOC(buf, COPY_BUFFER_SZ);
         }
 
         sfirst = sendoffset;
@@ -226,7 +238,7 @@ static int do_localcopy(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype se
 static int do_localcopy_gpu(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype,
                             MPI_Aint sendoffset, MPL_pointer_attr_t * send_attr, void *recvbuf,
                             MPI_Aint recvcount, MPI_Datatype recvtype, MPI_Aint recvoffset,
-                            MPL_pointer_attr_t * recv_attr, MPL_gpu_copy_direction_t dir,
+                            MPL_pointer_attr_t * recv_attr,
                             MPL_gpu_engine_type_t enginetype, bool commit, MPIR_gpu_req * gpu_req)
 {
     int mpi_errno = MPI_SUCCESS;
@@ -291,13 +303,31 @@ static int do_localcopy_gpu(const void *sendbuf, MPI_Aint sendcount, MPI_Datatyp
             goto fn_exit;
         }
 
-        int fast_copy_threshold = MPIR_CVAR_GPU_FAST_COPY_MAX_SIZE;
+        MPL_gpu_copy_direction_t dir;
+        if (send_attr->type == MPL_GPU_POINTER_DEV) {
+            if (recv_attr->type == MPL_GPU_POINTER_DEV) {
+                dir = MPL_GPU_COPY_D2D_OUTGOING;
+            } else {
+                dir = MPL_GPU_COPY_D2H;
+            }
+        } else if (recv_attr->type == MPL_GPU_POINTER_DEV) {
+            dir = MPL_GPU_COPY_H2D;
+        } else {
+            dir = MPL_GPU_COPY_DIRECTION_NONE;
+        }
+
+        int fast_copy_threshold;
         if (dir == MPL_GPU_COPY_H2D) {
             /* Used in ofi_events.h when unpacking from received pack_buffer to original device buffer */
             fast_copy_threshold = MPIR_CVAR_GPU_FAST_COPY_MAX_SIZE_H2D;
         } else if (dir == MPL_GPU_COPY_D2H) {
             fast_copy_threshold = MPIR_CVAR_GPU_FAST_COPY_MAX_SIZE_D2H;
+        } else if (dir == MPL_GPU_COPY_D2D_OUTGOING || dir == MPL_GPU_COPY_D2D_INCOMING) {
+            fast_copy_threshold = MPIR_CVAR_GPU_FAST_COPY_MAX_SIZE_D2D;
+        } else {
+            fast_copy_threshold = MPIR_CVAR_GPU_FAST_COPY_MAX_SIZE;
         }
+
         if (copy_sz <= fast_copy_threshold) {
             mpl_errno = MPL_gpu_fast_memcpy(send_ptr, send_attr, recv_ptr, recv_attr, copy_sz);
             MPIR_ERR_CHKANDJUMP(mpl_errno != MPL_SUCCESS, mpi_errno, MPI_ERR_OTHER,
@@ -321,6 +351,7 @@ static int do_localcopy_gpu(const void *sendbuf, MPI_Aint sendcount, MPI_Datatyp
             MPIR_ERR_CHKANDJUMP(dev_id == -1, mpi_errno, MPI_ERR_OTHER,
                                 "**mpl_gpu_get_dev_id_from_attr");
 
+            MPID_THREAD_CS_ENTER(VCI, MPIR_THREAD_VCI_GPU_MUTEX);
             if (gpu_req == NULL) {
                 MPL_gpu_request req;
                 mpl_errno = MPL_gpu_imemcpy(recv_ptr, send_ptr, copy_sz, dev_id, dir, enginetype,
@@ -340,6 +371,7 @@ static int do_localcopy_gpu(const void *sendbuf, MPI_Aint sendcount, MPI_Datatyp
                                     "**mpl_gpu_imemcpy");
                 gpu_req->type = MPIR_GPU_REQUEST;
             }
+            MPID_THREAD_CS_EXIT(VCI, MPIR_THREAD_VCI_GPU_MUTEX);
         }
 #else /* !MPL_HAVE_ZE */
         goto fn_fallback;
@@ -383,6 +415,9 @@ int MPIR_Localcopy(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtyp
 
     MPIR_FUNC_ENTER;
 
+    MPIR_DATATYPE_ASSERT_BUILTIN(sendtype);
+    MPIR_DATATYPE_ASSERT_BUILTIN(recvtype);
+
     mpi_errno =
         do_localcopy(sendbuf, sendcount, sendtype, 0, recvbuf, recvcount, recvtype, 0,
                      LOCALCOPY_BLOCKING, NULL);
@@ -403,6 +438,9 @@ int MPIR_Ilocalcopy(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendty
 
     MPIR_FUNC_ENTER;
 
+    MPIR_DATATYPE_ASSERT_BUILTIN(sendtype);
+    MPIR_DATATYPE_ASSERT_BUILTIN(recvtype);
+
     mpi_errno = do_localcopy(sendbuf, sendcount, sendtype, 0, recvbuf, recvcount, recvtype,
                              0, LOCALCOPY_NONBLOCKING, typerep_req);
     MPIR_ERR_CHECK(mpi_errno);
@@ -440,17 +478,19 @@ int MPIR_Localcopy_stream(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype
 int MPIR_Localcopy_gpu(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype,
                        MPI_Aint sendoffset, MPL_pointer_attr_t * sendattr, void *recvbuf,
                        MPI_Aint recvcount, MPI_Datatype recvtype, MPI_Aint recvoffset,
-                       MPL_pointer_attr_t * recvattr, MPL_gpu_copy_direction_t dir,
-                       MPL_gpu_engine_type_t enginetype, bool commit)
+                       MPL_pointer_attr_t * recvattr, MPL_gpu_engine_type_t enginetype, bool commit)
 {
     int mpi_errno = MPI_SUCCESS;
 
     MPIR_FUNC_ENTER;
 
+    MPIR_DATATYPE_ASSERT_BUILTIN(sendtype);
+    MPIR_DATATYPE_ASSERT_BUILTIN(recvtype);
+
 #ifdef MPL_HAVE_GPU
     mpi_errno =
         do_localcopy_gpu(sendbuf, sendcount, sendtype, sendoffset, sendattr, recvbuf, recvcount,
-                         recvtype, recvoffset, recvattr, dir, enginetype, commit, NULL);
+                         recvtype, recvoffset, recvattr, enginetype, commit, NULL);
     MPIR_ERR_CHECK(mpi_errno);
 #else
     mpi_errno =
@@ -469,24 +509,32 @@ int MPIR_Localcopy_gpu(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sen
 int MPIR_Ilocalcopy_gpu(const void *sendbuf, MPI_Aint sendcount, MPI_Datatype sendtype,
                         MPI_Aint sendoffset, MPL_pointer_attr_t * sendattr, void *recvbuf,
                         MPI_Aint recvcount, MPI_Datatype recvtype, MPI_Aint recvoffset,
-                        MPL_pointer_attr_t * recvattr, MPL_gpu_copy_direction_t dir,
+                        MPL_pointer_attr_t * recvattr,
                         MPL_gpu_engine_type_t enginetype, bool commit, MPIR_gpu_req * req)
 {
     int mpi_errno = MPI_SUCCESS;
 
     MPIR_FUNC_ENTER;
 
+    MPIR_DATATYPE_ASSERT_BUILTIN(sendtype);
+    MPIR_DATATYPE_ASSERT_BUILTIN(recvtype);
+
 #ifdef MPL_HAVE_GPU
     mpi_errno =
         do_localcopy_gpu(sendbuf, sendcount, sendtype, sendoffset, sendattr, recvbuf, recvcount,
-                         recvtype, recvoffset, recvattr, dir, enginetype, commit, req);
+                         recvtype, recvoffset, recvattr, enginetype, commit, req);
     MPIR_ERR_CHECK(mpi_errno);
 #else
     mpi_errno =
         do_localcopy(sendbuf, sendcount, sendtype, sendoffset, recvbuf, recvcount, recvtype,
                      recvoffset, LOCALCOPY_NONBLOCKING, &req->u.y_req);
     MPIR_ERR_CHECK(mpi_errno);
-    req->type = MPIR_TYPEREP_REQUEST;
+
+    if (req->u.y_req.req == MPIR_TYPEREP_REQ_NULL) {
+        req->type = MPIR_NULL_REQUEST;
+    } else {
+        req->type = MPIR_TYPEREP_REQUEST;
+    }
 #endif
 
   fn_exit:
