@@ -1,7 +1,6 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2006 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpid_nem_impl.h"
@@ -23,23 +22,18 @@ MPID_nem_cell_ptr_t MPID_nem_prefetched_cell = 0;
 
 unsigned short *MPID_nem_recv_seqno = 0;
 
-#undef FUNCNAME
-#define FUNCNAME MPID_nem_mpich_init
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int
 MPID_nem_mpich_init(void)
 {
     int mpi_errno = MPI_SUCCESS;
     int i;
-    MPIU_CHKPMEM_DECL (2);
-    MPIDI_STATE_DECL(MPID_STATE_MPID_NEM_MPICH_INIT);
+    MPIR_CHKPMEM_DECL();
 
-    MPIDI_FUNC_ENTER(MPID_STATE_MPID_NEM_MPICH_INIT);
+    MPIR_FUNC_ENTER;
 
     MPID_nem_prefetched_cell = NULL;
 
-    MPIU_CHKPMEM_MALLOC (MPID_nem_recv_seqno, unsigned short *, sizeof(*MPID_nem_recv_seqno) * MPID_nem_mem_region.num_procs, mpi_errno, "recv seqno");
+    MPIR_CHKPMEM_MALLOC (MPID_nem_recv_seqno, sizeof(*MPID_nem_recv_seqno) * MPID_nem_mem_region.num_procs, MPL_MEM_SHM);
 
     for (i = 0; i < MPID_nem_mem_region.num_procs; ++i)
     {
@@ -47,7 +41,7 @@ MPID_nem_mpich_init(void)
     }
 
     /* set up fbox queue */
-    MPIU_CHKPMEM_MALLOC (MPID_nem_fboxq_elem_list, MPID_nem_fboxq_elem_t *, MPID_nem_mem_region.num_local * sizeof(MPID_nem_fboxq_elem_t), mpi_errno, "fastbox element list");
+    MPIR_CHKPMEM_MALLOC (MPID_nem_fboxq_elem_list, MPID_nem_mem_region.num_local * sizeof(MPID_nem_fboxq_elem_t), MPL_MEM_SHM);
 
     for (i = 0; i < MPID_nem_mem_region.num_local; ++i)
     {
@@ -55,7 +49,7 @@ MPID_nem_mpich_init(void)
         MPID_nem_fboxq_elem_list[i].prev = NULL;
         MPID_nem_fboxq_elem_list[i].next = NULL;
         MPID_nem_fboxq_elem_list[i].grank = MPID_nem_mem_region.local_procs[i];
-        MPID_nem_fboxq_elem_list[i].fbox = &MPID_nem_mem_region.mailboxes.in[i]->mpich;
+        MPID_nem_fboxq_elem_list[i].fbox = MPID_nem_mem_region.mailboxes.in[i];
     }
 	
     MPID_nem_fboxq_head = NULL;
@@ -64,48 +58,55 @@ MPID_nem_mpich_init(void)
     MPID_nem_curr_fbox_all_poll = &MPID_nem_fboxq_elem_list[0];
     MPID_nem_fboxq_elem_list_last = &MPID_nem_fboxq_elem_list[MPID_nem_mem_region.num_local - 1];
 
-    MPIU_CHKPMEM_COMMIT();
+    MPIR_CHKPMEM_COMMIT();
 fn_exit:
-    MPIDI_FUNC_EXIT(MPID_STATE_MPID_NEM_MPICH_INIT);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
 fn_fail:
     /* --BEGIN ERROR HANDLING-- */
-    MPIU_CHKPMEM_REAP();
+    MPIR_CHKPMEM_REAP();
     goto fn_exit;
     /* --END ERROR HANDLING-- */
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPID_nem_send_iov
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPID_nem_send_iov(MPIDI_VC_t *vc, MPID_Request **sreq_ptr, MPL_IOV *iov, int n_iov)
+int MPID_nem_send_iov(MPIDI_VC_t *vc, MPIR_Request **sreq_ptr, struct iovec *iov, int n_iov)
 {
     int mpi_errno = MPI_SUCCESS;
-    MPIDI_msg_sz_t data_sz;
+    intptr_t data_sz;
     int i;
     int iov_data_copied;
-    MPID_Request *sreq = *sreq_ptr;
-    MPL_IOV *data_iov = &iov[1]; /* iov of just the data, not the header */
+    MPIR_Request *sreq = *sreq_ptr;
+    struct iovec *data_iov = &iov[1]; /* iov of just the data, not the header */
     int data_n_iov = n_iov - 1;
 
-    MPIDI_STATE_DECL(MPID_STATE_MPID_NEM_SEND_IOV);
 
-    MPIDI_FUNC_ENTER(MPID_STATE_MPID_NEM_SEND_IOV);
+    MPIR_FUNC_ENTER;
 
     if (*sreq_ptr == NULL)
     {
 	/* create a request */
-	sreq = MPID_Request_create();
-	MPIU_Assert(sreq != NULL);
-	MPIU_Object_set_ref(sreq, 2);
-	sreq->kind = MPID_REQUEST_SEND;
+	sreq = MPIR_Request_create(MPIR_REQUEST_KIND__UNDEFINED);
+	MPIR_Assert(sreq != NULL);
+	MPIR_Object_set_ref(sreq, 2);
+	sreq->kind = MPIR_REQUEST_KIND__SEND;
         sreq->dev.OnDataAvail = 0;
     }
 
+    /* directly use the iov-based send API if it is provided */
+    if (vc->ch.iSendIov) {
+        MPIR_Assert(n_iov >= 1 && n_iov <= MPL_IOV_LIMIT);
+
+        /* header and remaining iovs */
+        mpi_errno = vc->ch.iSendIov(vc, sreq, iov[0].iov_base, iov[0].iov_len, &iov[1], n_iov - 1);
+        MPIR_ERR_CHECK(mpi_errno);
+
+        *sreq_ptr = sreq;
+        goto fn_exit;
+    }
+
     data_sz = 0;
     for (i = 0; i < data_n_iov; ++i)
-        data_sz += data_iov[i].MPL_IOV_LEN;
+        data_sz += data_iov[i].iov_len;
 
 
     if (!MPIDI_Request_get_srbuf_flag(sreq))
@@ -114,30 +115,30 @@ int MPID_nem_send_iov(MPIDI_VC_t *vc, MPID_Request **sreq_ptr, MPL_IOV *iov, int
         /* --BEGIN ERROR HANDLING-- */
         if (sreq->dev.tmpbuf_sz == 0)
         {
-            MPIU_DBG_MSG(CH3_CHANNEL,TYPICAL,"SRBuf allocation failure");
+            MPL_DBG_MSG(MPIDI_CH3_DBG_CHANNEL,TYPICAL,"SRBuf allocation failure");
             mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL,
-                                             FCNAME, __LINE__, MPI_ERR_OTHER, "**nomem", 0);
+                                             __func__, __LINE__, MPI_ERR_OTHER, "**nomem", 0);
             sreq->status.MPI_ERROR = mpi_errno;
             goto fn_exit;
         }
         /* --END ERROR HANDLING-- */
     }
 
-    MPIU_Assert(sreq->dev.tmpbuf_sz >= data_sz);
+    MPIR_Assert(sreq->dev.tmpbuf_sz >= data_sz);
 
     iov_data_copied = 0;
     for (i = 0; i < data_n_iov; ++i) {
-        MPIU_Memcpy((char*) sreq->dev.tmpbuf + iov_data_copied, data_iov[i].MPL_IOV_BUF, data_iov[i].MPL_IOV_LEN);
-        iov_data_copied += data_iov[i].MPL_IOV_LEN;
+        MPIR_Memcpy((char*) sreq->dev.tmpbuf + iov_data_copied, data_iov[i].iov_base, data_iov[i].iov_len);
+        iov_data_copied += data_iov[i].iov_len;
     }
 
-    mpi_errno = vc->ch.iSendContig(vc, sreq, iov[0].MPL_IOV_BUF, iov[0].MPL_IOV_LEN, sreq->dev.tmpbuf, data_sz);
-    if (mpi_errno) MPIR_ERR_POP(mpi_errno);
+    mpi_errno = vc->ch.iSendContig(vc, sreq, iov[0].iov_base, iov[0].iov_len, sreq->dev.tmpbuf, data_sz);
+    MPIR_ERR_CHECK(mpi_errno);
 
     *sreq_ptr = sreq;
 
  fn_exit:
-    MPIDI_FUNC_EXIT(MPID_STATE_MPID_NEM_SEND_IOV);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
  fn_fail:
     goto fn_exit;
