@@ -1,25 +1,19 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpidimpl.h"
 
-#undef FUNCNAME
-#define FUNCNAME MPID_Cancel_recv
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPID_Cancel_recv(MPID_Request * rreq)
+int MPID_Cancel_recv(MPIR_Request * rreq)
 {
     int netmod_cancelled = TRUE;
     int mpi_errno = MPI_SUCCESS;
 
-    MPIDI_STATE_DECL(MPID_STATE_MPID_CANCEL_RECV);
     
-    MPIDI_FUNC_ENTER(MPID_STATE_MPID_CANCEL_RECV);
+    MPIR_FUNC_ENTER;
     
-    MPIU_Assert(rreq->kind == MPID_REQUEST_RECV);
+    MPIR_Assert(rreq->kind == MPIR_REQUEST_KIND__RECV);
     
     /* If the netmod has its own cancel_recv function, we need to call
        it here. ANYSOURCE cancels (netmod and otherwise) are handled by
@@ -36,23 +30,21 @@ int MPID_Cancel_recv(MPID_Request * rreq)
 
     if (netmod_cancelled && MPIDI_CH3U_Recvq_DP(rreq))
     {
-	MPIU_DBG_MSG_P(CH3_OTHER,VERBOSE,
+	MPL_DBG_MSG_P(MPIDI_CH3_DBG_OTHER,VERBOSE,
 		       "request 0x%08x cancelled", rreq->handle);
         MPIR_STATUS_SET_CANCEL_BIT(rreq->status, TRUE);
         MPIR_STATUS_SET_COUNT(rreq->status, 0);
         mpi_errno = MPID_Request_complete(rreq);
-        if (mpi_errno != MPI_SUCCESS) {
-            MPIR_ERR_POP(mpi_errno);
-        }
+        MPIR_ERR_CHECK(mpi_errno);
     }
     else
     {
-	MPIU_DBG_MSG_P(CH3_OTHER,VERBOSE,
+	MPL_DBG_MSG_P(MPIDI_CH3_DBG_OTHER,VERBOSE,
 	    "request 0x%08x already matched, unable to cancel", rreq->handle);
     }
 
  fn_exit:
-    MPIDI_FUNC_EXIT(MPID_STATE_MPID_CANCEL_RECV);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
  fn_fail:
     goto fn_exit;
