@@ -1,17 +1,26 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
 
 #include "mpidimpl.h"
-#ifdef USE_PMI2_API
-#include "pmi2.h"
-#else
-#include "pmi.h"
-#endif
 
-#define MAX_JOBID_LEN 1024
+/*
+=== BEGIN_MPI_T_CVAR_INFO_BLOCK ===
+
+cvars:
+    - name        : MPIR_CVAR_CH3_PG_VERBOSE
+      category    : CH3
+      type        : boolean
+      default     : 0
+      class       : none
+      verbosity   : MPI_T_VERBOSITY_USER_BASIC
+      scope       : MPI_T_SCOPE_GROUP_EQ
+      description : >-
+        If set, print the PG state on finalize.
+
+=== END_MPI_T_CVAR_INFO_BLOCK ===
+*/
 
 /* FIXME: These routines need a description.  What is their purpose?  Who
    calls them and why?  What does each one do?
@@ -21,59 +30,30 @@ static MPIDI_PG_t * MPIDI_PG_iterator_next = NULL;
 static MPIDI_PG_Compare_ids_fn_t MPIDI_PG_Compare_ids_fn;
 static MPIDI_PG_Destroy_fn_t MPIDI_PG_Destroy_fn;
 
-/* Set verbose to 1 to record changes to the process group structure. */
-static int verbose = 0;
-
 /* Key track of the process group corresponding to the MPI_COMM_WORLD 
    of this process */
 static MPIDI_PG_t *pg_world = NULL;
 
 #define MPIDI_MAX_KVS_KEY_LEN      256
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Init
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
-int MPIDI_PG_Init(int *argc_p, char ***argv_p, 
-		  MPIDI_PG_Compare_ids_fn_t compare_ids_fn, 
+int MPIDI_PG_Init(MPIDI_PG_Compare_ids_fn_t compare_ids_fn, 
 		  MPIDI_PG_Destroy_fn_t destroy_fn)
 {
     int mpi_errno = MPI_SUCCESS;
-    char *p;
     
     MPIDI_PG_Compare_ids_fn = compare_ids_fn;
     MPIDI_PG_Destroy_fn     = destroy_fn;
 
-    /* Check for debugging options.  We use MPICHD_DBG and -mpichd-dbg 
-       to avoid confusion with the code in src/util/dbg/dbg_printf.c */
-    p = getenv( "MPICHD_DBG_PG" );
-    if (p && ( strcmp( p, "YES" ) == 0 || strcmp( p, "yes" ) == 0) )
-	verbose = 1;
-    if (argc_p && argv_p) {
-	int argc = *argc_p, i;
-	char **argv = *argv_p;
-        /* applied patch from Juha Jeronen, req #3920 */
-	for (i=1; i<argc && argv[i]; i++) {
-	    if (strcmp( "-mpichd-dbg-pg", argv[i] ) == 0) {
-		int j;
-		verbose = 1;
-		for (j=i; j<argc-1; j++) {
-		    argv[j] = argv[j+1];
-		}
-		argv[argc-1] = NULL;
-		*argc_p = argc - 1;
-		break;
-	    }
-	}
+    /* initialize the device fields in builtin groups */
+#ifdef MPID_DEV_GROUP_DECL
+    for (int i = 0; i < MPIR_GROUP_N_BUILTIN; i++) {
+        MPID_Group_init_hook(MPIR_Group_builtin + i);
     }
-
+#endif
     return mpi_errno;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Finalize
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
+extern int MPIDI_lpid_counter;
 /*@ 
    MPIDI_PG_Finalize - Finalize the process groups, including freeing all
    process group structures
@@ -82,31 +62,20 @@ int MPIDI_PG_Finalize(void)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIDI_PG_t *pg, *pgNext;
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_PG_FINALIZE);
 
-    MPIDI_FUNC_ENTER(MPID_STATE_MPIDI_PG_FINALIZE);
+    MPIR_FUNC_ENTER;
 
     /* Print the state of the process groups */
-    if (verbose) {
+    if (MPIR_CVAR_CH3_PG_VERBOSE) {
 	MPIU_PG_Printall( stdout );
     }
 
-    /* FIXME - straighten out the use of PMI_Finalize - no use after 
-       PG_Finalize */
-    if (pg_world->connData) {
-#ifdef USE_PMI2_API
-        mpi_errno = PMI2_Finalize();
-        if (mpi_errno) MPIR_ERR_SET(mpi_errno, MPI_ERR_OTHER, "**ch3|pmi_finalize");
-#else
-	int rc;
-	rc = PMI_Finalize();
-	if (rc) {
-	    MPIR_ERR_SET1(mpi_errno,MPI_ERR_OTHER, 
-			  "**ch3|pmi_finalize", 
-			  "**ch3|pmi_finalize %d", rc);
-	}
-#endif
+    /* release the vcrt in builtin groups, since they don't really get freed */
+#ifdef MPID_DEV_GROUP_DECL
+    for (int i = 0; i < MPIR_GROUP_N_BUILTIN; i++) {
+        MPID_Group_free_hook(MPIR_Group_builtin + i);
     }
+#endif
 
     /* Free the storage associated with the process groups */
     pg = MPIDI_PG_list;
@@ -118,11 +87,11 @@ int MPIDI_PG_Finalize(void)
 	   fails to use MPI_Comm_disconnect on communicators that
 	   were created with the dynamic process routines.*/
         /* XXX DJG FIXME-MT should we be checking this? */
-	if (MPIU_Object_get_ref(pg) == 0 || 1) {
+	if (MPIR_Object_get_ref(pg) == 0 || 1) {
 	    if (pg == MPIDI_Process.my_pg)
 		MPIDI_Process.my_pg = NULL;
 
-	    MPIU_Object_set_ref(pg, 0); /* satisfy assertions in PG_Destroy */
+	    MPIR_Object_set_ref(pg, 0); /* satisfy assertions in PG_Destroy */
 	    MPIDI_PG_Destroy( pg );
 	}
 	pg     = pgNext;
@@ -137,6 +106,7 @@ int MPIDI_PG_Finalize(void)
 	MPIDI_PG_Destroy(MPIDI_Process.my_pg);
     } 
     MPIDI_Process.my_pg = NULL;
+    MPIDI_lpid_counter = 0;
 
     /* ifdefing out this check because the list will not be NULL in 
        Ch3_finalize because
@@ -149,13 +119,13 @@ int MPIDI_PG_Finalize(void)
     { 
 	
 	/* --BEGIN ERROR HANDLING-- */
-	mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL, FCNAME, __LINE__, MPI_ERR_INTERN,
+	mpi_errno = MPIR_Err_create_code(MPI_SUCCESS, MPIR_ERR_FATAL, __func__, __LINE__, MPI_ERR_INTERN,
         "**dev|pg_finalize|list_not_empty", NULL); 
 	/* --END ERROR HANDLING-- */
     }
 #endif
 
-    MPIDI_FUNC_EXIT(MPID_STATE_MPIDI_PG_FINALIZE);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
 }
 
@@ -170,25 +140,19 @@ int MPIDI_PG_Finalize(void)
 
    The new process group is returned in pg_ptr 
 */
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Create
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_Create(int vct_sz, void * pg_id, MPIDI_PG_t ** pg_ptr)
 {
     MPIDI_PG_t * pg = NULL, *pgnext;
     int p;
     int mpi_errno = MPI_SUCCESS;
-    MPIU_CHKPMEM_DECL(2);
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_PG_CREATE);
+    MPIR_CHKPMEM_DECL();
 
-    MPIDI_FUNC_ENTER(MPID_STATE_MPIDI_PG_CREATE);
+    MPIR_FUNC_ENTER;
     
-    MPIU_CHKPMEM_MALLOC(pg,MPIDI_PG_t*,sizeof(MPIDI_PG_t),mpi_errno,"pg");
-    MPIU_CHKPMEM_MALLOC(pg->vct,MPIDI_VC_t *,sizeof(MPIDI_VC_t)*vct_sz,
-			mpi_errno,"pg->vct");
+    MPIR_CHKPMEM_MALLOC(pg, sizeof(MPIDI_PG_t), MPL_MEM_GROUP);
+    MPIR_CHKPMEM_MALLOC(pg->vct, sizeof(MPIDI_VC_t)*vct_sz, MPL_MEM_GROUP);
 
-    if (verbose) {
+    if (MPIR_CVAR_CH3_PG_VERBOSE) {
 	fprintf( stdout, "Creating a process group of size %d\n", vct_sz );
 	fflush(stdout);
     }
@@ -197,7 +161,7 @@ int MPIDI_PG_Create(int vct_sz, void * pg_id, MPIDI_PG_t ** pg_ptr)
     /* The reference count indicates the number of vc's that are or 
        have been in use and not disconnected. It starts at zero,
        except for MPI_COMM_WORLD. */
-    MPIU_Object_set_ref(pg, 0);
+    MPIR_Object_set_ref(pg, 0);
     pg->size = vct_sz;
     pg->id   = pg_id;
     pg->finalize = 0;
@@ -251,29 +215,24 @@ int MPIDI_PG_Create(int vct_sz, void * pg_id, MPIDI_PG_t ** pg_ptr)
     *pg_ptr = pg;
     
   fn_exit:
-    MPIDI_FUNC_EXIT(MPID_STATE_MPIDI_PG_CREATE);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
     
   fn_fail:
-    MPIU_CHKPMEM_REAP();
+    MPIR_CHKPMEM_REAP();
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Destroy
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_Destroy(MPIDI_PG_t * pg)
 {
     MPIDI_PG_t * pg_prev;
     MPIDI_PG_t * pg_cur;
     int i;
     int mpi_errno = MPI_SUCCESS;
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_PG_DESTROY);
 
-    MPIDI_FUNC_ENTER(MPID_STATE_MPIDI_PG_DESTROY);
+    MPIR_FUNC_ENTER;
 
-    MPIU_Assert(MPIU_Object_get_ref(pg) == 0);
+    MPIR_Assert(MPIR_Object_get_ref(pg) == 0);
 
     pg_prev = NULL;
     pg_cur = MPIDI_PG_list;
@@ -291,7 +250,7 @@ int MPIDI_PG_Destroy(MPIDI_PG_t * pg)
             else
                 pg_prev->next = pg->next;
 
-            MPIU_DBG_MSG_FMT(CH3_DISCONNECT, VERBOSE, (MPIU_DBG_FDEST, "destroying pg=%p pg->id=%s", pg, (char *)pg->id));
+            MPL_DBG_MSG_FMT(MPIDI_CH3_DBG_DISCONNECT, VERBOSE, (MPL_DBG_FDEST, "destroying pg=%p pg->id=%s", pg, (char *)pg->id));
 
             for (i = 0; i < pg->size; ++i) {
                 /* FIXME it would be good if we could make this assertion.
@@ -304,9 +263,9 @@ int MPIDI_PG_Destroy(MPIDI_PG_t * pg)
 		      counts IS required and missing one is a bug.)
                    2) There is a real bug lurking out there somewhere and we
                       just haven't hit it in the tests yet.  */
-                /*MPIU_Assert(MPIU_Object_get_ref(pg->vct[i]) == 0);*/
+                /*MPIR_Assert(MPIR_Object_get_ref(pg->vct[i]) == 0);*/
 
-                MPIU_DBG_MSG_FMT(CH3_DISCONNECT, VERBOSE, (MPIU_DBG_FDEST, "about to free pg->vct=%p which contains vc=%p", pg->vct, &pg->vct[i]));
+                MPL_DBG_MSG_FMT(MPIDI_CH3_DBG_DISCONNECT, VERBOSE, (MPL_DBG_FDEST, "about to free pg->vct=%p which contains vc=%p", pg->vct, &pg->vct[i]));
 
                 /* This used to be handled in MPIDI_VCRT_Release, but that was
                    not the right place to do this.  The VC should only be freed
@@ -323,17 +282,17 @@ int MPIDI_PG_Destroy(MPIDI_PG_t * pg)
             }
 
 	    MPIDI_PG_Destroy_fn(pg);
-	    MPIU_Free(pg->vct);
+	    MPL_free(pg->vct);
 	    if (pg->connData) {
 		if (pg->freeConnInfo) {
 		    (*pg->freeConnInfo)( pg );
 		}
 		else {
-		    MPIU_Free(pg->connData);
+		    MPL_free(pg->connData);
 		}
 	    }
 	    mpi_errno = MPIDI_CH3_PG_Destroy(pg);
-	    MPIU_Free(pg);
+	    MPL_free(pg);
 
 	    goto fn_exit;
 	}
@@ -347,23 +306,18 @@ int MPIDI_PG_Destroy(MPIDI_PG_t * pg)
 		  "**dev|pg_not_found", "**dev|pg_not_found %p", pg);
 
   fn_exit:
-    MPIDI_FUNC_EXIT(MPID_STATE_MPIDI_PG_DESTROY);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
     goto fn_exit;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Find
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_Find(void * id, MPIDI_PG_t ** pg_ptr)
 {
     MPIDI_PG_t * pg;
     int mpi_errno = MPI_SUCCESS;
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_PG_FIND);
 
-    MPIDI_FUNC_ENTER(MPID_STATE_MPIDI_PG_FIND);
+    MPIR_FUNC_ENTER;
     
     pg = MPIDI_PG_list;
     while (pg != NULL)
@@ -380,25 +334,17 @@ int MPIDI_PG_Find(void * id, MPIDI_PG_t ** pg_ptr)
     *pg_ptr = NULL;
 
   fn_exit:
-    MPIDI_FUNC_EXIT(MPID_STATE_MPIDI_PG_FIND);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Id_compare
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_Id_compare(void * id1, void *id2)
 {
     return MPIDI_PG_Compare_ids_fn(id1, id2);
 }
 
 /* iter always points at the next element */
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Get_next
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_Get_next(MPIDI_PG_iterator *iter, MPIDI_PG_t ** pg_ptr)
 {
     *pg_ptr = (*iter);
@@ -409,19 +355,11 @@ int MPIDI_PG_Get_next(MPIDI_PG_iterator *iter, MPIDI_PG_t ** pg_ptr)
     return MPI_SUCCESS;
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Has_next
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_Has_next(MPIDI_PG_iterator *iter)
 {
     return (*iter != NULL);
 }
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Get_iterator
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_Get_iterator(MPIDI_PG_iterator *iter)
 {
     *iter = MPIDI_PG_list;
@@ -437,16 +375,11 @@ int MPIDI_PG_Get_iterator(MPIDI_PG_iterator *iter)
    hence in spawn) in ch3u_port.c */
 /* Note: Allocated memory that is returned in str_ptr.  The user of
    this routine must free that data */
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_To_string
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_To_string(MPIDI_PG_t *pg_ptr, char **str_ptr, int *lenStr)
 {
     int mpi_errno = MPI_SUCCESS;
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_PG_TO_STRING);
 
-    MPIDI_FUNC_ENTER(MPID_STATE_MPIDI_PG_TO_STRING);
+    MPIR_FUNC_ENTER;
 
     /* Replace this with the new string */
     if (pg_ptr->connInfoToString) {
@@ -458,7 +391,7 @@ int MPIDI_PG_To_string(MPIDI_PG_t *pg_ptr, char **str_ptr, int *lenStr)
 
     /*printf( "PgToString: Pg string is %s\n", *str_ptr ); fflush(stdout);*/
 fn_exit:
-    MPIDI_FUNC_EXIT(MPID_STATE_MPIDI_PG_TO_STRING);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
 fn_fail:
     goto fn_exit;
@@ -472,10 +405,6 @@ fn_fail:
    true.  In addition, the connection information is set up using the 
    information in the input string.
 */
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Create_from_string
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_Create_from_string(const char * str, MPIDI_PG_t ** pg_pptr, 
 				int *flag)
 {
@@ -483,9 +412,8 @@ int MPIDI_PG_Create_from_string(const char * str, MPIDI_PG_t ** pg_pptr,
     const char *p;
     int vct_sz;
     MPIDI_PG_t *existing_pg, *pg_ptr=0;
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_PG_CREATE_FROM_STRING);
 
-    MPIDI_FUNC_ENTER(MPID_STATE_MPIDI_PG_CREATE_FROM_STRING);
+    MPIR_FUNC_ENTER;
 
     /*printf( "PgCreateFromString: Creating pg from %s\n", str ); 
       fflush(stdout); */
@@ -493,7 +421,7 @@ int MPIDI_PG_Create_from_string(const char * str, MPIDI_PG_t ** pg_pptr,
        it to the find routine */
     /* printf( "Looking for pg with id %s\n", str );fflush(stdout); */
     mpi_errno = MPIDI_PG_Find((void *)str, &existing_pg);
-    if (mpi_errno) MPIR_ERR_POP(mpi_errno);
+    MPIR_ERR_CHECK(mpi_errno);
 
     if (existing_pg != NULL) {
 	/* return the existing PG */
@@ -506,23 +434,22 @@ int MPIDI_PG_Create_from_string(const char * str, MPIDI_PG_t ** pg_pptr,
 
     /* Get the size from the string */
     p = str;
-    while (*p) p++; p++;
+    while (*p) p++;
+    p++;
     vct_sz = atoi(p);
 
     mpi_errno = MPIDI_PG_Create(vct_sz, (void *)str, pg_pptr);
-    if (mpi_errno != MPI_SUCCESS) {
-	MPIR_ERR_POP(mpi_errno);
-    }
+    MPIR_ERR_CHECK(mpi_errno);
     
     pg_ptr = *pg_pptr;
-    pg_ptr->id = MPIU_Strdup( str );
+    pg_ptr->id = MPL_strdup( str );
     
     /* Set up the functions to use strings to manage connection information */
     MPIDI_PG_InitConnString( pg_ptr );
     (*pg_ptr->connInfoFromString)( str, pg_ptr );
 
 fn_exit:
-    MPIDI_FUNC_EXIT(MPID_STATE_MPIDI_PG_CREATE_FROM_STRING);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
 fn_fail:
     goto fn_exit;
@@ -545,7 +472,7 @@ fn_fail:
 void MPIDI_PG_IdToNum( MPIDI_PG_t *pg, int *id )
 {
     const char *p = (const char *)pg->id;
-    int pgid = 0;
+    unsigned int pgid = 0;
 
     while (*p) {
         pgid += *p++;
@@ -581,72 +508,26 @@ void MPIDI_PG_IdToNum( MPIDI_PG_t *pg, int *id )
    This is a collective call (for scalability) over all of the processes in 
    the same MPI_COMM_WORLD.
 */
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_SetConnInfo
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_SetConnInfo( int rank, const char *connString )
 {
-#ifdef USE_PMI2_API
     int mpi_errno = MPI_SUCCESS;
+
     int len;
-    char key[PMI2_MAX_KEYLEN];
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_PG_SetConnInfo);
-
-    MPIDI_FUNC_ENTER(MPID_STATE_MPIDI_PG_SetConnInfo);
-
-    len = MPL_snprintf(key, sizeof(key), "P%d-businesscard", rank);
+    char key[40];
+    len = snprintf(key, sizeof(key), "P%d-businesscard", rank);
     MPIR_ERR_CHKANDJUMP1(len < 0 || len > sizeof(key), mpi_errno, MPI_ERR_OTHER, "**snprintf", "**snprintf %d", len);
 
-    mpi_errno = PMI2_KVS_Put(key, connString);
-    if (mpi_errno) MPIR_ERR_POP(mpi_errno);
+    mpi_errno = MPIR_pmi_kvs_put(key, connString);
+    MPIR_ERR_CHECK(mpi_errno);
+
+    mpi_errno = MPIR_pmi_barrier();
+    MPIR_ERR_CHECK(mpi_errno);
 
-    mpi_errno = PMI2_KVS_Fence();
-    if (mpi_errno) MPIR_ERR_POP(mpi_errno);
-    
  fn_exit:
-    MPIDI_FUNC_EXIT(MPID_STATE_MPIDI_PG_SetConnInfo);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
  fn_fail:
     goto fn_exit;
-#else
-    int mpi_errno = MPI_SUCCESS;
-    int pmi_errno;
-    int len;
-    char key[128];
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_PG_SetConnInfo);
-
-    MPIDI_FUNC_ENTER(MPID_STATE_MPIDI_PG_SetConnInfo);
-
-    MPIU_Assert(pg_world->connData);
-    
-    len = MPL_snprintf(key, sizeof(key), "P%d-businesscard", rank);
-    if (len < 0 || len > sizeof(key)) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, "**snprintf",
-			     "**snprintf %d", len);
-    }
-    pmi_errno = PMI_KVS_Put(pg_world->connData, key, connString );
-    if (pmi_errno != PMI_SUCCESS) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, "**pmi_kvs_put",
-			     "**pmi_kvs_put %d", pmi_errno);
-    }
-    pmi_errno = PMI_KVS_Commit(pg_world->connData);
-    if (pmi_errno != PMI_SUCCESS) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, "**pmi_kvs_commit",
-			     "**pmi_kvs_commit %d", pmi_errno);
-    }
-    
-    pmi_errno = PMI_Barrier();
-    if (pmi_errno != PMI_SUCCESS) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, "**pmi_barrier",
-			     "**pmi_barrier %d", pmi_errno);
-    }
- fn_exit:
-    MPIDI_FUNC_EXIT(MPID_STATE_MPIDI_PG_SetConnInfo);
-    return mpi_errno;
- fn_fail:
-    goto fn_exit;
-#endif
 }
 
 /* For all of these routines, the format of the process group description
@@ -671,57 +552,22 @@ int MPIDI_PG_SetConnInfo( int rank, const char *connString )
 /* The "KVS" versions are for the process group to which the calling 
    process belongs.  These use the PMI_KVS routines to access the
    process information */
-#undef FUNCNAME
-#define FUNCNAME getConnInfoKVS
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static int getConnInfoKVS( int rank, char *buf, int bufsize, MPIDI_PG_t *pg )
 {
-#ifdef USE_PMI2_API
-    char key[MPIDI_MAX_KVS_KEY_LEN];
-    int  mpi_errno = MPI_SUCCESS, rc;
-    int vallen;
+    int mpi_errno = MPI_SUCCESS;
 
-    rc = MPL_snprintf(key, MPIDI_MAX_KVS_KEY_LEN, "P%d-businesscard", rank );
-    if (rc < 0 || rc > MPIDI_MAX_KVS_KEY_LEN) {
-	MPIR_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER,"**nomem");
-    }
+    int len;
+    char key[40];
+    len = snprintf(key, sizeof(key), "P%d-businesscard", rank);
+    MPIR_ERR_CHKANDJUMP1(len < 0 || len > sizeof(key), mpi_errno, MPI_ERR_OTHER, "**snprintf", "**snprintf %d", len);
 
-    mpi_errno = PMI2_KVS_Get(pg->connData, PMI2_ID_NULL, key, buf, bufsize, &vallen);
-    if (mpi_errno) {
-	MPIDI_PG_CheckForSingleton();
-	mpi_errno = PMI2_KVS_Get(pg->connData, PMI2_ID_NULL, key, buf, bufsize, &vallen);
-        if (mpi_errno) MPIR_ERR_POP(mpi_errno);
-    }
- fn_exit:
-    return mpi_errno;
- fn_fail:
-    goto fn_exit;
-#else
-    char key[MPIDI_MAX_KVS_KEY_LEN];
-    int  mpi_errno = MPI_SUCCESS, rc, pmi_errno;
-
-    rc = MPL_snprintf(key, MPIDI_MAX_KVS_KEY_LEN, "P%d-businesscard", rank );
-    if (rc < 0 || rc > MPIDI_MAX_KVS_KEY_LEN) {
-	MPIR_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER,"**nomem");
-    }
-
-    MPID_THREAD_CS_ENTER(POBJ, MPIR_THREAD_POBJ_PMI_MUTEX);
-    pmi_errno = PMI_KVS_Get(pg->connData, key, buf, bufsize );
-    if (pmi_errno) {
-	MPIDI_PG_CheckForSingleton();
-	pmi_errno = PMI_KVS_Get(pg->connData, key, buf, bufsize );
-    }
-    MPID_THREAD_CS_EXIT(POBJ, MPIR_THREAD_POBJ_PMI_MUTEX);
-    if (pmi_errno) {
-	MPIR_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER,"**pmi_kvs_get");
-    }
+    mpi_errno = MPIR_pmi_kvs_get(rank, key, buf, bufsize);
+    MPIR_ERR_CHECK(mpi_errno);
 
  fn_exit:
     return mpi_errno;
  fn_fail:
     goto fn_exit;
-#endif
 }
 
 /* *slen is the length of the string, including the null terminator.  So if the
@@ -739,7 +585,7 @@ static int connToStringKVS( char **buf_p, int *slen, MPIDI_PG_t *pg )
        needed space */
     len = 0;
     curSlen = 10 + pg->size * 128;
-    string = (char *)MPIU_Malloc( curSlen );
+    string = (char *)MPL_malloc( curSlen, MPL_MEM_STRINGS );
 
     /* Start with the id of the pg */
     while (*pg_idStr && len < curSlen) 
@@ -747,7 +593,7 @@ static int connToStringKVS( char **buf_p, int *slen, MPIDI_PG_t *pg )
     string[len++] = 0;
     
     /* Add the size of the pg */
-    MPL_snprintf( &string[len], curSlen - len, "%d", pg->size );
+    snprintf( &string[len], curSlen - len, "%d", pg->size );
     while (string[len]) len++;
     len++;
 
@@ -783,7 +629,7 @@ static int connToStringKVS( char **buf_p, int *slen, MPIDI_PG_t *pg )
 	if (len + vallen + 1 >= curSlen) {
 	    char *nstring = 0;
             curSlen += (pg->size - i) * (vallen + 1 );
-	    nstring = MPIU_Realloc( string, curSlen );
+	    nstring = MPL_realloc( string, curSlen, MPL_MEM_STRINGS );
 	    if (!nstring) {
 		MPIR_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER,"**nomem");
 	    }
@@ -795,14 +641,14 @@ static int connToStringKVS( char **buf_p, int *slen, MPIDI_PG_t *pg )
 	}
     }
 
-    MPIU_Assert(len <= curSlen);
+    MPIR_Assert(len <= curSlen);
 
     *buf_p = string;
     *slen  = len;
  fn_exit:
     return mpi_errno;
  fn_fail:
-    if (string) MPIU_Free(string);
+    MPL_free(string);
     goto fn_exit;
 }
 static int connFromStringKVS( const char *buf ATTRIBUTE((unused)), 
@@ -813,62 +659,22 @@ static int connFromStringKVS( const char *buf ATTRIBUTE((unused)),
 }
 static int connFreeKVS( MPIDI_PG_t *pg )
 {
-    if (pg->connData) {
-	MPIU_Free( pg->connData );
-    }
+    MPL_free( pg->connData );
     return MPI_SUCCESS;
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_InitConnKVS
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_InitConnKVS( MPIDI_PG_t *pg )
 {
-#ifdef USE_PMI2_API
-    int mpi_errno = MPI_SUCCESS;
-    
-    pg->connData = (char *)MPIU_Malloc(MAX_JOBID_LEN);
-    if (pg->connData == NULL) {
-	MPIR_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER, "**nomem");
-    }
-    
-    mpi_errno = PMI2_Job_GetId(pg->connData, MAX_JOBID_LEN);
-    if (mpi_errno) MPIR_ERR_POP(mpi_errno);
-#else
-    int pmi_errno, kvs_name_sz;
     int mpi_errno = MPI_SUCCESS;
 
-    pmi_errno = PMI_KVS_Get_name_length_max( &kvs_name_sz );
-    if (pmi_errno != PMI_SUCCESS) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, 
-			     "**pmi_kvs_get_name_length_max", 
-			     "**pmi_kvs_get_name_length_max %d", pmi_errno);
-    }
-    
-    pg->connData = (char *)MPIU_Malloc(kvs_name_sz + 1);
-    if (pg->connData == NULL) {
-	MPIR_ERR_SETANDJUMP(mpi_errno,MPI_ERR_OTHER, "**nomem");
-    }
-    
-    pmi_errno = PMI_KVS_Get_my_name(pg->connData, kvs_name_sz);
-    if (pmi_errno != PMI_SUCCESS) {
-	MPIR_ERR_SETANDJUMP1(mpi_errno,MPI_ERR_OTHER, 
-			     "**pmi_kvs_get_my_name", 
-			     "**pmi_kvs_get_my_name %d", pmi_errno);
-    }
-#endif
+    pg->connData = MPL_strdup(MPIR_pmi_job_id());
     pg->getConnInfo        = getConnInfoKVS;
     pg->connInfoToString   = connToStringKVS;
     pg->connInfoFromString = connFromStringKVS;
     pg->freeConnInfo       = connFreeKVS;
 
- fn_exit:
     return mpi_errno;
- fn_fail:
-    if (pg->connData) { MPIU_Free(pg->connData); }
-    goto fn_exit;
 }
 
 /* Return the kvsname associated with the MPI_COMM_WORLD of this process. */
@@ -905,48 +711,31 @@ static int getConnInfo( int rank, char *buf, int bufsize, MPIDI_PG_t *pg )
 
     /* printf( "Copying %s to buf\n", connInfo->connStrings[rank] ); fflush(stdout); */
     
-    MPIU_Strncpy( buf, connInfo->connStrings[rank], bufsize );
+    MPL_strncpy( buf, connInfo->connStrings[rank], bufsize );
     return MPI_SUCCESS;
 }
 
-#undef FUNCNAME
-#define FUNCNAME connToString
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 static int connToString( char **buf_p, int *slen, MPIDI_PG_t *pg )
 {
     int mpi_errno = MPI_SUCCESS;
     char *str = NULL, *pg_id;
     int  i, len=0;
-    MPIU_CHKPMEM_DECL(1);
+    MPIR_CHKPMEM_DECL();
     MPIDI_ConnInfo *connInfo = (MPIDI_ConnInfo *)pg->connData;
 
     /* Create this from the string array */
-    MPIU_CHKPMEM_MALLOC(str, char *, connInfo->toStringLen, mpi_errno, "str");
+    MPIR_CHKPMEM_MALLOC(str, connInfo->toStringLen, MPL_MEM_STRINGS);
 
 #if defined(MPICH_DEBUG_MEMINIT)
     memset(str, 0, connInfo->toStringLen);
 #endif
 
     pg_id = pg->id;
-    /* FIXME: This is a hack, and it doesn't even work */
-    /*    MPIDI_PrintConnStrToFile( stdout, __FILE__, __LINE__, 
-	  "connToString: pg id is", (char *)pg_id );*/
-    /* This is intended to cause a process to transition from a singleton
-       to a non-singleton. */
-    /* XXX DJG TODO figure out what this little bit is all about. */
-    if (strstr( pg_id, "singinit_kvs" ) == pg_id) {
-#ifdef USE_PMI2_API
-        MPIU_Assertp(0); /* don't know what to do here for pmi2 yet.  DARIUS */
-#else
-	PMI_KVS_Get_my_name( pg->id, 256 );
-#endif
-    }
     
     while (*pg_id) str[len++] = *pg_id++;
     str[len++] = 0;
     
-    MPL_snprintf( &str[len], 20, "%d", pg->size);
+    snprintf( &str[len], 20, "%d", pg->size);
     /* Skip over the length */
     while (str[len++]);
 
@@ -967,10 +756,10 @@ static int connToString( char **buf_p, int *slen, MPIDI_PG_t *pg )
     *slen = len;
 
 fn_exit:
-    MPIU_CHKPMEM_COMMIT();
+    MPIR_CHKPMEM_COMMIT();
     return mpi_errno;
 fn_fail:
-    MPIU_CHKPMEM_REAP();
+    MPIR_CHKPMEM_REAP();
     goto fn_exit;
     
 }
@@ -983,19 +772,21 @@ static int connFromString( const char *buf, MPIDI_PG_t *pg )
     /* printf( "Starting with buf = %s\n", buf );fflush(stdout); */
 
     /* Skip the pg id */
-    while (*buf) buf++; buf++;
+    while (*buf) buf++;
+    buf++;
 
     /* Determine the size of the pg */
     pg->size = atoi( buf );
-    while (*buf) buf++; buf++;
+    while (*buf) buf++;
+    buf++;
 
-    conninfo = (MPIDI_ConnInfo *)MPIU_Malloc( sizeof(MPIDI_ConnInfo) );
-    conninfo->connStrings = (char **)MPIU_Malloc( pg->size * sizeof(char *));
+    conninfo = (MPIDI_ConnInfo *)MPL_malloc( sizeof(MPIDI_ConnInfo), MPL_MEM_OTHER );
+    conninfo->connStrings = (char **)MPL_malloc( pg->size * sizeof(char *), MPL_MEM_STRINGS);
 
     /* For now, make a copy of each item */
     for (i=0; i<pg->size; i++) {
 	/* printf( "Adding conn[%d] = %s\n", i, buf );fflush(stdout); */
-	conninfo->connStrings[i] = MPIU_Strdup( buf );
+	conninfo->connStrings[i] = MPL_strdup( buf );
 	while (*buf) buf++;
 	buf++;
     }
@@ -1013,15 +804,15 @@ static int connFree( MPIDI_PG_t *pg )
     int i;
 
     for (i=0; i<pg->size; i++) {
-	MPIU_Free( conninfo->connStrings[i] );
+	MPL_free( conninfo->connStrings[i] );
     }
-    MPIU_Free( conninfo->connStrings );
-    MPIU_Free( conninfo );
+    MPL_free( conninfo->connStrings );
+    MPL_free( conninfo );
 
     return MPI_SUCCESS;
 }
 
-#ifdef USE_DBG_LOGGING
+#ifdef MPL_USE_DBG_LOGGING
 /* This is a temporary routine that is used to print out the pg string.
    A better approach may be to convert it into a single (long) string
    with no nulls. */
@@ -1030,18 +821,20 @@ int MPIDI_PrintConnStr( const char *file, int line,
 {
     int pg_size, i;
 
-    MPIU_DBG_Outevent( file, line, MPIU_DBG_CH3_CONNECT, 0, "%s", label );
-    MPIU_DBG_Outevent( file, line, MPIU_DBG_CH3_CONNECT, 0, "%s", str );
+    MPL_dbg_outevent( file, line, MPIDI_CH3_DBG_CONNECT, 0, "%s", label );
+    MPL_dbg_outevent( file, line, MPIDI_CH3_DBG_CONNECT, 0, "%s", str );
     
     /* Skip the pg id */
-    while (*str) str++; str++;
+    while (*str) str++;
+    str++;
 
     /* Determine the size of the pg */
     pg_size = atoi( str );
-    while (*str) str++; str++;
+    while (*str) str++;
+    str++;
 
     for (i=0; i<pg_size; i++) {
-	MPIU_DBG_Outevent( file, line, MPIU_DBG_CH3_CONNECT, 0, "%s", str );
+	MPL_dbg_outevent( file, line, MPIDI_CH3_DBG_CONNECT, 0, "%s", str );
 	while (*str) str++;
 	str++;
     }
@@ -1055,12 +848,14 @@ int MPIDI_PrintConnStrToFile( FILE *fd, const char *file, int line,
     fprintf( fd, "ConnStr from %s(%d); %s\n\t%s\n", file, line, label, str );
     
     /* Skip the pg id */
-    while (*str) str++; str++;
+    while (*str) str++;
+    str++;
 
     fprintf( fd, "\t%s\n", str );
     /* Determine the size of the pg */
     pg_size = atoi( str );
-    while (*str) str++; str++;
+    while (*str) str++;
+    str++;
 
     for (i=0; i<pg_size; i++) {
 	fprintf( fd, "\t%s\n", str ); 
@@ -1086,10 +881,6 @@ int MPIDI_PG_InitConnString( MPIDI_PG_t *pg )
 }
 
 /* Temp to get connection value for rank r */
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_GetConnString
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 int MPIDI_PG_GetConnString( MPIDI_PG_t *pg, int rank, char *val, int vallen )
 {
     int mpi_errno = MPI_SUCCESS;
@@ -1105,10 +896,6 @@ int MPIDI_PG_GetConnString( MPIDI_PG_t *pg, int rank, char *val, int vallen )
 }
 
 
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Dup_vcr
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 /*@
   MPIDI_PG_Dup_vcr - Duplicate a virtual connection from a process group
 
@@ -1126,33 +913,28 @@ int MPIDI_PG_GetConnString( MPIDI_PG_t *pg, int rank, char *val, int vallen )
 int MPIDI_PG_Dup_vcr( MPIDI_PG_t *pg, int rank, MPIDI_VC_t **vc_p )
 {
     MPIDI_VC_t *vc;
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_PG_DUP_VCR);
 
-    MPIDI_FUNC_ENTER(MPID_STATE_MPIDI_PG_DUP_VCR);
+    MPIR_FUNC_ENTER;
 
     vc = &pg->vct[rank];
     /* Increase the reference count of the vc.  If the reference count 
        increases from 0 to 1, increase the reference count of the 
        process group *and* the reference count of the vc (this
-       allows us to distinquish between Comm_free and Comm_disconnect) */
+       allows us to distinguish between Comm_free and Comm_disconnect) */
     /* FIXME-MT: This should be a fetch and increment for thread-safety */
-    if (MPIU_Object_get_ref(vc) == 0) {
+    if (MPIR_Object_get_ref(vc) == 0) {
 	MPIDI_PG_add_ref(pg);
 	MPIDI_VC_add_ref(vc);
     }
     MPIDI_VC_add_ref(vc);
     *vc_p = vc;
 
-    MPIDI_FUNC_EXIT(MPID_STATE_MPIDI_PG_DUP_VCR);
+    MPIR_FUNC_EXIT;
     return MPI_SUCCESS;
 }
 
 /* FIXME: This routine should invoke a close method on the connection,
    rather than have all of the code here */
-#undef FUNCNAME
-#define FUNCNAME MPIDI_PG_Close_VCs
-#undef FCNAME
-#define FCNAME MPL_QUOTE(FUNCNAME)
 /*@
   MPIDI_PG_Close_VCs - Close all virtual connections on all process groups.
   
@@ -1164,14 +946,13 @@ int MPIDI_PG_Close_VCs( void )
 {
     MPIDI_PG_t * pg = MPIDI_PG_list;
     int mpi_errno = MPI_SUCCESS;
-    MPIDI_STATE_DECL(MPID_STATE_MPIDI_PG_CLOSE_VCS);
 
-    MPIDI_FUNC_ENTER(MPID_STATE_MPIDI_PG_CLOSE_VCS);
+    MPIR_FUNC_ENTER;
 
     while (pg) {
 	int i, inuse, n, i_start;
 
-	MPIU_DBG_MSG_S(CH3_DISCONNECT,VERBOSE,"Closing vcs for pg %s",
+	MPL_DBG_MSG_S(MPIDI_CH3_DBG_DISCONNECT,VERBOSE,"Closing vcs for pg %s",
 		       (char *)pg->id );
 
         /* We want to reduce the chance of having all processes send
@@ -1187,7 +968,7 @@ int MPIDI_PG_Close_VCs( void )
 	    /* If the VC is myself then skip the close message */
 	    if (pg == MPIDI_Process.my_pg && i == MPIDI_Process.my_pg_rank) {
                 /* XXX DJG FIXME-MT should we be checking this? */
-                if (MPIU_Object_get_ref(vc) != 0) {
+                if (MPIR_Object_get_ref(vc) != 0) {
                     MPIDI_PG_release_ref(pg, &inuse);
                 }
 		continue;
@@ -1196,11 +977,11 @@ int MPIDI_PG_Close_VCs( void )
 	    if (vc->state == MPIDI_VC_STATE_ACTIVE ||
 		vc->state == MPIDI_VC_STATE_REMOTE_CLOSE) {
 		mpi_errno = MPIDI_CH3U_VC_SendClose( vc, i );
-                if (mpi_errno) MPIR_ERR_POP(mpi_errno);
+                MPIR_ERR_CHECK(mpi_errno);
 	    } else if (vc->state == MPIDI_VC_STATE_INACTIVE ||
                        vc->state == MPIDI_VC_STATE_MORIBUND) {
                 /* XXX DJG FIXME-MT should we be checking this? */
-                if (MPIU_Object_get_ref(vc) != 0) {
+                if (MPIR_Object_get_ref(vc) != 0) {
 		    /* FIXME: If the reference count for the vc is not 0,
 		       something is wrong */
                     MPIDI_PG_release_ref(pg, &inuse);
@@ -1212,7 +993,7 @@ int MPIDI_PG_Close_VCs( void )
                 if (vc->state == MPIDI_VC_STATE_INACTIVE)
                     MPIDI_CHANGE_VC_STATE(vc, INACTIVE_CLOSED);
             } else {
-		MPIU_DBG_MSG_FMT(CH3_DISCONNECT,VERBOSE,(MPIU_DBG_FDEST,
+		MPL_DBG_MSG_FMT(MPIDI_CH3_DBG_DISCONNECT,VERBOSE,(MPL_DBG_FDEST,
 		     "vc=%p: not sending a close to %d, vc in state %s", vc,i,
 		     MPIDI_VC_GetStateString(vc->state)));
 	    }
@@ -1226,7 +1007,7 @@ int MPIDI_PG_Close_VCs( void )
        handles any close requests that this code generates) */
 
  fn_exit:
-    MPIDI_FUNC_EXIT(MPID_STATE_MPIDI_PG_CLOSE_VCS);
+    MPIR_FUNC_EXIT;
     return mpi_errno;
  fn_fail:
     goto fn_exit;
@@ -1247,10 +1028,10 @@ int MPIU_PG_Printall( FILE *fp )
     while (pg) {
         /* XXX DJG FIXME-MT should we be checking this? */
 	fprintf( fp, "size = %d, refcount = %d, id = %s\n", 
-		 pg->size, MPIU_Object_get_ref(pg), (char *)pg->id );
+		 pg->size, MPIR_Object_get_ref(pg), (char *)pg->id );
 	for (i=0; i<pg->size; i++) {
 	    fprintf( fp, "\tVCT rank = %d, refcount = %d, lpid = %d, state = %d \n", 
-		     pg->vct[i].pg_rank, MPIU_Object_get_ref(&pg->vct[i]),
+		     pg->vct[i].pg_rank, MPIR_Object_get_ref(&pg->vct[i]),
 		     pg->vct[i].lpid, (int)pg->vct[i].state );
 	}
 	fflush(fp);
@@ -1262,17 +1043,5 @@ int MPIU_PG_Printall( FILE *fp )
 
 int MPIDI_PG_CheckForSingleton( void )
 {
-
-#ifdef USE_PMI2_API
-    /* PMI2 FIXME for now we just always assume we aren't doing singleton init */
-#else
-    if (strstr((char*)pg_world->id,"singinit_kvs") == (char *)pg_world->id) {
-	char buf[256];
-	/* Force an enroll */
-	PMI_KVS_Get( "foobar", "foobar", buf, sizeof(buf) );
-	PMI_KVS_Get_my_name( pg_world->id, 256 );
-	PMI_KVS_Get_my_name( pg_world->connData, 256 );
-    }
-#endif
     return MPI_SUCCESS;
 }
