@@ -85,22 +85,22 @@ int MPIDI_RMA_init(void)
 {
     int mpi_errno = MPI_SUCCESS;
     int i;
-    MPIR_CHKPMEM_DECL(3);
+    MPIR_CHKPMEM_DECL();
 
 
     MPIR_FUNC_ENTER;
 
-    MPIR_CHKPMEM_MALLOC(global_rma_op_pool_start, MPIDI_RMA_Op_t *,
+    MPIR_CHKPMEM_MALLOC(global_rma_op_pool_start,
                         sizeof(MPIDI_RMA_Op_t) * MPIR_CVAR_CH3_RMA_OP_GLOBAL_POOL_SIZE,
-                        mpi_errno, "RMA op pool", MPL_MEM_RMA);
+                        MPL_MEM_RMA);
     for (i = 0; i < MPIR_CVAR_CH3_RMA_OP_GLOBAL_POOL_SIZE; i++) {
         global_rma_op_pool_start[i].pool_type = MPIDI_RMA_POOL_GLOBAL;
         DL_APPEND(global_rma_op_pool_head, &(global_rma_op_pool_start[i]));
     }
 
-    MPIR_CHKPMEM_MALLOC(global_rma_target_pool_start, MPIDI_RMA_Target_t *,
+    MPIR_CHKPMEM_MALLOC(global_rma_target_pool_start,
                         sizeof(MPIDI_RMA_Target_t) * MPIR_CVAR_CH3_RMA_TARGET_GLOBAL_POOL_SIZE,
-                        mpi_errno, "RMA target pool", MPL_MEM_RMA);
+                        MPL_MEM_RMA);
     for (i = 0; i < MPIR_CVAR_CH3_RMA_TARGET_GLOBAL_POOL_SIZE; i++) {
         global_rma_target_pool_start[i].pool_type = MPIDI_RMA_POOL_GLOBAL;
         DL_APPEND(global_rma_target_pool_head, &(global_rma_target_pool_start[i]));
@@ -123,6 +123,10 @@ void MPIDI_RMA_finalize(void)
 
     MPL_free(global_rma_op_pool_start);
     MPL_free(global_rma_target_pool_start);
+    global_rma_op_pool_head = NULL;
+    global_rma_op_pool_start = NULL;
+    global_rma_target_pool_head = NULL;
+    global_rma_target_pool_start = NULL;
 
     MPIR_FUNC_EXIT;
 }
@@ -133,7 +137,6 @@ int MPID_Win_free(MPIR_Win ** win_ptr)
     int mpi_errno = MPI_SUCCESS;
     int in_use;
     MPIR_Comm *comm_ptr;
-    MPIR_Errflag_t errflag = MPIR_ERR_NONE;
 
     MPIR_FUNC_ENTER;
 
@@ -161,7 +164,7 @@ int MPID_Win_free(MPIR_Win ** win_ptr)
         MPIR_ERR_CHECK(mpi_errno);
     }
 
-    mpi_errno = MPIR_Barrier((*win_ptr)->comm_ptr, &errflag);
+    mpi_errno = MPIR_Barrier((*win_ptr)->comm_ptr, 0);
     MPIR_ERR_CHECK(mpi_errno);
 
     /* Free window resources in lower layer. */
