@@ -14,7 +14,7 @@
 
 int MPID_Init(int, int *);
 int MPID_InitCompleted(void);
-int MPID_Allocate_vci(int *vci);
+int MPID_Allocate_vci(int *vci, bool is_shared);
 int MPID_Deallocate_vci(int vci);
 MPL_STATIC_INLINE_PREFIX int MPID_Cancel_recv(MPIR_Request *) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Cancel_send(MPIR_Request *) MPL_STATIC_INLINE_SUFFIX;
@@ -26,7 +26,6 @@ int MPID_Comm_get_all_failed_procs(MPIR_Comm *, MPIR_Group **, int);
 int MPID_Comm_revoke(MPIR_Comm *, int);
 int MPID_Comm_failure_ack(MPIR_Comm *);
 MPL_STATIC_INLINE_PREFIX int MPID_Comm_AS_enabled(MPIR_Comm *) MPL_STATIC_INLINE_SUFFIX;
-int MPID_Comm_get_lpid(MPIR_Comm *, int, uint64_t *, bool);
 int MPID_CS_finalize(void);
 int MPID_Finalize(void);
 int MPID_Get_universe_size(int *);
@@ -44,7 +43,7 @@ MPL_STATIC_INLINE_PREFIX int MPID_Mrecv(void *, MPI_Aint, MPI_Datatype, MPIR_Req
                                         MPI_Status *, MPIR_Request **) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Imrecv(void *, MPI_Aint, MPI_Datatype, MPIR_Request *,
                                          MPIR_Request **) MPL_STATIC_INLINE_SUFFIX;
-int MPID_Open_port(MPIR_Info *, char *);
+int MPID_Open_port(MPIR_Info * info, char *port, int len);
 int MPID_Close_port(const char *);
 int MPID_Comm_accept(const char *, MPIR_Info *, int, MPIR_Comm *, MPIR_Comm **);
 int MPID_Comm_connect(const char *, MPIR_Info *, int, MPIR_Comm *, MPIR_Comm **);
@@ -65,13 +64,10 @@ int MPID_Progress_activate(int id);
 int MPID_Progress_deactivate(int id);
 MPL_STATIC_INLINE_PREFIX int MPID_Recv(void *, MPI_Aint, MPI_Datatype, int, int, MPIR_Comm *, int,
                                        MPI_Status *, MPIR_Request **) MPL_STATIC_INLINE_SUFFIX;
-MPL_STATIC_INLINE_PREFIX int MPID_Recv_init(void *, MPI_Aint, MPI_Datatype, int, int, MPIR_Comm *,
-                                            int, MPIR_Request **) MPL_STATIC_INLINE_SUFFIX;
+int MPID_Recv_init(void *, MPI_Aint, MPI_Datatype, int, int, MPIR_Comm *, int, MPIR_Request **);
 MPL_STATIC_INLINE_PREFIX void MPID_Request_set_completed(MPIR_Request *) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Request_complete(MPIR_Request *) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Request_is_anysource(MPIR_Request *) MPL_STATIC_INLINE_SUFFIX;
-MPL_STATIC_INLINE_PREFIX void MPID_Prequest_free_hook(MPIR_Request *) MPL_STATIC_INLINE_SUFFIX;
-MPL_STATIC_INLINE_PREFIX void MPID_Part_request_free_hook(MPIR_Request *) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Send(const void *, MPI_Aint, MPI_Datatype, int, int, MPIR_Comm *,
                                        int, MPIR_Request **) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Ssend(const void *, MPI_Aint, MPI_Datatype, int, int, MPIR_Comm *,
@@ -81,18 +77,14 @@ MPL_STATIC_INLINE_PREFIX int MPID_Rsend(const void *, MPI_Aint, MPI_Datatype, in
 MPL_STATIC_INLINE_PREFIX int MPID_Irsend(const void *, MPI_Aint, MPI_Datatype, int, int,
                                          MPIR_Comm *, int,
                                          MPIR_Request **) MPL_STATIC_INLINE_SUFFIX;
-MPL_STATIC_INLINE_PREFIX int MPID_Send_init(const void *, MPI_Aint, MPI_Datatype, int, int,
-                                            MPIR_Comm *, int,
-                                            MPIR_Request **) MPL_STATIC_INLINE_SUFFIX;
-MPL_STATIC_INLINE_PREFIX int MPID_Ssend_init(const void *, MPI_Aint, MPI_Datatype, int, int,
-                                             MPIR_Comm *, int,
-                                             MPIR_Request **) MPL_STATIC_INLINE_SUFFIX;
-MPL_STATIC_INLINE_PREFIX int MPID_Bsend_init(const void *, MPI_Aint, MPI_Datatype, int, int,
-                                             MPIR_Comm *, int,
-                                             MPIR_Request **) MPL_STATIC_INLINE_SUFFIX;
-MPL_STATIC_INLINE_PREFIX int MPID_Rsend_init(const void *, MPI_Aint, MPI_Datatype, int, int,
-                                             MPIR_Comm *, int,
-                                             MPIR_Request **) MPL_STATIC_INLINE_SUFFIX;
+int MPID_Send_init(const void *, MPI_Aint, MPI_Datatype, int, int,
+                   MPIR_Comm *, int, MPIR_Request **);
+int MPID_Ssend_init(const void *, MPI_Aint, MPI_Datatype, int, int,
+                    MPIR_Comm *, int, MPIR_Request **);
+int MPID_Bsend_init(const void *, MPI_Aint, MPI_Datatype, int, int,
+                    MPIR_Comm *, int, MPIR_Request **);
+int MPID_Rsend_init(const void *, MPI_Aint, MPI_Datatype, int, int,
+                    MPIR_Comm *, int, MPIR_Request **);
 MPL_STATIC_INLINE_PREFIX int MPID_Startall(int, MPIR_Request *[]) MPL_STATIC_INLINE_SUFFIX;
 
 int MPID_Psend_init(const void *, int, MPI_Aint, MPI_Datatype, int, int, MPIR_Comm *,
@@ -174,61 +166,62 @@ int MPID_Type_commit_hook(MPIR_Datatype *);
 int MPID_Type_free_hook(MPIR_Datatype *);
 int MPID_Op_commit_hook(MPIR_Op *);
 int MPID_Op_free_hook(MPIR_Op *);
-int MPID_Intercomm_exchange_map(MPIR_Comm *, int, MPIR_Comm *, int, int *, uint64_t **, int *);
-int MPID_Create_intercomm_from_lpids(MPIR_Comm *, int, const uint64_t[]);
+int MPID_Intercomm_exchange(MPIR_Comm * local_comm, int local_leader,
+                            MPIR_Comm * peer_comm, int remote_leader, int tag,
+                            int context_id, int *remote_context_id_out,
+                            int *remote_size_out, MPIR_Lpid ** remote_lpids_out, int timeout);
+int MPID_Create_intercomm_from_lpids(MPIR_Comm *, int, const MPIR_Lpid[]);
 int MPID_Comm_commit_pre_hook(MPIR_Comm *);
 int MPID_Comm_free_hook(MPIR_Comm *);
 int MPID_Comm_set_hints(MPIR_Comm *, MPIR_Info *);
 int MPID_Comm_commit_post_hook(MPIR_Comm *);
 int MPID_Stream_create_hook(MPIR_Stream * stream);
 int MPID_Stream_free_hook(MPIR_Stream * stream);
-MPL_STATIC_INLINE_PREFIX int MPID_Barrier(MPIR_Comm *, MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+MPL_STATIC_INLINE_PREFIX int MPID_Barrier(MPIR_Comm *, int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Bcast(void *, MPI_Aint, MPI_Datatype, int, MPIR_Comm *,
-                                        MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                        int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Allreduce(const void *, void *, MPI_Aint, MPI_Datatype, MPI_Op,
-                                            MPIR_Comm *, MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                            MPIR_Comm *, int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Allgather(const void *, MPI_Aint, MPI_Datatype, void *, MPI_Aint,
                                             MPI_Datatype, MPIR_Comm *,
-                                            MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                            int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Allgatherv(const void *, MPI_Aint, MPI_Datatype, void *,
                                              const MPI_Aint *, const MPI_Aint *, MPI_Datatype,
-                                             MPIR_Comm *,
-                                             MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                             MPIR_Comm *, int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Scatter(const void *, MPI_Aint, MPI_Datatype, void *, MPI_Aint,
                                           MPI_Datatype, int, MPIR_Comm *,
-                                          MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                          int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Scatterv(const void *, const MPI_Aint *, const MPI_Aint *,
                                            MPI_Datatype, void *, MPI_Aint, MPI_Datatype, int,
-                                           MPIR_Comm *, MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                           MPIR_Comm *, int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Gather(const void *, MPI_Aint, MPI_Datatype, void *, MPI_Aint,
                                          MPI_Datatype, int, MPIR_Comm *,
-                                         MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                         int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Gatherv(const void *, MPI_Aint, MPI_Datatype, void *,
                                           const MPI_Aint *, const MPI_Aint *, MPI_Datatype, int,
-                                          MPIR_Comm *, MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                          MPIR_Comm *, int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Alltoall(const void *, MPI_Aint, MPI_Datatype, void *, MPI_Aint,
-                                           MPI_Datatype, MPIR_Comm *,
-                                           MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                           MPI_Datatype, MPIR_Comm *, int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Alltoallv(const void *, const MPI_Aint *, const MPI_Aint *,
                                             MPI_Datatype, void *, const MPI_Aint *,
                                             const MPI_Aint *, MPI_Datatype, MPIR_Comm *,
-                                            MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                            int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Alltoallw(const void *, const MPI_Aint[], const MPI_Aint[],
                                             const MPI_Datatype[], void *, const MPI_Aint[],
                                             const MPI_Aint[], const MPI_Datatype[], MPIR_Comm *,
-                                            MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                            int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Reduce(const void *, void *, MPI_Aint, MPI_Datatype, MPI_Op, int,
-                                         MPIR_Comm *, MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                         MPIR_Comm *, int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Reduce_scatter(const void *, void *, const MPI_Aint[],
                                                  MPI_Datatype, MPI_Op, MPIR_Comm *,
-                                                 MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                                 int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Reduce_scatter_block(const void *, void *, MPI_Aint, MPI_Datatype,
                                                        MPI_Op, MPIR_Comm *,
-                                                       MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                                       int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Scan(const void *, void *, MPI_Aint, MPI_Datatype, MPI_Op,
-                                       MPIR_Comm *, MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                       MPIR_Comm *, int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Exscan(const void *, void *, MPI_Aint, MPI_Datatype, MPI_Op,
-                                         MPIR_Comm *, MPIR_Errflag_t *) MPL_STATIC_INLINE_SUFFIX;
+                                         MPIR_Comm *, int) MPL_STATIC_INLINE_SUFFIX;
 MPL_STATIC_INLINE_PREFIX int MPID_Neighbor_allgather(const void *, MPI_Aint, MPI_Datatype, void *,
                                                      MPI_Aint, MPI_Datatype,
                                                      MPIR_Comm *) MPL_STATIC_INLINE_SUFFIX;
@@ -316,6 +309,103 @@ MPL_STATIC_INLINE_PREFIX int MPID_Iscatter(const void *, MPI_Aint, MPI_Datatype,
 MPL_STATIC_INLINE_PREFIX int MPID_Iscatterv(const void *, const MPI_Aint *, const MPI_Aint *,
                                             MPI_Datatype, void *, MPI_Aint, MPI_Datatype, int,
                                             MPIR_Comm *, MPIR_Request **) MPL_STATIC_INLINE_SUFFIX;
+int MPID_Bcast_init(void *buffer, MPI_Aint count, MPI_Datatype datatype,
+                    int root, MPIR_Comm * comm_ptr, MPIR_Info * info_ptr, MPIR_Request ** request);
+int MPID_Allreduce_init(const void *sendbuf, void *recvbuf, MPI_Aint count,
+                        MPI_Datatype datatype, MPI_Op op,
+                        MPIR_Comm * comm_ptr, MPIR_Info * info_ptr, MPIR_Request ** request);
+int MPID_Reduce_init(const void *sendbuf, void *recvbuf, MPI_Aint count,
+                     MPI_Datatype datatype, MPI_Op op, int root,
+                     MPIR_Comm * comm_ptr, MPIR_Info * info_ptr, MPIR_Request ** request);
+int MPID_Alltoall_init(const void *sendbuf, MPI_Aint sendcount,
+                       MPI_Datatype sendtype, void *recvbuf,
+                       MPI_Aint recvcount, MPI_Datatype recvtype,
+                       MPIR_Comm * comm_ptr, MPIR_Info * info_ptr, MPIR_Request ** request);
+int MPID_Alltoallv_init(const void *sendbuf, const MPI_Aint sendcounts[],
+                        const MPI_Aint sdispls[], MPI_Datatype sendtype,
+                        void *recvbuf, const MPI_Aint recvcounts[],
+                        const MPI_Aint rdispls[], MPI_Datatype recvtype,
+                        MPIR_Comm * comm_ptr, MPIR_Info * info_ptr, MPIR_Request ** request);
+int MPID_Alltoallw_init(const void *sendbuf, const MPI_Aint sendcounts[],
+                        const MPI_Aint sdispls[],
+                        const MPI_Datatype sendtypes[],
+                        void *recvbuf, const MPI_Aint recvcounts[],
+                        const MPI_Aint rdispls[],
+                        const MPI_Datatype recvtypes[],
+                        MPIR_Comm * comm_ptr, MPIR_Info * info_ptr, MPIR_Request ** request);
+int MPID_Allgather_init(const void *sendbuf, MPI_Aint sendcount,
+                        MPI_Datatype sendtype, void *recvbuf,
+                        MPI_Aint recvcount, MPI_Datatype recvtype,
+                        MPIR_Comm * comm_ptr, MPIR_Info * info_ptr, MPIR_Request ** request);
+int MPID_Allgatherv_init(const void *sendbuf, MPI_Aint sendcount,
+                         MPI_Datatype sendtype, void *recvbuf,
+                         const MPI_Aint * recvcounts,
+                         const MPI_Aint * displs, MPI_Datatype recvtype,
+                         MPIR_Comm * comm_ptr, MPIR_Info * info_ptr, MPIR_Request ** request);
+int MPID_Reduce_scatter_block_init(const void *sendbuf, void *recvbuf,
+                                   MPI_Aint recvcount,
+                                   MPI_Datatype datatype, MPI_Op op,
+                                   MPIR_Comm * comm, MPIR_Info * info, MPIR_Request ** request);
+int MPID_Reduce_scatter_init(const void *sendbuf, void *recvbuf,
+                             const MPI_Aint recvcounts[],
+                             MPI_Datatype datatype, MPI_Op op,
+                             MPIR_Comm * comm, MPIR_Info * info, MPIR_Request ** request);
+int MPID_Scan_init(const void *sendbuf, void *recvbuf, MPI_Aint count,
+                   MPI_Datatype datatype, MPI_Op op, MPIR_Comm * comm,
+                   MPIR_Info * info, MPIR_Request ** request);
+int MPID_Gather_init(const void *sendbuf, MPI_Aint sendcount,
+                     MPI_Datatype sendtype, void *recvbuf,
+                     MPI_Aint recvcount, MPI_Datatype recvtype, int root,
+                     MPIR_Comm * comm, MPIR_Info * info, MPIR_Request ** request);
+int MPID_Gatherv_init(const void *sendbuf, MPI_Aint sendcount,
+                      MPI_Datatype sendtype, void *recvbuf,
+                      const MPI_Aint recvcounts[], const MPI_Aint displs[],
+                      MPI_Datatype recvtype, int root, MPIR_Comm * comm,
+                      MPIR_Info * info, MPIR_Request ** request);
+int MPID_Scatter_init(const void *sendbuf, MPI_Aint sendcount,
+                      MPI_Datatype sendtype, void *recvbuf,
+                      MPI_Aint recvcount, MPI_Datatype recvtype, int root,
+                      MPIR_Comm * comm, MPIR_Info * info, MPIR_Request ** request);
+int MPID_Scatterv_init(const void *sendbuf, const MPI_Aint sendcounts[],
+                       const MPI_Aint displs[], MPI_Datatype sendtype,
+                       void *recvbuf, MPI_Aint recvcount,
+                       MPI_Datatype recvtype, int root, MPIR_Comm * comm,
+                       MPIR_Info * info, MPIR_Request ** request);
+int MPID_Barrier_init(MPIR_Comm * comm, MPIR_Info * info, MPIR_Request ** request);
+int MPID_Exscan_init(const void *sendbuf, void *recvbuf, MPI_Aint count,
+                     MPI_Datatype datatype, MPI_Op op, MPIR_Comm * comm,
+                     MPIR_Info * info, MPIR_Request ** request);
+int MPID_Neighbor_allgather_init(const void *sendbuf, MPI_Aint sendcount,
+                                 MPI_Datatype sendtype, void *recvbuf,
+                                 MPI_Aint recvcount, MPI_Datatype recvtype,
+                                 MPIR_Comm * comm, MPIR_Info * info, MPIR_Request ** request);
+int MPID_Neighbor_allgatherv_init(const void *sendbuf, MPI_Aint sendcount,
+                                  MPI_Datatype sendtype, void *recvbuf,
+                                  const MPI_Aint recvcounts[],
+                                  const MPI_Aint displs[],
+                                  MPI_Datatype recvtype, MPIR_Comm * comm,
+                                  MPIR_Info * info, MPIR_Request ** request);
+int MPID_Neighbor_alltoall_init(const void *sendbuf, MPI_Aint sendcount,
+                                MPI_Datatype sendtype, void *recvbuf,
+                                MPI_Aint recvcount, MPI_Datatype recvtype,
+                                MPIR_Comm * comm, MPIR_Info * info, MPIR_Request ** request);
+int MPID_Neighbor_alltoallv_init(const void *sendbuf,
+                                 const MPI_Aint sendcounts[],
+                                 const MPI_Aint sdispls[],
+                                 MPI_Datatype sendtype, void *recvbuf,
+                                 const MPI_Aint recvcounts[],
+                                 const MPI_Aint rdispls[],
+                                 MPI_Datatype recvtype, MPIR_Comm * comm,
+                                 MPIR_Info * info, MPIR_Request ** request);
+int MPID_Neighbor_alltoallw_init(const void *sendbuf,
+                                 const MPI_Aint sendcounts[],
+                                 const MPI_Aint sdispls[],
+                                 const MPI_Datatype sendtypes[],
+                                 void *recvbuf,
+                                 const MPI_Aint recvcounts[],
+                                 const MPI_Aint rdispls[],
+                                 const MPI_Datatype recvtypes[],
+                                 MPIR_Comm * comm, MPIR_Info * info, MPIR_Request ** request);
 int MPID_Send_enqueue(const void *buf, MPI_Aint count, MPI_Datatype datatype,
                       int dest, int tag, MPIR_Comm * comm_ptr);
 int MPID_Recv_enqueue(void *buf, MPI_Aint count, MPI_Datatype datatype,
@@ -383,8 +473,4 @@ extern MPL_dbg_class MPIDI_CH4_DBG_COMM;
 extern MPL_dbg_class MPIDI_CH4_DBG_MEMORY;
 #endif /* MPL_USE_DBG_LOGGING */
 
-/* routines only used during init */
-int MPIDI_create_init_comm(MPIR_Comm ** comm_ptr);
-void MPIDI_destroy_init_comm(MPIR_Comm ** comm_ptr);
-
 #endif /* MPIDCH4_H_INCLUDED */
