@@ -17,17 +17,17 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_NM_mpi_isend(const void *buf,
     int mpi_errno = MPI_SUCCESS;
 
     int context_offset = MPIR_PT2PT_ATTR_CONTEXT_OFFSET(attr);
-    MPIR_Errflag_t errflag = MPIR_PT2PT_ATTR_GET_ERRFLAG(attr);
+    int coll_attr = MPIR_PT2PT_ATTR_GET_ERRFLAG(attr);
     bool syncflag = MPIR_PT2PT_ATTR_GET_SYNCFLAG(attr);
 
-    int vni_src, vni_dst;
-    vni_src = 0;
-    vni_dst = 0;
+    int vci_src, vci_dst;
+    vci_src = 0;
+    vci_dst = 0;
 
-    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI(vni_src).lock);
+    MPID_THREAD_CS_ENTER(VCI, MPIDI_VCI_LOCK(vci_src));
     mpi_errno = MPIDIG_mpi_isend(buf, count, datatype, rank, tag, comm, context_offset, addr,
-                                 vni_src, vni_dst, request, syncflag, errflag);
-    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI(vni_src).lock);
+                                 vci_src, vci_dst, request, syncflag, coll_attr);
+    MPID_THREAD_CS_EXIT(VCI, MPIDI_VCI_LOCK(vci_src));
 
     return mpi_errno;
 }
