@@ -12,65 +12,58 @@
  * value that is generally common across the different providers. */
 #define MPIDI_OFI_MAX_ERR_DATA_SIZE 64
 
-static int peek_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
-static int peek_empty_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
-static int send_huge_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
-static int ssend_ack_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
-static int chunk_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * req);
-static int inject_emu_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * req);
-static int accept_probe_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
-static int dynproc_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
-static int am_isend_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
-static int am_isend_rdma_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
-static int am_isend_pipeline_event(int vni, struct fi_cq_tagged_entry *wc,
+static int peek_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
+static int peek_empty_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
+static int ssend_ack_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
+static int chunk_done_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * req);
+static int inject_emu_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * req);
+static int dynproc_done_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
+static int am_isend_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
+static int am_isend_rdma_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq);
+static int am_isend_pipeline_event(int vci, struct fi_cq_tagged_entry *wc,
                                    MPIR_Request * dont_use_me);
-static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
-static int am_read_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * dont_use_me);
+static int am_recv_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq);
+static int am_read_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * dont_use_me);
 
-static int peek_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
+static int peek_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIR_FUNC_ENTER;
 
-    if (MPIDI_OFI_HUGE_SEND & wc->tag) {
-        mpi_errno = MPIDI_OFI_peek_huge_event(vni, wc, rreq);
-        goto fn_exit;
-    }
-
     rreq->status.MPI_SOURCE = MPIDI_OFI_cqe_get_source(wc, false);
     rreq->status.MPI_TAG = MPIDI_OFI_init_get_tag(wc->tag);
     rreq->status.MPI_ERROR = MPI_SUCCESS;
+
+    if (MPIDI_OFI_is_tag_rndv(wc->tag)) {
+        mpi_errno = MPIDI_OFI_peek_rndv_event(vci, wc, rreq);
+        goto fn_exit;
+    }
+
     MPIR_STATUS_SET_COUNT(rreq->status, wc->len);
-    /* util_id should be the last thing to change in rreq. Reason is
-     * we use util_id to indicate peek_event has completed and all the
+    /* peek_status should be the last thing to change in rreq. Reason is
+     * we use peek_status to indicate peek_event has completed and all the
      * relevant values have been copied to rreq. */
-    MPL_atomic_release_store_int(&(MPIDI_OFI_REQUEST(rreq, util_id)), MPIDI_OFI_PEEK_FOUND);
+    MPL_atomic_release_store_int(&(MPIDI_OFI_REQUEST(rreq, peek_status)), MPIDI_OFI_PEEK_FOUND);
 
   fn_exit:
     MPIR_FUNC_EXIT;
     return mpi_errno;
 }
 
-static int peek_empty_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
+static int peek_empty_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
 {
     MPIR_FUNC_ENTER;
-    MPIDI_OFI_dynamic_process_request_t *ctrl;
 
     switch (MPIDI_OFI_REQUEST(rreq, event_id)) {
         case MPIDI_OFI_EVENT_PEEK:
             rreq->status.MPI_ERROR = MPI_SUCCESS;
-            /* util_id should be the last thing to change in rreq. Reason is
-             * we use util_id to indicate peek_event has completed and all the
+            /* peek_status should be the last thing to change in rreq. Reason is
+             * we use peek_status to indicate peek_event has completed and all the
              * relevant values have been copied to rreq. */
-            MPL_atomic_release_store_int(&(MPIDI_OFI_REQUEST(rreq, util_id)),
+            MPL_atomic_release_store_int(&(MPIDI_OFI_REQUEST(rreq, peek_status)),
                                          MPIDI_OFI_PEEK_NOT_FOUND);
             break;
 
-        case MPIDI_OFI_EVENT_ACCEPT_PROBE:
-            ctrl = (MPIDI_OFI_dynamic_process_request_t *) rreq;
-            ctrl->done = MPIDI_OFI_PEEK_NOT_FOUND;
-            break;
-
         default:
             MPIR_Assert(0);
             break;
@@ -80,62 +73,19 @@ static int peek_empty_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request
     return MPI_SUCCESS;
 }
 
-static int send_huge_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
+static int ssend_ack_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
 {
-    int mpi_errno = MPI_SUCCESS;
-    int c, num_nics;
+    MPIDI_OFI_ack_request_t *req = (MPIDI_OFI_ack_request_t *) sreq;
     MPIR_FUNC_ENTER;
-
-    MPIR_cc_decr(sreq->cc_ptr, &c);
-
-    if (c == 0) {
-        MPIR_Comm *comm;
-        struct fid_mr **huge_send_mrs;
-
-        comm = sreq->comm;
-        num_nics = MPIDI_OFI_COMM(comm).enable_striping ? MPIDI_OFI_global.num_nics : 1;
-        huge_send_mrs = MPIDI_OFI_REQUEST(sreq, huge.send_mrs);
-
-        /* Clean up the memory region */
-        for (int i = 0; i < num_nics; i++) {
-            uint64_t key = fi_mr_key(huge_send_mrs[i]);
-            MPIDI_OFI_CALL(fi_close(&huge_send_mrs[i]->fid), mr_unreg);
-            if (!MPIDI_OFI_ENABLE_MR_PROV_KEY) {
-                MPIDI_OFI_mr_key_free(MPIDI_OFI_LOCAL_MR_KEY, key);
-            }
-        }
-        MPL_free(huge_send_mrs);
-
-        if (MPIDI_OFI_REQUEST(sreq, noncontig.pack.pack_buffer)) {
-            MPL_free(MPIDI_OFI_REQUEST(sreq, noncontig.pack.pack_buffer));
-        }
-
-        MPIR_Datatype_release_if_not_builtin(MPIDI_OFI_REQUEST(sreq, datatype));
-        MPIDI_CH4_REQUEST_FREE(sreq);
-    }
-    /* c != 0, ssend */
-  fn_exit:
-    MPIR_FUNC_EXIT;
-    return mpi_errno;
-  fn_fail:
-    goto fn_exit;
-}
-
-static int ssend_ack_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
-{
-    int mpi_errno;
-    MPIDI_OFI_ssendack_request_t *req = (MPIDI_OFI_ssendack_request_t *) sreq;
-    MPIR_FUNC_ENTER;
-    mpi_errno =
-        MPIDI_OFI_send_event(vni, NULL, req->signal_req,
-                             MPIDI_OFI_REQUEST(req->signal_req, event_id));
-
+    /* no need to cleanup the packed data buffer, it's done when the main send completes */
+    MPIDI_Request_complete_fast(req->signal_req);
+    /* req->ack_hdr should be NULL */
     MPL_free(req);
     MPIR_FUNC_EXIT;
-    return mpi_errno;
+    return MPI_SUCCESS;
 }
 
-static int chunk_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
+static int chunk_done_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
 {
     int c;
     MPIR_FUNC_ENTER;
@@ -151,7 +101,7 @@ static int chunk_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request
     return MPI_SUCCESS;
 }
 
-static int inject_emu_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
+static int inject_emu_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
 {
     int incomplete;
     MPIR_FUNC_ENTER;
@@ -161,26 +111,14 @@ static int inject_emu_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request
     if (!incomplete) {
         MPL_free(MPIDI_OFI_REQUEST(req, util.inject_buf));
         MPIDI_CH4_REQUEST_FREE(req);
-        MPIDI_OFI_global.per_vni[vni].am_inflight_inject_emus -= 1;
+        MPIDI_OFI_global.per_vci[vci].am_inflight_inject_emus -= 1;
     }
 
     MPIR_FUNC_EXIT;
     return MPI_SUCCESS;
 }
 
-static int accept_probe_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
-{
-    MPIR_FUNC_ENTER;
-    MPIDI_OFI_dynamic_process_request_t *ctrl = (MPIDI_OFI_dynamic_process_request_t *) rreq;
-    ctrl->source = MPIDI_OFI_cqe_get_source(wc, false);
-    ctrl->tag = MPIDI_OFI_init_get_tag(wc->tag);
-    ctrl->msglen = wc->len;
-    ctrl->done = MPIDI_OFI_PEEK_FOUND;
-    MPIR_FUNC_EXIT;
-    return MPI_SUCCESS;
-}
-
-static int dynproc_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
+static int dynproc_done_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
 {
     MPIR_FUNC_ENTER;
     MPIDI_OFI_dynamic_process_request_t *ctrl = (MPIDI_OFI_dynamic_process_request_t *) rreq;
@@ -189,7 +127,7 @@ static int dynproc_done_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Reque
     return MPI_SUCCESS;
 }
 
-static int am_isend_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
+static int am_isend_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIDI_OFI_am_header_t *msg_hdr;
@@ -199,7 +137,7 @@ static int am_isend_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     msg_hdr = &MPIDI_OFI_AM_SREQ_HDR(sreq, msg_hdr);
     MPID_Request_complete(sreq);
 
-    MPIDU_genq_private_pool_free_cell(MPIDI_global.per_vci[vni].pack_buf_pool,
+    MPIDU_genq_private_pool_free_cell(MPIDI_global.per_vci[vci].pack_buf_pool,
                                       MPIDI_OFI_AM_SREQ_HDR(sreq, pack_buffer));
     MPIDI_OFI_AM_SREQ_HDR(sreq, pack_buffer) = NULL;
     mpi_errno = MPIDIG_global.origin_cbs[msg_hdr->handler_id] (sreq);
@@ -212,7 +150,7 @@ static int am_isend_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     goto fn_exit;
 }
 
-static int am_isend_rdma_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
+static int am_isend_rdma_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * sreq)
 {
     int mpi_errno = MPI_SUCCESS;
 
@@ -226,7 +164,7 @@ static int am_isend_rdma_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Requ
     return mpi_errno;
 }
 
-static int am_isend_pipeline_event(int vni, struct fi_cq_tagged_entry *wc,
+static int am_isend_pipeline_event(int vci, struct fi_cq_tagged_entry *wc,
                                    MPIR_Request * dont_use_me)
 {
     int mpi_errno = MPI_SUCCESS;
@@ -240,10 +178,10 @@ static int am_isend_pipeline_event(int vni, struct fi_cq_tagged_entry *wc,
     sreq = ofi_req->sreq;
     MPID_Request_complete(sreq);        /* FIXME: Should not call MPIDI in NM ? */
 
-    MPIDU_genq_private_pool_free_cell(MPIDI_global.per_vci[vni].pack_buf_pool,
+    MPIDU_genq_private_pool_free_cell(MPIDI_global.per_vci[vci].pack_buf_pool,
                                       ofi_req->pack_buffer);
 
-    MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.per_vni[vni].am_hdr_buf_pool, ofi_req);
+    MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.per_vci[vci].am_hdr_buf_pool, ofi_req);
 
     int is_done = MPIDIG_am_send_async_finish_seg(sreq);
 
@@ -260,12 +198,11 @@ static int am_isend_pipeline_event(int vni, struct fi_cq_tagged_entry *wc,
     goto fn_exit;
 }
 
-static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
+static int am_recv_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * rreq)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIDI_OFI_am_header_t *am_hdr;
     MPIDI_OFI_am_unordered_msg_t *uo_msg = NULL;
-    fi_addr_t fi_src_addr;
     uint16_t expected_seqno, next_seqno;
     MPIR_FUNC_ENTER;
 
@@ -294,23 +231,24 @@ static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     }
 #endif
 
-    expected_seqno = MPIDI_OFI_am_get_next_recv_seqno(vni, am_hdr->fi_src_addr);
+    expected_seqno = MPIDI_OFI_am_get_next_recv_seqno(vci, am_hdr->src_id);
     if (am_hdr->seqno != expected_seqno) {
         /* This message came earlier than the one that we were expecting.
          * Put it in the queue to process it later. */
         MPL_DBG_MSG_FMT(MPIDI_CH4_DBG_GENERAL, TERSE,
                         (MPL_DBG_FDEST,
-                         "Expected seqno=%d but got %d (am_type=%d addr=%" PRIx64 "). "
+                         "Expected seqno=%d but got %d (am_type=%d src_rank=%d). "
                          "Enqueueing it to the queue.\n",
-                         expected_seqno, am_hdr->seqno, am_hdr->am_type, am_hdr->fi_src_addr));
-        mpi_errno = MPIDI_OFI_am_enqueue_unordered_msg(vni, orig_buf);
+                         expected_seqno, am_hdr->seqno, am_hdr->am_type, am_hdr->src_rank));
+        mpi_errno = MPIDI_OFI_am_enqueue_unordered_msg(vci, orig_buf);
         MPIR_ERR_CHECK(mpi_errno);
         goto fn_exit;
     }
 
     /* Received an expected message */
+    uint64_t remote_id;
   fn_repeat:
-    fi_src_addr = am_hdr->fi_src_addr;
+    remote_id = am_hdr->src_id;
     next_seqno = am_hdr->seqno + 1;
 
     void *p_data;
@@ -368,7 +306,8 @@ static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     MPL_free(uo_msg);
 
     /* See if we can process other messages in the queue */
-    if ((uo_msg = MPIDI_OFI_am_claim_unordered_msg(vni, fi_src_addr, next_seqno)) != NULL) {
+    uo_msg = MPIDI_OFI_am_claim_unordered_msg(vci, remote_id, next_seqno);
+    if (uo_msg != NULL) {
         am_hdr = &uo_msg->am_hdr;
         orig_buf = am_hdr;
 #ifdef NEEDS_STRICT_ALIGNMENT
@@ -379,8 +318,8 @@ static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
         goto fn_repeat;
     }
 
-    /* Record the next expected sequence number from fi_src_addr */
-    MPIDI_OFI_am_set_next_recv_seqno(vni, fi_src_addr, next_seqno);
+    /* Record the next expected sequence number */
+    MPIDI_OFI_am_set_next_recv_seqno(vci, remote_id, next_seqno);
 
   fn_exit:
     MPIR_FUNC_EXIT;
@@ -389,7 +328,7 @@ static int am_recv_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     goto fn_exit;
 }
 
-static int am_read_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * dont_use_me)
+static int am_read_event(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * dont_use_me)
 {
     int mpi_errno = MPI_SUCCESS;
     MPIR_Request *rreq;
@@ -406,7 +345,9 @@ static int am_read_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
             goto fn_exit;
         }
     } else if (ofi_req->rreq_hdr->lmt_type == MPIDI_OFI_AM_LMT_UNPACK) {
-        int done = MPIDI_OFI_am_lmt_unpack_event(rreq);
+        bool done;
+        mpi_errno = MPIDI_OFI_am_lmt_unpack_event(rreq, &done);
+        MPIR_ERR_CHECK(mpi_errno);
         if (!done) {
             goto fn_exit;
         }
@@ -429,105 +370,125 @@ static int am_read_event(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request *
     MPIDIG_REQUEST(rreq, req->target_cmpl_cb) (rreq);
     MPID_Request_complete(rreq);
   fn_exit:
-    MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.per_vni[vni].am_hdr_buf_pool, ofi_req);
+    MPIDU_genq_private_pool_free_cell(MPIDI_OFI_global.per_vci[vci].am_hdr_buf_pool, ofi_req);
     MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
     goto fn_exit;
 }
 
-int MPIDI_OFI_dispatch_function(int vni, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
+int MPIDI_OFI_dispatch_function(int vci, struct fi_cq_tagged_entry *wc, MPIR_Request * req)
 {
     int mpi_errno = MPI_SUCCESS;
 
     if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_SEND) {
         /* Passing the event_id as a parameter; do not need to load it from the
          * request object each time the send_event handler is invoked */
-        mpi_errno = MPIDI_OFI_send_event(vni, wc, req, MPIDI_OFI_EVENT_SEND);
+        mpi_errno = MPIDI_OFI_send_event(vci, wc, req, MPIDI_OFI_EVENT_SEND);
         goto fn_exit;
     } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_RECV) {
         /* Passing the event_id as a parameter; do not need to load it from the
          * request object each time the send_event handler is invoked */
-        mpi_errno = MPIDI_OFI_recv_event(vni, wc, req, MPIDI_OFI_EVENT_RECV);
+        mpi_errno = MPIDI_OFI_recv_event(vci, wc, req, MPIDI_OFI_EVENT_RECV);
         goto fn_exit;
-    } else if (likely(MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND)) {
-        mpi_errno = am_isend_event(vni, wc, req);
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND) {
+        mpi_errno = am_isend_event(vci, wc, req);
         goto fn_exit;
-    } else if (likely(MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND_RDMA)) {
-        mpi_errno = am_isend_rdma_event(vni, wc, req);
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND_RDMA) {
+        mpi_errno = am_isend_rdma_event(vci, wc, req);
         goto fn_exit;
-    } else if (likely(MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND_PIPELINE)) {
-        mpi_errno = am_isend_pipeline_event(vni, wc, req);
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_SEND_PIPELINE) {
+        mpi_errno = am_isend_pipeline_event(vci, wc, req);
         goto fn_exit;
-    } else if (likely(MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_RECV)) {
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_RECV) {
         if (wc->flags & FI_RECV)
-            mpi_errno = am_recv_event(vni, wc, req);
+            mpi_errno = am_recv_event(vci, wc, req);
 
         if (unlikely(wc->flags & FI_MULTI_RECV)) {
             MPIDI_OFI_am_repost_request_t *am = (MPIDI_OFI_am_repost_request_t *) req;
-            mpi_errno = MPIDI_OFI_am_repost_buffer(vni, am->index);
+            mpi_errno = MPIDI_OFI_am_repost_buffer(vci, am->index);
         }
 
         goto fn_exit;
-    } else if (likely(MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_READ)) {
-        mpi_errno = am_read_event(vni, wc, req);
+    } else if (MPIDI_OFI_REQUEST(req, event_id) == MPIDI_OFI_EVENT_AM_READ) {
+        mpi_errno = am_read_event(vci, wc, req);
         goto fn_exit;
     } else if (unlikely(1)) {
         switch (MPIDI_OFI_REQUEST(req, event_id)) {
             case MPIDI_OFI_EVENT_PEEK:
-                mpi_errno = peek_event(vni, wc, req);
-                break;
-
-            case MPIDI_OFI_EVENT_RECV_HUGE:
-                if (wc->tag & MPIDI_OFI_HUGE_SEND) {
-                    mpi_errno = MPIDI_OFI_recv_huge_event(vni, wc, req);
-                } else {
-                    mpi_errno = MPIDI_OFI_recv_event(vni, wc, req, MPIDI_OFI_EVENT_RECV_HUGE);
-                }
+                mpi_errno = peek_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_RECV_PACK:
-                mpi_errno = MPIDI_OFI_recv_event(vni, wc, req, MPIDI_OFI_EVENT_RECV_PACK);
+                mpi_errno = MPIDI_OFI_recv_event(vci, wc, req, MPIDI_OFI_EVENT_RECV_PACK);
                 break;
 
             case MPIDI_OFI_EVENT_RECV_NOPACK:
-                mpi_errno = MPIDI_OFI_recv_event(vni, wc, req, MPIDI_OFI_EVENT_RECV_NOPACK);
-                break;
-
-            case MPIDI_OFI_EVENT_SEND_HUGE:
-                mpi_errno = send_huge_event(vni, wc, req);
+                mpi_errno = MPIDI_OFI_recv_event(vci, wc, req, MPIDI_OFI_EVENT_RECV_NOPACK);
                 break;
 
             case MPIDI_OFI_EVENT_SEND_PACK:
-                mpi_errno = MPIDI_OFI_send_event(vni, wc, req, MPIDI_OFI_EVENT_SEND_PACK);
+                mpi_errno = MPIDI_OFI_send_event(vci, wc, req, MPIDI_OFI_EVENT_SEND_PACK);
                 break;
 
             case MPIDI_OFI_EVENT_SEND_NOPACK:
-                mpi_errno = MPIDI_OFI_send_event(vni, wc, req, MPIDI_OFI_EVENT_SEND_NOPACK);
+                mpi_errno = MPIDI_OFI_send_event(vci, wc, req, MPIDI_OFI_EVENT_SEND_NOPACK);
                 break;
 
             case MPIDI_OFI_EVENT_SSEND_ACK:
-                mpi_errno = ssend_ack_event(vni, wc, req);
+                mpi_errno = ssend_ack_event(vci, wc, req);
+                break;
+
+            case MPIDI_OFI_EVENT_RNDV_CTS:
+                mpi_errno = MPIDI_OFI_rndv_cts_event(vci, wc, req);
+                break;
+
+            case MPIDI_OFI_EVENT_PIPELINE_RECV_DATASIZE:
+                mpi_errno = MPIDI_OFI_pipeline_recv_datasize_event(wc, req);
+                break;
+
+            case MPIDI_OFI_EVENT_PIPELINE_SEND_CHUNK:
+                mpi_errno = MPIDI_OFI_pipeline_send_chunk_event(wc, req);
+                break;
+
+            case MPIDI_OFI_EVENT_PIPELINE_RECV_CHUNK:
+                mpi_errno = MPIDI_OFI_pipeline_recv_chunk_event(wc, req);
+                break;
+
+            case MPIDI_OFI_EVENT_RNDVREAD_RECV_MRS:
+                mpi_errno = MPIDI_OFI_rndvread_recv_mrs_event(wc, req);
+                break;
+
+            case MPIDI_OFI_EVENT_RNDVREAD_READ_CHUNK:
+                mpi_errno = MPIDI_OFI_rndvread_read_chunk_event(wc, req);
+                break;
+
+            case MPIDI_OFI_EVENT_RNDVREAD_ACK:
+                mpi_errno = MPIDI_OFI_rndvread_ack_event(wc, req);
+                break;
+
+            case MPIDI_OFI_EVENT_RNDVWRITE_RECV_MRS:
+                mpi_errno = MPIDI_OFI_rndvwrite_recv_mrs_event(wc, req);
+                break;
+
+            case MPIDI_OFI_EVENT_RNDVWRITE_WRITE_CHUNK:
+                mpi_errno = MPIDI_OFI_rndvwrite_write_chunk_event(wc, req);
+                break;
+
+            case MPIDI_OFI_EVENT_RNDVWRITE_ACK:
+                mpi_errno = MPIDI_OFI_rndvwrite_ack_event(wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_CHUNK_DONE:
-                mpi_errno = chunk_done_event(vni, wc, req);
-                break;
-
-            case MPIDI_OFI_EVENT_HUGE_CHUNK_DONE:
-                mpi_errno = MPIDI_OFI_huge_chunk_done_event(vni, wc, req);
+                mpi_errno = chunk_done_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_INJECT_EMU:
-                mpi_errno = inject_emu_event(vni, wc, req);
+                mpi_errno = inject_emu_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_DYNPROC_DONE:
-                mpi_errno = dynproc_done_event(vni, wc, req);
-                break;
-
-            case MPIDI_OFI_EVENT_ACCEPT_PROBE:
-                mpi_errno = accept_probe_event(vni, wc, req);
+                mpi_errno = dynproc_done_event(vci, wc, req);
                 break;
 
             case MPIDI_OFI_EVENT_ABORT:
@@ -542,7 +503,7 @@ int MPIDI_OFI_dispatch_function(int vni, struct fi_cq_tagged_entry *wc, MPIR_Req
     return mpi_errno;
 }
 
-int MPIDI_OFI_handle_cq_error(int vni, int nic, ssize_t ret)
+int MPIDI_OFI_handle_cq_error(int vci, int nic, ssize_t ret)
 {
     int mpi_errno = MPI_SUCCESS;
     struct fi_cq_err_entry e;
@@ -551,7 +512,7 @@ int MPIDI_OFI_handle_cq_error(int vni, int nic, ssize_t ret)
     ssize_t ret_cqerr;
     MPIR_FUNC_ENTER;
 
-    int ctx_idx = MPIDI_OFI_get_ctx_index(NULL, vni, nic);
+    int ctx_idx = MPIDI_OFI_get_ctx_index(vci, nic);
     switch (ret) {
         case -FI_EAVAIL:
             /* Provide separate error buffer for each thread. This makes the
@@ -572,20 +533,21 @@ int MPIDI_OFI_handle_cq_error(int vni, int nic, ssize_t ret)
 
                     switch (req->kind) {
                         case MPIR_REQUEST_KIND__SEND:
-                            mpi_errno = MPIDI_OFI_dispatch_function(vni, NULL, req);
+                            mpi_errno = MPIDI_OFI_dispatch_function(vci, NULL, req);
                             break;
 
                         case MPIR_REQUEST_KIND__RECV:
                             req->status.MPI_ERROR = MPI_ERR_TRUNCATE;
                             mpi_errno =
-                                MPIDI_OFI_dispatch_function(vni, (struct fi_cq_tagged_entry *) &e,
+                                MPIDI_OFI_dispatch_function(vci, (struct fi_cq_tagged_entry *) &e,
                                                             req);
                             break;
 
                         default:
-                            MPIR_ERR_SETFATALANDJUMP4(mpi_errno, MPI_ERR_OTHER, "**ofid_poll",
-                                                      "**ofid_poll %s %d %s %s", __SHORT_FILE__,
-                                                      __LINE__, __func__, fi_strerror(e.err));
+                            MPIR_ERR_SETFATALANDJUMP2(mpi_errno, MPI_ERR_OTHER, "**ofid_poll",
+                                                      "**ofid_poll %s %s",
+                                                      MPIDI_OFI_DEFAULT_NIC_NAME,
+                                                      fi_strerror(e.err));
                     }
 
                     break;
@@ -596,43 +558,47 @@ int MPIDI_OFI_handle_cq_error(int vni, int nic, ssize_t ret)
                      * NOTE: assuming only the receive request can be cancelled and reach here
                      */
                     int event_id = MPIDI_OFI_REQUEST(req, event_id);
-                    if (event_id == MPIDI_OFI_EVENT_DYNPROC_DONE) {
-                        dynproc_done_event(vni, e.op_context, req);
-                    } else {
-                        /* assume it is a pending recv */
-                        MPIR_STATUS_SET_CANCEL_BIT(req->status, TRUE);
-                        MPIR_STATUS_SET_COUNT(req->status, 0);
-                        if ((event_id == MPIDI_OFI_EVENT_RECV_PACK ||
-                             event_id == MPIDI_OFI_EVENT_GET_HUGE) &&
-                            MPIDI_OFI_REQUEST(req, noncontig.pack.pack_buffer)) {
-                            MPL_free(MPIDI_OFI_REQUEST(req, noncontig.pack.pack_buffer));
-                        } else if (MPIDI_OFI_ENABLE_PT2PT_NOPACK &&
-                                   event_id == MPIDI_OFI_EVENT_RECV_NOPACK &&
-                                   MPIDI_OFI_REQUEST(req, noncontig.nopack)) {
-                            MPL_free(MPIDI_OFI_REQUEST(req, noncontig.nopack));
-                        }
-                        MPIR_Datatype_release_if_not_builtin(MPIDI_OFI_REQUEST(req, datatype));
-                        MPIDI_Request_complete_fast(req);
+                    switch (event_id) {
+                        case MPIDI_OFI_EVENT_DYNPROC_DONE:
+                            dynproc_done_event(vci, e.op_context, req);
+                            break;
+                        case MPIDI_OFI_EVENT_RECV:
+                        case MPIDI_OFI_EVENT_RECV_PACK:
+                        case MPIDI_OFI_EVENT_RECV_NOPACK:
+                            MPIR_STATUS_SET_CANCEL_BIT(req->status, TRUE);
+                            MPIR_STATUS_SET_COUNT(req->status, 0);
+                            MPIR_Datatype_release_if_not_builtin(MPIDI_OFI_REQUEST(req, datatype));
+                            if ((event_id == MPIDI_OFI_EVENT_RECV_PACK) &&
+                                MPIDI_OFI_REQUEST(req, noncontig.pack.pack_buffer)) {
+                                MPL_free(MPIDI_OFI_REQUEST(req, noncontig.pack.pack_buffer));
+                            } else if (event_id == MPIDI_OFI_EVENT_RECV_NOPACK) {
+                                MPL_free(MPIDI_OFI_REQUEST(req, noncontig.nopack.iovs));
+                            }
+                            MPIDI_Request_complete_fast(req);
+                            break;
+                        default:
+                            /* Assert? */
+                            break;
                     }
                     break;
 
                 case FI_ENOMSG:
                     req = MPIDI_OFI_context_to_request(e.op_context);
-                    mpi_errno = peek_empty_event(vni, NULL, req);
+                    mpi_errno = peek_empty_event(vci, NULL, req);
                     break;
 
                 default:
-                    MPIR_ERR_SETFATALANDJUMP4(mpi_errno, MPI_ERR_OTHER, "**ofid_poll",
-                                              "**ofid_poll %s %d %s %s", __SHORT_FILE__,
-                                              __LINE__, __func__, fi_strerror(e.err));
+                    MPIR_ERR_SETFATALANDJUMP2(mpi_errno, MPI_ERR_OTHER, "**ofid_poll",
+                                              "**ofid_poll %s %s",
+                                              MPIDI_OFI_DEFAULT_NIC_NAME, fi_strerror(e.err));
             }
 
             break;
 
         default:
-            MPIR_ERR_SETFATALANDJUMP4(mpi_errno, MPI_ERR_OTHER, "**ofid_poll",
-                                      "**ofid_poll %s %d %s %s", __SHORT_FILE__, __LINE__,
-                                      __func__, fi_strerror(errno));
+            MPIR_ERR_SETFATALANDJUMP2(mpi_errno, MPI_ERR_OTHER, "**ofid_poll",
+                                      "**ofid_poll %s %s",
+                                      MPIDI_OFI_DEFAULT_NIC_NAME, fi_strerror(errno));
     }
 
   fn_exit:
@@ -641,3 +607,35 @@ int MPIDI_OFI_handle_cq_error(int vni, int nic, ssize_t ret)
   fn_fail:
     goto fn_exit;
 }
+
+int MPIDI_OFI_send_ack(MPIR_Request * rreq, int context_id, void *hdr, int hdr_sz, int vci_local,
+                       int vci_remote)
+{
+    int mpi_errno = MPI_SUCCESS;
+
+    MPIR_Comm *comm = rreq->comm;
+    int tag = rreq->status.MPI_TAG;
+    /* NOTE: we are dst_rank, reply to src_rank */
+    int src_rank = rreq->status.MPI_SOURCE;
+
+    /* NOTE: this is a reply, we don't need rank in the match bits */
+    uint64_t match_bits = MPIDI_OFI_init_sendtag(context_id, 0, tag);
+    match_bits |= MPIDI_OFI_ACK_SEND;
+
+    if (vci_remote == -1) {
+        /* MPI_ANY_SOURCE */
+        vci_remote = MPIDI_get_vci(SRC_VCI_FROM_RECVER, rreq->comm, rreq->status.MPI_SOURCE,
+                                   rreq->comm->rank, rreq->status.MPI_TAG);
+    }
+
+    int nic = 0;
+    int ctx_idx = MPIDI_OFI_get_ctx_index(vci_local, nic);
+    MPIDI_av_entry_t *av = MPIDIU_comm_rank_to_av(comm, src_rank);
+    fi_addr_t dest_addr = MPIDI_OFI_av_to_phys(av, vci_local, nic, vci_remote, nic);
+    MPIDI_OFI_CALL_RETRY(fi_tinject(MPIDI_OFI_global.ctx[ctx_idx].tx, hdr, hdr_sz,
+                                    dest_addr, match_bits), vci_local, tinject);
+  fn_exit:
+    return mpi_errno;
+  fn_fail:
+    goto fn_exit;
+}
