@@ -67,12 +67,12 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_NB_RG_rank0_hold_buf_completion(void *v
         MPIDI_POSIX_RELEASE_GATHER_NB_IREDUCE_GATHER_FLAG_ADDR(rank, segment, num_ranks);
 
     /* Rank 0 keeps checking if the a particular cell is available */
-    if (rank == 0 && (MPL_atomic_acquire_load_uint64(my_gather_flag_addr) == -1) &&
+    if (rank == 0 && (MPL_atomic_acquire_load_uint64(my_gather_flag_addr) == (uint64_t) - 1) &&
         (last_seq_no == per_call_data->seq_no - num_cells || last_seq_no < num_cells)) {
         /* Buffer can be overwritten */
         if (rank == 0) {
             /* Mark the buffer as occupied */
-            MPL_atomic_release_store_uint64(my_gather_flag_addr, -2);
+            MPL_atomic_release_store_uint64(my_gather_flag_addr, (uint64_t) - 2);
             /* Update the release_flag, so that the children ranks can move on */
             MPL_atomic_release_store_uint64(my_release_flag_addr, per_call_data->seq_no);
             *done = 1;
@@ -222,7 +222,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_NB_RG_reduce_data_cb(MPIR_Comm * comm,
 
     if (rank == 0 && root == 0) {
         /* Mark the buffer as available, if root was rank 0 */
-        MPL_atomic_release_store_uint64(my_gather_flag_addr, -1);
+        MPL_atomic_release_store_uint64(my_gather_flag_addr, (uint64_t) - 1);
     } else {
         /* Update my gather flag to seq_no (so that my parent can be unblocked) */
         MPL_atomic_release_store_uint64(my_gather_flag_addr, per_call_data->seq_no);
@@ -243,7 +243,6 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_NB_RG_reduce_start_sendrecv_completion(
     MPIDI_POSIX_release_gather_comm_t *nb_release_gather_info_ptr;
     nb_release_gather_info_ptr = &MPIDI_POSIX_COMM(comm_ptr, nb_release_gather);
     int rank = MPIR_Comm_rank(comm_ptr);
-    MPIR_Errflag_t errflag = MPIR_ERR_NONE;
 
     /* If root is not rank 0, rank 0 sends data to root */
     if (root != 0) {
@@ -253,7 +252,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_NB_RG_reduce_start_sendrecv_completion(
         } else if (rank == 0) {
             MPIC_Isend(MPIDI_POSIX_RELEASE_GATHER_NB_REDUCE_DATA_ADDR(rank, segment),
                        per_call_data->count, per_call_data->datatype, per_call_data->root,
-                       per_call_data->tag, comm_ptr, &(per_call_data->sreq), &errflag);
+                       per_call_data->tag, comm_ptr, &(per_call_data->sreq), 0);
         }
     }
 
@@ -314,7 +313,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_NB_RG_reduce_finish_sendrecv_completion
                 MPIR_Request_free(per_call_data->sreq);
                 per_call_data->sreq = NULL;
                 /* Mark the buffer as available for next use */
-                MPL_atomic_release_store_uint64(my_gather_flag_addr, -1);
+                MPL_atomic_release_store_uint64(my_gather_flag_addr, (uint64_t) - 1);
                 /* Update the last seq no completed */
                 nb_release_gather_info_ptr->ireduce_last_seq_no_completed[segment] =
                     per_call_data->seq_no;
@@ -353,7 +352,7 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_nb_release_gather_ireduce_impl(void *se
 {
     MPIR_FUNC_ENTER;
 
-    int mpi_errno = MPI_SUCCESS, mpi_errno_ret = MPI_SUCCESS;
+    int mpi_errno = MPI_SUCCESS;
     int tag;
     int first_vtx_id = -1, second_vtx_id, third_vtx_id, fourth_vtx_id, fifth_vtx_id, sixth_vtx_id,
         seventh_vtx_id, prev_vtx_id;
@@ -447,48 +446,33 @@ MPL_STATIC_INLINE_PREFIX int MPIDI_POSIX_nb_release_gather_ireduce_impl(void *se
             n_incoming = 1;
             prev_vtx_id = first_vtx_id;
         }
-        mpi_errno_ret =
-            MPIR_TSP_sched_generic(reserve_buf_type_id, data, sched, n_incoming, &prev_vtx_id,
-                                   &first_vtx_id);
-        if (mpi_errno_ret)
-            MPIR_ERR_ADD(mpi_errno, mpi_errno_ret);
+        mpi_errno = MPIR_TSP_sched_generic(reserve_buf_type_id, data, sched, n_incoming,
+                                           &prev_vtx_id, &first_vtx_id);
+        MPIR_ERR_CHECK(mpi_errno);
 
-        mpi_errno_ret =
-            MPIR_TSP_sched_generic(propagate_release_flag_type_id, data, sched, 1, &first_vtx_id,
-                                   &second_vtx_id);
-        if (mpi_errno_ret)
-            MPIR_ERR_ADD(mpi_errno, mpi_errno_ret);
+        mpi_errno = MPIR_TSP_sched_generic(propagate_release_flag_type_id, data, sched, 1,
+                                           &first_vtx_id, &second_vtx_id);
+        MPIR_ERR_CHECK(mpi_errno);
 
-        mpi_errno_ret =
-            MPIR_TSP_sched_cb(&MPIDI_POSIX_NB_RG_all_datacopy_cb, data, sched, 1, &second_vtx_id,
-                              &third_vtx_id);
+        mpi_errno = MPIR_TSP_sched_cb(&MPIDI_POSIX_NB_RG_all_datacopy_cb, data, sched, 1,
+                                      &second_vtx_id, &third_vtx_id);
+        MPIR_ERR_CHECK(mpi_errno);
 
-        if (mpi_errno_ret)
-            MPIR_ERR_ADD(mpi_errno, mpi_errno_ret);
+        mpi_errno = MPIR_TSP_sched_generic(gather_type_id, data, sched, 1,
+                                           &third_vtx_id, &fourth_vtx_id);
+        MPIR_ERR_CHECK(mpi_errno);
 
-        mpi_errno_ret =
-            MPIR_TSP_sched_generic(gather_type_id, data, sched, 1, &third_vtx_id, &fourth_vtx_id);
-        if (mpi_errno_ret)
-            MPIR_ERR_ADD(mpi_errno, mpi_errno_ret);
+        mpi_errno = MPIR_TSP_sched_cb(&MPIDI_POSIX_NB_RG_reduce_data_cb, data, sched, 1,
+                                      &fourth_vtx_id, &fifth_vtx_id);
+        MPIR_ERR_CHECK(mpi_errno);
 
-        mpi_errno_ret =
-            MPIR_TSP_sched_cb(&MPIDI_POSIX_NB_RG_reduce_data_cb, data, sched, 1, &fourth_vtx_id,
-                              &fifth_vtx_id);
+        mpi_errno = MPIR_TSP_sched_generic(start_sendrecv_type_id, data, sched, 1,
+                                           &fifth_vtx_id, &sixth_vtx_id);
+        MPIR_ERR_CHECK(mpi_errno);
 
-        if (mpi_errno_ret)
-            MPIR_ERR_ADD(mpi_errno, mpi_errno_ret);
-
-        mpi_errno_ret =
-            MPIR_TSP_sched_generic(start_sendrecv_type_id, data, sched, 1, &fifth_vtx_id,
-                                   &sixth_vtx_id);
-        if (mpi_errno_ret)
-            MPIR_ERR_ADD(mpi_errno, mpi_errno_ret);
-
-        mpi_errno_ret =
-            MPIR_TSP_sched_generic(finish_sendrecv_type_id, data, sched, 1, &sixth_vtx_id,
-                                   &seventh_vtx_id);
-        if (mpi_errno_ret)
-            MPIR_ERR_ADD(mpi_errno, mpi_errno_ret);
+        mpi_errno = MPIR_TSP_sched_generic(finish_sendrecv_type_id, data, sched, 1,
+                                           &sixth_vtx_id, &seventh_vtx_id);
+        MPIR_ERR_CHECK(mpi_errno);
 
         offset += chunk_count;
     }
