@@ -6,7 +6,6 @@
 #include <mpidimpl.h>
 #include "mpl_shm.h"
 #include "mpidu_init_shm.h"
-#include "mpidu_shm_seg.h"
 
 #include <stdlib.h>
 #ifdef HAVE_UNISTD_H
@@ -19,16 +18,27 @@
 #include <sys/shm.h>
 #endif
 
+extern int MPIDU_Init_shm_local_size;
+extern int MPIDU_Init_shm_local_rank;
+
+struct memory_seg {
+    size_t segment_len;
+    MPL_shm_hnd_t hnd;
+    char *base_addr;
+    bool symmetrical;
+    bool is_shm;
+};
+
 typedef struct memory_list {
     void *ptr;
-    MPIDU_shm_seg_t *memory;
+    struct memory_seg *memory;
     struct memory_list *next;
 } memory_list_t;
 
 static memory_list_t *memory_head = NULL;
 static memory_list_t *memory_tail = NULL;
 
-static int check_alloc(MPIDU_shm_seg_t * memory);
+static int check_alloc(struct memory_seg *memory);
 
 /* MPIDU_Init_shm_alloc(len, ptr_p)
 
@@ -39,18 +49,21 @@ int MPIDU_Init_shm_alloc(size_t len, void **ptr)
     int mpi_errno = MPI_SUCCESS, mpl_err = 0;
     void *current_addr;
     size_t segment_len = len;
-    int local_rank = MPIR_Process.local_rank;
-    int num_local = MPIR_Process.local_size;
-    MPIDU_shm_seg_t *memory = NULL;
+    struct memory_seg *memory = NULL;
     memory_list_t *memory_node = NULL;
-    MPIR_CHKPMEM_DECL(3);
+    MPIR_CHKPMEM_DECL();
 
     MPIR_FUNC_ENTER;
 
     MPIR_Assert(segment_len > 0);
 
-    MPIR_CHKPMEM_MALLOC(memory, MPIDU_shm_seg_t *, sizeof(*memory), mpi_errno, "memory_handle",
-                        MPL_MEM_OTHER);
+    if (MPIDU_Init_shm_local_size == 1) {
+        *ptr = MPL_aligned_alloc(MPL_CACHELINE_SIZE, len, MPL_MEM_SHM);
+        MPIR_ERR_CHKANDJUMP(!*ptr, mpi_errno, MPI_ERR_OTHER, "**nomem");
+        goto fn_exit;
+    }
+
+    MPIR_CHKPMEM_MALLOC(memory, sizeof(*memory), MPL_MEM_SHM);
 
     mpl_err = MPL_shm_hnd_init(&(memory->hnd));
     MPIR_ERR_CHKANDJUMP(mpl_err, mpi_errno, MPI_ERR_OTHER, "**alloc_shar_mem");
@@ -59,20 +72,9 @@ int MPIDU_Init_shm_alloc(size_t len, void **ptr)
 
     char *serialized_hnd = NULL;
     int serialized_hnd_size = 0;
-    /* if there is only one process on this processor, don't use shared memory */
-    if (num_local == 1) {
-        char *addr;
 
-        MPIR_CHKPMEM_MALLOC(addr, char *, segment_len + MPIDU_SHM_CACHE_LINE_LEN, mpi_errno,
-                            "segment", MPL_MEM_SHM);
-
-        memory->base_addr = addr;
-        current_addr =
-            (char *) (((uintptr_t) addr + (uintptr_t) MPIDU_SHM_CACHE_LINE_LEN - 1) &
-                      (~((uintptr_t) MPIDU_SHM_CACHE_LINE_LEN - 1)));
-        memory->symmetrical = 1;
-    } else {
-        if (local_rank == 0) {
+    {
+        if (MPIDU_Init_shm_local_rank == 0) {
             /* root prepare shm segment */
             mpl_err = MPL_shm_seg_create_and_attach(memory->hnd, memory->segment_len,
                                                     (void **) &(memory->base_addr), 0);
@@ -100,13 +102,14 @@ int MPIDU_Init_shm_alloc(size_t len, void **ptr)
 
         MPIDU_Init_shm_barrier();
 
-        if (local_rank == 0) {
+        if (MPIDU_Init_shm_local_rank == 0) {
             /* memory->hnd no longer needed */
             mpl_err = MPL_shm_seg_remove(memory->hnd);
             MPIR_ERR_CHKANDJUMP(mpl_err, mpi_errno, MPI_ERR_OTHER, "**remove_shar_mem");
         }
         current_addr = memory->base_addr;
-        memory->symmetrical = 0;
+        memory->symmetrical = false;
+        memory->is_shm = true;
 
         mpi_errno = check_alloc(memory);
         MPIR_ERR_CHECK(mpi_errno);
@@ -116,34 +119,157 @@ int MPIDU_Init_shm_alloc(size_t len, void **ptr)
 
     *ptr = current_addr;
 
-    MPIR_CHKPMEM_MALLOC(memory_node, memory_list_t *, sizeof(*memory_node), mpi_errno,
-                        "memory_node", MPL_MEM_OTHER);
+    MPIR_CHKPMEM_MALLOC(memory_node, sizeof(*memory_node), MPL_MEM_SHM);
     memory_node->ptr = *ptr;
     memory_node->memory = memory;
     LL_APPEND(memory_head, memory_tail, memory_node);
 
-    MPIR_CHKPMEM_COMMIT();
   fn_exit:
     MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
     /* --BEGIN ERROR HANDLING-- */
+    if (MPIDU_Init_shm_local_size > 1) {
+        MPL_shm_seg_remove(memory->hnd);
+        MPL_shm_hnd_finalize(&(memory->hnd));
+    }
+    MPIR_CHKPMEM_REAP();
+    goto fn_exit;
+    /* --END ERROR HANDLING-- */
+}
+
+int MPIDU_Init_shm_comm_alloc(MPIR_Comm * comm, size_t len, void **ptr)
+{
+    int mpi_errno = MPI_SUCCESS, mpl_err = 0;
+    void *current_addr;
+    size_t segment_len = len;
+    struct memory_seg *memory = NULL;
+    memory_list_t *memory_node = NULL;
+    MPIR_CHKPMEM_DECL();
+
+    MPIR_FUNC_ENTER;
+
+    if (MPIDU_Init_shm_local_size == 1) {
+        *ptr = MPL_aligned_alloc(MPL_CACHELINE_SIZE, len, MPL_MEM_SHM);
+        MPIR_ERR_CHKANDJUMP(!*ptr, mpi_errno, MPI_ERR_OTHER, "**nomem");
+        goto fn_exit;
+    }
+
+    MPIR_Comm *node_comm = comm->node_comm;
+    bool is_root;
+    if (node_comm) {
+        is_root = (node_comm->rank == 0);
+    } else {
+        is_root = true;
+    }
+
+    MPIR_Assert(segment_len > 0);
+    MPIR_CHKPMEM_MALLOC(memory, sizeof(*memory), MPL_MEM_OTHER);
+    mpl_err = MPL_shm_hnd_init(&(memory->hnd));
+    MPIR_ERR_CHKANDJUMP(mpl_err, mpi_errno, MPI_ERR_OTHER, "**alloc_shar_mem");
+
+    memory->segment_len = segment_len;
+
+    char *serialized_hnd = NULL;
+    int serialized_hnd_size = 0;
+    char serialized_hnd_buffer[MPIDU_INIT_SHM_BLOCK_SIZE];
+    bool need_attach;
+    bool need_remove;
+    if (is_root) {
+        if (!MPIDU_Init_shm_atomic_key_exist()) {
+            /* We need to create the shm segment */
+            mpl_err = MPL_shm_seg_create_and_attach(memory->hnd, memory->segment_len,
+                                                    (void **) &(memory->base_addr), 0);
+            MPIR_ERR_CHKANDJUMP(mpl_err, mpi_errno, MPI_ERR_OTHER, "**alloc_shar_mem");
+
+            mpl_err = MPL_shm_hnd_get_serialized_by_ref(memory->hnd, &serialized_hnd);
+            MPIR_ERR_CHKANDJUMP(mpl_err, mpi_errno, MPI_ERR_OTHER, "**alloc_shar_mem");
+            serialized_hnd_size = strlen(serialized_hnd) + 1;   /* add 1 for null char */
+
+            MPIDU_Init_shm_atomic_put(serialized_hnd, serialized_hnd_size);
+            need_attach = false;
+            need_remove = true;
+        } else {
+            /* Just retrieve the existing serialized handle */
+            MPIDU_Init_shm_atomic_get(serialized_hnd_buffer, MPIDU_INIT_SHM_BLOCK_SIZE);
+            serialized_hnd = serialized_hnd_buffer;
+            serialized_hnd_size = strlen(serialized_hnd) + 1;   /* add 1 for null char */
+            need_attach = true;
+            need_remove = false;
+        }
+        MPIR_Assert(serialized_hnd_size <= MPIDU_INIT_SHM_BLOCK_SIZE);
+        if (node_comm) {
+            mpi_errno = MPIR_Bcast_impl(serialized_hnd, MPIDU_INIT_SHM_BLOCK_SIZE,
+                                        MPIR_BYTE_INTERNAL, 0, node_comm, MPIR_COLL_ATTR_SYNC);
+            MPIR_ERR_CHECK(mpi_errno);
+        }
+    } else {
+        mpi_errno = MPIR_Bcast_impl(serialized_hnd_buffer, MPIDU_INIT_SHM_BLOCK_SIZE,
+                                    MPIR_BYTE_INTERNAL, 0, node_comm, MPIR_COLL_ATTR_SYNC);
+        MPIR_ERR_CHECK(mpi_errno);
+        serialized_hnd = serialized_hnd_buffer;
+        serialized_hnd_size = strlen(serialized_hnd) + 1;       /* add 1 for null char */
+        need_attach = true;
+        need_remove = false;
+    }
+    if (need_attach) {
+        mpl_err = MPL_shm_hnd_deserialize(memory->hnd, serialized_hnd, strlen(serialized_hnd));
+        MPIR_ERR_CHKANDJUMP(mpl_err, mpi_errno, MPI_ERR_OTHER, "**alloc_shar_mem");
+
+        mpl_err = MPL_shm_seg_attach(memory->hnd, memory->segment_len,
+                                     (void **) &memory->base_addr, 0);
+        MPIR_ERR_CHKANDJUMP(mpl_err, mpi_errno, MPI_ERR_OTHER, "**attach_shar_mem");
+    }
+
+    if (node_comm) {
+        mpi_errno = MPIR_Barrier_impl(node_comm, MPIR_COLL_ATTR_SYNC);
+        MPIR_ERR_CHECK(mpi_errno);
+    }
+    if (need_remove) {
+        /* memory->hnd no longer needed */
+        mpl_err = MPL_shm_seg_remove(memory->hnd);
+        MPIR_ERR_CHKANDJUMP(mpl_err, mpi_errno, MPI_ERR_OTHER, "**remove_shar_mem");
+    }
+
+    current_addr = memory->base_addr;
+    memory->symmetrical = false;
+    memory->is_shm = true;
+
+    mpi_errno = check_alloc(memory);
+    MPIR_ERR_CHECK(mpi_errno);
+
+    /* assign sections of the shared memory segment to their pointers */
+    *ptr = current_addr;
+
+    MPIR_CHKPMEM_MALLOC(memory_node, sizeof(*memory_node), MPL_MEM_OTHER);
+    memory_node->ptr = *ptr;
+    memory_node->memory = memory;
+    LL_APPEND(memory_head, memory_tail, memory_node);
+
+  fn_exit:
+    MPIR_FUNC_EXIT;
+    return mpi_errno;
+  fn_fail:
     MPL_shm_seg_remove(memory->hnd);
     MPL_shm_hnd_finalize(&(memory->hnd));
     MPIR_CHKPMEM_REAP();
     goto fn_exit;
-    /* --END ERROR HANDLING-- */
 }
 
 /* MPIDU_SHM_Seg_free() free the shared memory segment */
 int MPIDU_Init_shm_free(void *ptr)
 {
     int mpi_errno = MPI_SUCCESS, mpl_err = 0;
-    MPIDU_shm_seg_t *memory = NULL;
+    struct memory_seg *memory = NULL;
     memory_list_t *el = NULL;
 
     MPIR_FUNC_ENTER;
 
+    if (MPIDU_Init_shm_local_size == 1) {
+        MPL_free(ptr);
+        goto fn_exit;
+    }
+
     /* retrieve memory handle for baseaddr */
     LL_FOREACH(memory_head, el) {
         if (el->ptr == ptr) {
@@ -156,17 +282,14 @@ int MPIDU_Init_shm_free(void *ptr)
 
     MPIR_Assert(memory != NULL);
 
-    if (MPIR_Process.local_size == 1)
-        MPL_free(memory->base_addr);
-    else {
-        mpl_err = MPL_shm_seg_detach(memory->hnd, (void **) &(memory->base_addr),
-                                     memory->segment_len);
-        MPIR_ERR_CHKANDJUMP(mpl_err, mpi_errno, MPI_ERR_OTHER, "**detach_shar_mem");
-    }
+    mpl_err = MPL_shm_seg_detach(memory->hnd, (void **) &(memory->base_addr), memory->segment_len);
+    MPIR_ERR_CHKANDJUMP(mpl_err, mpi_errno, MPI_ERR_OTHER, "**detach_shar_mem");
 
   fn_exit:
-    MPL_shm_hnd_finalize(&(memory->hnd));
-    MPL_free(memory);
+    if (MPIDU_Init_shm_local_size > 1) {
+        MPL_shm_hnd_finalize(&(memory->hnd));
+        MPL_free(memory);
+    }
     MPIR_FUNC_EXIT;
     return mpi_errno;
   fn_fail:
@@ -178,6 +301,10 @@ int MPIDU_Init_shm_is_symm(void *ptr)
     int ret = -1;
     memory_list_t *el;
 
+    if (MPIDU_Init_shm_local_size == 1) {
+        return 1;
+    }
+
     /* retrieve memory handle for baseaddr */
     LL_FOREACH(memory_head, el) {
         if (el->ptr == ptr) {
@@ -192,7 +319,7 @@ int MPIDU_Init_shm_is_symm(void *ptr)
 /* check_alloc() checks to see whether the shared memory segment is
    allocated at the same virtual memory address at each process.
 */
-static int check_alloc(MPIDU_shm_seg_t * memory)
+static int check_alloc(struct memory_seg *memory)
 {
     int mpi_errno = MPI_SUCCESS;
     int is_sym;
@@ -200,7 +327,7 @@ static int check_alloc(MPIDU_shm_seg_t * memory)
 
     MPIR_FUNC_ENTER;
 
-    if (MPIR_Process.local_rank == 0) {
+    if (MPIDU_Init_shm_local_rank == 0) {
         MPIDU_Init_shm_put(memory->base_addr, sizeof(void *));
     }
 
@@ -225,9 +352,9 @@ static int check_alloc(MPIDU_shm_seg_t * memory)
     }
 
     if (is_sym) {
-        memory->symmetrical = 1;
+        memory->symmetrical = true;
     } else {
-        memory->symmetrical = 0;
+        memory->symmetrical = false;
     }
 
     MPIR_FUNC_EXIT;
