@@ -1,8 +1,8 @@
-/* -*- Mode: C; c-basic-offset:4 ; indent-tabs-mode:nil ; -*- */
 /*
- *  (C) 2001 by Argonne National Laboratory.
- *      See COPYRIGHT in top-level directory.
+ * Copyright (C) by Argonne National Laboratory
+ *     See COPYRIGHT in top-level directory
  */
+
 /*
  * Test of reduce scatter.
  *
@@ -12,21 +12,29 @@
  * Can be called with any number of processors.
  */
 
-#include "mpi.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include "mpicolltest.h"
 #include "mpitest.h"
+#include "mpicolltest.h"
 
-int main(int argc, char **argv)
+#ifdef MULTI_TESTS
+#define run coll_redscat
+int run(const char *arg);
+#endif
+
+int run(const char *arg)
 {
     int err = 0;
     int *sendbuf, recvbuf, *recvcounts;
     int size, rank, i, sumval;
     MPI_Comm comm;
 
+    int is_blocking = 1;
+
+    MTestArgList *head = MTestArgListCreate_arg(arg);
+    if (MTestArgListGetInt_with_default(head, "nonblocking", 0)) {
+        is_blocking = 0;
+    }
+    MTestArgListDestroy(head);
 
-    MTest_Init(&argc, &argv);
     comm = MPI_COMM_WORLD;
 
     MPI_Comm_size(comm, &size);
@@ -38,7 +46,7 @@ int main(int argc, char **argv)
     for (i = 0; i < size; i++)
         recvcounts[i] = 1;
 
-    MTest_Reduce_scatter(sendbuf, &recvbuf, recvcounts, MPI_INT, MPI_SUM, comm);
+    MTest_Reduce_scatter(is_blocking, sendbuf, &recvbuf, recvcounts, MPI_INT, MPI_SUM, comm);
 
     sumval = size * rank + ((size - 1) * size) / 2;
 /* recvbuf should be size * (rank + i) */
@@ -50,7 +58,6 @@ int main(int argc, char **argv)
 
     free(sendbuf);
     free(recvcounts);
-    MTest_Finalize(err);
 
-    return MTestReturnValue(err);
+    return err;
 }
