@@ -1,9 +1,9 @@
-! This file created from test/mpi/f77/spawn/spawnmult2f.f with f77tof90
-! -*- Mode: Fortran; -*- 
+! This file created from f77/spawn/spawnmult2f.f with f77tof90
 !
-!  (C) 2003 by Argonne National Laboratory.
-!      See COPYRIGHT in top-level directory.
+! Copyright (C) by Argonne National Laboratory
+!     See COPYRIGHT in top-level directory
 !
+
 ! This tests spawn_mult by using the same executable and no command-line
 ! options.  The attribute MPI_APPNUM is used to determine which
 ! executable is running.
@@ -23,10 +23,18 @@
        integer appnum
        logical flag
        integer ierr
+       integer can_spawn
+
        errs = 0
 
        call MTest_Init( ierr )
 
+       call MTestSpawnPossible( can_spawn, errs )
+        if ( can_spawn .eq. 0 ) then
+            call MTest_Finalize( errs )
+            goto 300
+        endif
+
        call MPI_Comm_get_parent( parentcomm, ierr )
 
        if (parentcomm .eq. MPI_COMM_NULL) then
@@ -51,7 +59,7 @@
         call MPI_Comm_rank( intercomm, rank, ierr )
 
         if (parentcomm .eq. MPI_COMM_NULL) then
-!           Master 
+!           Parent
             if (rsize .ne. np(1) + np(2)) then
                 errs = errs + 1
                 print *, "Did not create ", np(1)+np(2),                    &
@@ -110,7 +118,7 @@
              print *, "appnum was not set"
          endif
 
-!       Send the errs back to the master process 
+!       Send the errs back to the parent process 
         call MPI_Ssend( errs, 1, MPI_INTEGER, 0, 1, intercomm, ierr )
         endif
 
@@ -120,7 +128,9 @@
 !       Note that the MTest_Finalize get errs only over COMM_WORLD 
         if (parentcomm .eq. MPI_COMM_NULL) then
             call MTest_Finalize( errs )
+        else
+            call MPI_Finalize( ierr )
         endif
 
-        call MPI_Finalize( ierr )
+ 300    continue
         end
